#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: AstroImageGuider.tcl,v 1.25.1.6 2005/12/09 18:38:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# sskole  31/01/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#guidecam_block begin

itk::usual AstroImageGuider {}

class cat::AstroImageGuider {
    inherit itk::Widget

    # constructor

    constructor {args} {
	global ::env
	eval itk_initialize $args

	# set default temp image file
	if {"$itk_option(-tmpfile)" == ""} {
	    config -tmpfile "/tmp/im[pid].[incr count_]"
	}
	
	# open log file used to keep track of URLs (for debugging)
	set dir $env(HOME)/.skycat
	if {! [file isdirectory $dir]} {
	    catch {mkdir $dir}
	}
	catch {
	    set logfile_ [open $dir/log a+]
	    fcntl $logfile_ LINEBUF 1
	}

	# create a pipe to get feedback during image transfers
	lassign [pipe] rfd_ wfd_
	fileevent $rfd_ readable [code $this feedback]
	
	# do window layout
	layout_dialog

 	# create an object for running interruptable batch image requests
	Batch $this.batch -command [code $this request_done]

	set initialized_ 1
	config -image $itk_option(-image)
    }


    # destructor - delete C++ based objects so that the temp image
    # files are deleted

    destructor {
	catch {$this.im delete}
	catch {file delete $itk_option(-tmpfile)}
	catch {close $rfd_}
	catch {close $wfd_}
    }

    method get_state {} { return $state_}


    method set_imagesvr {name} {
	config -imagesvr $name
	$this.im open $name

	set image $itk_option(-image)
    }

    # do the dialog window layout
    
    method layout_dialog {} {
	# create astroimage object and setup feedback
	astroimage $this.im
	$this.im feedback $wfd_

	# set the image server (default if not set)
	if {"$itk_option(-imagesvr)" == ""} {
	    if {[catch {set imagesvr_list [$this.im info imagesvr]} msg]} {
		error_dialog $msg
		return
	    }
	    set itk_option(-imagesvr) [lindex $imagesvr_list [expr [llength $imagesvr_list]-1]]
	}
	set_imagesvr $itk_option(-imagesvr)
    }


    # set/reset widget states while busy 

    method set_state {state} {
	set state_ $state
	if {"$state" == "normal"} {
	    catch {blt::busy release $this.main}
	    catch {focus -lastfor $this.main}
	} else {
	    catch {focus .}
	    catch {blt::busy hold $this.main}
	}
	update idletasks
	$progressBar reset
    }

   
    # this method is called by the fileevent handler during the image transfer 
    # from the HTTP C++ class to give the user feedback about how much there is 
    # left to copy, etc...
    # Read a line from the feedback pipe, which should contain text to be displayed
    # in the progress widget.

    method feedback {} {
	set text [gets $rfd_]
	if {"$state_" != "normal"} {
	    if {[scan $text {total length: %d bytes} n] == 1} {
		$progressBar config -to $n
	    } elseif {[scan $text {read %d bytes} n] == 1} {
		$progressBar config -value $n
	    } elseif {[scan $text {url: %s} url] == 1} {
	    	catch {
		    puts $logfile_ "[fmtclock [getclock]]\n$itk_option(-imagesvr)\n$url\n\n"
		    flush $logfile_;}
	     }
	}
	$progressBar config -text $text
	update idletasks
    }
    

    # This method is called when the background request is done.
    # The arguments are the error status (0 is OK) and the result,
    # which is a list of {filename center-pos}

    method request_done {status result} {
	global gvar 

	if {$status} {
	    set_state normal
	    error_dialog $result
	    after 0 [list $progressBar config -text $result]
	} else {
	    lassign $result filename pos copyright
	    $progressBar config -text "Loading Image..."
	    update idletasks
	    
	    # load the image
	    $itk_option(-image) clear
	    if {[catch {$itk_option(-image) config -file $filename} msg]} {
		error_dialog $msg
		$itk_option(-image) clear
	    }
	    while { [$itk_option(-image) isclear] } {
                update
            }
            catch {file delete $filename}
	    if {[llength $pos] >= 2} {
		lassign $pos ra dec equinox
	    }
	    set_state normal
	    if [info exists gvar(armDisplay)] {$gvar(armDisplay) updateImageData}
	}
	if {"[set instance "::[AstroCat::get_current_instance]"]" != "::"} {
	    $instance set_from_image
	    $instance clear
	    $instance search
	}
    }
    
    method interrupt {} {
	$this.batch interrupt
	catch {file delete $itk_option(-tmpfile)}
	set_state normal
    }

    public proc new_imagesvr {image {imagesvr "dss@eso"}} {
	set i "$imagesvr"
	if ![info exists instances_($i)] {
	    return [set instances_($i) [AstroImageGuider .guider[incr n_instances_] -image $image -imagesvr $imagesvr]]
	}
	return $instances_($i)
    }
   
    # Get the requested image from the image server based on the given arguments
    # for the world coord position or name and the given width and height
    # if flag is 1, the entries are updated from the arguments.

    method getimage_from_args {ra dec equinox width height} {

	set cmd "$this.im getimage -tmpfile $itk_option(-tmpfile)"
	lappend cmd "-equinox" $equinox
	lappend cmd "-pos" [list $ra $dec]
	lappend cmd -width $width
	lappend cmd -height $height

	# start the request in the background
	set_state disabled
	$progressBar config -text "Attempting to contact image server..."	
	$progressBar look_busy

	$this.batch bg_eval [code [format {
	    list [%s] [%s.im centerpos] 
	} $cmd $this]]
    }
    

    # -- options --

    # default image server name 
    itk_option define -imagesvr imagesvr Imagesvr {}

    #  image handle (itcl class)
    itk_option define -image image Image {} {
	if {$initialized_} {
	    set image_ [$itk_option(-image) get_image]
	}
    }

    # temp image file to use
    itk_option define -tmpfile tmpfile Tmpfile {}

    
    # -- protected members --
    protected variable progressBar .guidecam1.progress

    # internal rtdimage widget for main image
    protected variable image_

    # pipe to read feedback during image transfer
    protected variable rfd_

    # pipe to write feedback during image transfer
    protected variable wfd_

    # current state: normal, disabled (i.e.: waiting)
    protected variable state_ {normal}

    # log file handle (used to log URLs)
    protected variable logfile_ stdout

    # flag: set at end of constructor
    protected variable initialized_ 0

    # -- common variables (shared by all instances) --
    
    # count used for filename generation
    common count_ 0

    common instances_

    common n_instances_ 0 
}

#guidecam_block end
