#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: armDisplay.tcl,v 1.25.1.6 2005/12/09 18:38:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# tcsmgr1 2002-02-03 added coude focus support
# sskole  31/01/00  created
# sskole  16/12/00  addColon: Checking position against empty string  

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#guidecam_block begin


class armDisplay {
    inherit itk::Archetype
    
    constructor {args} {}
    destructor  {} 

    # public methods and data
    public method updateGuideStars {}
    public method updateImageData {}
    public  method updateArmData {\
	    dlg2Update\
	    rot\
	    adap\
	    tt\
	    showArm\
	    showDet\
	    showAdap\
	    focus\
	    simul\
	    updateRate\
	    sRa\
	    sDec\
	    aRa\
	    aDec\
	    rRa\
	    rDec\
	    imgSvr\
	    loadImg\
	    vignetLim\
	    {altPos 0}\
	    {detecX 0}\
	    {detecY 0}\
	    {autoFocus 1}}

    public variable updateGraphicId

    # private methods and data
    # methods
    private method getImgObj        {}
    private method starInField      {starRa starDec} 
    private method removeColon      {position}
    private method addColon         {position}
    private method sendCmd          {envName procName cmd arguments {timeout 15000}} 
    private method probe2Star       {}
    private method deleteItems      {args}
    private method loadImage        {}
   
    # Drawing methods
    private method rThetaToWidgetXY {rad theta}
    private method armXYToWidgetXY  {adapPos ttPosRad x2 y2} 
    private method drawAdapRot      {}
    private method drawProbeBase    {{destinationPos 0}}
    private method drawProbePoly    {{destinationPos 0}}
    private method drawDetector     {}

    # Update methods
    private method updateInsData    {}
    private method updateAdConfig   {}
    private method updateGraphic    {}
    private method updateTTAngle    {}
    private method updateAdAngle    {}
    private method updateTelData    {}
    private method updateProbeData  {}

    # debug messages
    private method puts_debug       { str }

    # data
    private variable attrName    
    private variable tcsId	     0
    private variable ttCenterPos     0
    private variable scaleFactor     0
    private variable armShape    
    private variable focusId	     2 
    private variable simulation      1
    private variable imageSvr        ""
    private variable autoLoadImage   1
    private variable autoSetFocus    1

    # detector data
    private variable detFixed        0
    private variable detAngle		0
    # taken from the ws (simulation mode) or
    # from the lcu in normal operation
    private variable detX	     150 
    private variable detY	     150
    private variable detXoffs	     0 
    private variable detYoffs	     0

    #Object coordinates
    private variable objX            0
    private variable objY            0
    
    #Guide star coordinates
    private variable starX           0
    private variable starY           0
    
    #Arm length in mm and in pixels
    private variable armLenmm	     0
    private variable armLen          0
    
    #Scale in mm for one arcsec on the sky
    private variable imageScale      0

    #Size of field in arcmin and field radius in pixels
    private variable fieldSizeArcMin 0
    private variable fieldRad        0
    private variable nonvigRadius    0 

    private variable adSetPos        0
    private variable ttSetPos        0
    private variable rotOnSkyPos     0
    private variable adPos           0
    private variable rotPos          0
    private variable adapterAbs      0
    private variable ttPos           0

    private variable updateRateMs    3000
    private variable dlg2Update      ""
    private variable showProbeArm    0
    private variable showDetector    0
    private variable showAdapter     0
    private variable pi              3.141592654
    private variable fromRad2Deg     0
    private variable fromDeg2Rad     0

    private variable oldRefTelRa     0
    private variable oldRefTelDec    0
    private variable refTelRa        0
    private variable refTelDec       0
    private variable actTelRa        0
    private variable actTelDec       0
    private variable probeRa         0
    private variable probeDec        0
    private variable starRa          0
    private variable starDec         0

    private variable tcsEnv
    private variable astroImage      ""
    private variable globalShow      0
    private variable timeDelay       0
    private variable startTime       0
    private variable vigLim          ""
    private variable arcsecPerPixel  1.9
    private variable imageHeight     1400
    private variable probeSimDataFlag 0
    private variable altProbePos     0
    
    private variable debug_flag      0

    itk_option define -image image Image {}

}; # end of class definition


body armDisplay::constructor {args} {
    global env gvar

    set gvar(armDisplay) $this
    set fromRad2Deg [expr 180 / $pi]
    set fromDeg2Rad [expr $pi / 180]
 
    eval itk_initialize $args 

    set itk_component(canvas) [$itk_option(-image) get_canvas]
    
    if [$itk_option(-image) isclear] {
	#defaults
	set starRa "0:00:00.0"
	set starDec "0:00:00.0"
    } else {
	#Put arm and telescope at the center of the image
	lassign [[getImgObj] wcscenter] starRa starDec
    }
    set actTelRa $starRa
    set actTelDec $starDec
    set refTelRa $starRa
    set refTelDec $starDec
    set oldRefTelRa $starRa
    set oldRefTelDec $starDec
    set probeRa $starRa
    set probeDec $starDec

    if [info exists env(TCS_ENVNAME)] {
	set tcsEnv $env(TCS_ENVNAME)
    } else {set tcsEnv $env(RTAPENV)}
    set attrName(insData)       "@$tcsEnv:Appl_data:TCS:msw:insData"
    set attrName(focus)         "@$tcsEnv:Appl_data:TCS:msw:foc.currentFocus"
    set attrName(telSubstate)   "@$tcsEnv:Appl_data:TCS:msw.substate"
    set attrName(trkSubstate)   "@$tcsEnv:Appl_data:TCS:trk.substate"
    set attrName(refTelRa)      "@$tcsEnv:Appl_data:TCS:trk:data:position:refPos.alpha"
    set attrName(refTelDec)     "@$tcsEnv:Appl_data:TCS:trk:data:position:refPos.delta"
    set attrName(actTelRa)      "@$tcsEnv:Appl_data:TCS:trk:data:position:actPos.RACurrent"
    set attrName(actTelDec)     "@$tcsEnv:Appl_data:TCS:trk:data:position:actPos.decCurrent"
    set attrName(altPos)        "@$tcsEnv:Appl_data:TCS:LCU:alt:trk.position"
    set attrName(parAngle)      "@$tcsEnv:Appl_data:TCS:LCU:alt:trk:intCoord.parallAngle"
    set attrName(probeRa)       "@$tcsEnv:Appl_data:TCS:tif:data.probeRA"
    set attrName(probeDec)      "@$tcsEnv:Appl_data:TCS:tif:data.probeDEC"
    set attrName(agSubstate)    "@$tcsEnv:Appl_data:TCS:agws.substate"
    set attrName(vigLim)        "@$tcsEnv:Appl_data:TCS:agws:control:searchCriteria.vigLim"
    set attrName(guideStarTbl)  "@$tcsEnv:Appl_data:TCS:agws:control:guideStarTable.tbl"
}

body armDisplay::destructor {} {
    
    after cancel $updateGraphicId
    deleteItems tagProbeBase tagProbePoly tagTargetSpot\
	    tagSetProbeBase tagSetProbePoly tagSetTargetSpot\
	    tagDetPoly tagDetArrow tagAdapter tagRotator
}

body armDisplay::getImgObj {} {return [$itk_option(-image) get_image]}

body armDisplay::addColon {position} {
    
    if {"$position" == ""} {return $position}

    set pos [format "%.2f" $position]
    set absolute [ctoken pos "."]
    if {! [ctype digit $absolute]} {
	set start 1
	set stop 7
	append result [string index $absolute 0]
	set absolute [format "%07d" $absolute]
    } else {
	set start 0
	set stop 6
	set absolute [format "%06d" $absolute]
    }
    for {set i $start} {$i < $stop} {incr i} {
	if {$i != $start && [expr ($i + $start) % 2] == 0} {
	    append result ":"
	} 
	append result [string index $absolute $i]
    }
    return "$result$pos"
}

body armDisplay::removeColon {position} {
    
    set pos $position
    set result ""
    while {[set token [ctoken pos ":"]] != ""} {
	append result $token
    }
    return $result
}

body armDisplay::starInField {starRa starDec} {

    if [catch {[getImgObj] convert coords $starRa $starDec wcs stX stY canvas} result] {
	puts "starInField: Cannot convert star position: $result"
	return 0
    }

    #A distance is returned in arcseconds
    if [catch {[getImgObj] wcsdist $objX $objY $stX $stY} starDist] {
	puts "starInField: Could not calculate star distance: $starDist"
	return 0
    }

    if {$starDist >= [expr 60 * $fieldSizeArcMin / 2]} {
	warning_dialog "\nThe selected star is outside the adapter field of view!"
	return 0
    } else {return 1}
}

body armDisplay::updateImageData {} {

    set wcs [[getImgObj] wcsset]
    set arcsecPerPix [lindex $wcs 2]
    if {"$arcsecPerPix" != ""} {
	set arcsecPerPixel $arcsecPerPix
    }
    set height [lindex $wcs 6]
    if {"$height" != ""} {
	set imageHeight $height
    }
    puts_debug "armDisplay::updateImageData():imageHeight $imageHeight arcsecPerPix $arcsecPerPix"

}

body armDisplay::updateArmData {\
	dlg\
	rot\
	adap\
	tt\
	showArm\
	showDet\
	showAdap\
	focus\
	simul\
	updateRate\
	sRa\
	sDec\
	aRa\
	aDec\
	rRa\
	rDec\
	imgSvr\
	loadImg\
	vignetLim\
	{altPos 0}\
	{detecX 0}\
	{detecY 0}\
	{autoFocus 1}} {

    puts_debug "armDisplay::updateArmData(...)"
    puts_debug "armDisplay::updateArmData():autoFocus $autoFocus focus $focus"

    set dlg2Update $dlg
    set showProbeArm $showArm
    set showDetector $showDet
    set showAdapter $showAdap
    set imageSvr $imgSvr
    set autoLoadImage $loadImg
    set vigLim $vignetLim
    set autoSetFocus $autoFocus

    if {$altProbePos != $altPos} {
	set altProbePos $altPos
	set probeSimDataFlag 1
    }

    if {$updateRate != ""} {
	set updateRateMs [expr $updateRate * 1000]
    }

    if {!$autoSetFocus} {
	set focusId $focus
	updateAdConfig
    }

    if {$simul} {
	if {$focusId != $focus} {
	    set focusId $focus
	    updateAdConfig
	}
	set detX $detecX
	set detY $detecY
    } 
    set simulation $simul

    # Telescope
    if {$aRa != ""} {
	set actTelRa $aRa
	if {$simulation} {set probeSimDataFlag 1}
    }
    if {$aDec != ""} {
	set actTelDec $aDec
	if {$simulation} {set probeSimDataFlag 1}
    }

    if {$rRa != ""} {
	set oldRefTelRa $refTelRa
	set refTelRa $rRa
    }
    if {$rDec != ""} {
	set oldRefTelDec $refTelDec
	set refTelDec $rDec
    }
    
    #Guide probe
    if {$rot != ""} {
	set rotOnSkyPos [expr $rot * $fromDeg2Rad]
	#In simulation mode the absolute rotator angle and the rotator
	#angle on sky are the same
	set rotPos $rotOnSkyPos
    }
    if {$adap != ""} {
	set adPos [expr $adap * $fromDeg2Rad]
	#In simulation mode the absolute adapter angle and the adapter
	#angle on sky are the same
	set adapterAbs $adPos
	set adSetPos $adPos
    }
    if {$tt != ""} {
	set ttPos $tt
	set ttSetPos $tt
    }

    if {("$sRa" != "" && "$sRa" != "$starRa") || \
	    ("$sDec" != "" && "$sDec" != "$starDec")} {
	set starRa $sRa
	set starDec $sDec
	if {$simulation} {
	    if ![starInField $sRa $sDec] {return}
	    set probeSimDataFlag 1
	    set probeRa $starRa
	    set probeDec $starDec
	} else {probe2Star}
    }

    after cancel $updateGraphicId
    updateGraphic
}

body armDisplay::deleteItems {args} {
   
    foreach item $args {
	set found [$itk_component(canvas) find withtag $item]
	if {$found != ""} { $itk_component(canvas) delete $item }
    }
}

body armDisplay::rThetaToWidgetXY {rad theta} {
    
    set x [expr $objX + $rad * cos($theta) * $scaleFactor]
    
    # For nasmyth or coude the y-axis is reverse
    if {!$simulation && ($focusId == 2 || $focusId == 3 || $focusId == 5)} {
	set y [expr $objY - $rad * sin($theta) * $scaleFactor]
    } else {
	set y [expr $objY + $rad * sin($theta) * $scaleFactor]
    }
    
    puts_debug "armDisplay::rThetaToWidgetXY($rad $theta -> $x $y)"

    return "$x $y"
}

body armDisplay::armXYToWidgetXY {adapPos ttPosRad x2 y2} {
    
    set alpha [expr $adapPos + $ttPosRad]
    set cosAlpha [expr cos($alpha)]
    set sinAlpha [expr sin($alpha)]
    
    # origin of arm-fixed system on center field system
    
    set xorig [expr $armLen * sin(-$adapPos)]
    set yorig [expr $armLen * cos(-$adapPos)]
    
    # new coordinates on adapter plane
    
    set x11 [expr $x2 * $cosAlpha - $y2 * $sinAlpha]
    set y11 [expr $x2 * $sinAlpha + $y2 * $cosAlpha]

    set x1 [expr $x11 + $xorig]
    set y1 [expr $y11 + $yorig]
    
    # canvas coordinates
    
    set x0 [expr $objX + $x1 * $scaleFactor]
    
    # For nasmyth or coude the y-axis is reverse
    if {!$simulation && ($focusId == 2 || $focusId == 3 || $focusId == 5)} {
	set y0 [expr $objY - $y1 * $scaleFactor]
    } else {
	set y0 [expr $objY + $y1 * $scaleFactor]
    }

    return "$x0 $y0"
}

body armDisplay::drawProbeBase {{destinationPos 0}} {
    
    puts_debug "armDisplay::drawProbeBase($destinationPos armLen $armLen)"

    if {$showProbeArm == 0} {
	deleteItems tagProbeBase tagSetProbeBase
	return
    }

    if {$destinationPos} {
	set adapPos $adSetPos
	set color white
	set tagProbe tagSetProbeBase
    } else {
	set adapPos $adPos
	set color yellow
	set tagProbe tagProbeBase
    }
    deleteItems $tagProbe

    set tmp [rThetaToWidgetXY $armLen [expr ($pi / 2) + $adapPos]]
    set x0 [lindex $tmp 0]
    set y0 [lindex $tmp 1]

    # hardcoded radius
    set radius [expr 137.0 * $scaleFactor]
    
    $itk_component(canvas) create oval \
	    [expr $x0 - $radius] [expr $y0 - $radius] \
	    [expr $x0 + $radius] [expr $y0 + $radius] \
	    -tags $tagProbe -outline $color -fill ""
}

body armDisplay::drawProbePoly {{destinationPos 0}} {
    

    puts_debug "armDisplay::drawProbePoly($destinationPos)"

    if {$showProbeArm == 0} {
	deleteItems tagSetProbePoly tagSetTargetSpot tagProbePoly tagTargetSpot
	return
    }

    if {$destinationPos} {
	set ttPosRad [expr $ttSetPos * $fromDeg2Rad]
	set adapPos $adSetPos
	set color white
	set tagProbe tagSetProbePoly
	set tagSpot tagSetTargetSpot
    } else {
	set ttPosRad [expr $ttPos * $fromDeg2Rad]
	set adapPos $adPos
	set color yellow
	set tagProbe tagProbePoly
	set tagSpot tagTargetSpot
    }
    deleteItems $tagProbe $tagSpot

    set tmp1 [armXYToWidgetXY $adapPos $ttPosRad \
	    [lindex $armShape(0) 0]  -[lindex $armShape(0) 1]]
    set tmp2 [armXYToWidgetXY $adapPos $ttPosRad \
	    [lindex $armShape(1) 0]  -[lindex $armShape(1) 1]]
    set tmp3 [armXYToWidgetXY $adapPos $ttPosRad \
	    [lindex $armShape(2) 0]  -[lindex $armShape(2) 1]]
    set tmp4 [armXYToWidgetXY $adapPos $ttPosRad \
	    [lindex $armShape(3) 0]  -[lindex $armShape(3) 1]]
    set tmp5 [armXYToWidgetXY $adapPos $ttPosRad \
	    -[lindex $armShape(3) 0]  -[lindex $armShape(3) 1]]
    set tmp6 [armXYToWidgetXY $adapPos $ttPosRad \
	    -[lindex $armShape(2) 0]  -[lindex $armShape(2) 1]]
    set tmp7 [armXYToWidgetXY $adapPos $ttPosRad \
	    -[lindex $armShape(1) 0]  -[lindex $armShape(1) 1]]
    set tmp8 [armXYToWidgetXY $adapPos $ttPosRad \
	    -[lindex $armShape(0) 0]  -[lindex $armShape(0) 1]]

    $itk_component(canvas) create line \
	    [lindex $tmp1 0] [lindex $tmp1 1] \
	    [lindex $tmp2 0] [lindex $tmp2 1] \
	    [lindex $tmp3 0] [lindex $tmp3 1] \
	    [lindex $tmp4 0] [lindex $tmp4 1] \
	    [lindex $tmp5 0] [lindex $tmp5 1] \
	    [lindex $tmp6 0] [lindex $tmp6 1] \
	    [lindex $tmp7 0] [lindex $tmp7 1] \
	    [lindex $tmp8 0] [lindex $tmp8 1] \
	    -tag $tagProbe -width 1 -fill $color

    # draw the target view spot
    set tmp [armXYToWidgetXY $adapPos $ttPosRad 0.0 -$armLen]
    set rad [expr $scaleFactor * 6]
    
    $itk_component(canvas) create oval \
	    [expr [lindex $tmp 0] - $rad] [expr [lindex $tmp 1] - $rad] \
	    [expr [lindex $tmp 0] + $rad] [expr [lindex $tmp 1] + $rad] \
	    -tags $tagSpot -fill "" -outline $color
}

body armDisplay::drawDetector {} {

    puts_debug "armDisplay::drawDetector()"
    
    deleteItems tagDetPoly tagDetArrow
    if {$showDetector == 0} {
	return
    }

    #Draw detector box
    set cosRot [expr cos($rotOnSkyPos)]
    set sinRot [expr sin($rotOnSkyPos)]
    set dx [expr ($detX / 2.0) / $arcsecPerPixel]
    set dy [expr ($detY / 2.0) / $arcsecPerPixel]
    set offsx [ expr $detXoffs / $arcsecPerPixel]
    set offsy [ expr $detYoffs / $arcsecPerPixel]
    set coordList {{$dx $dy} {-$dx $dy} {-$dx -$dy} {$dx -$dy}}

    foreach coord $coordList {
	set x [lindex $coord 0]
	set y [lindex $coord 1] 
	lappend ptList [expr $objX + ( ($x-$offsx) * $cosRot + ($y-$offsy) * $sinRot) * $scaleFactor]
	lappend ptList [expr $objY + ( ($x-$offsx) * $sinRot - ($y-$offsy) * $cosRot) * $scaleFactor]
    }
    
    $itk_component(canvas) create line \
	    [lindex $ptList 0] [lindex $ptList 1] \
	    [lindex $ptList 2] [lindex $ptList 3] \
	    [lindex $ptList 4] [lindex $ptList 5] \
	    [lindex $ptList 6] [lindex $ptList 7] \
	    [lindex $ptList 0] [lindex $ptList 1] \
	    -tags tagDetPoly -fill green

    # draw an arrow line in the directon of detector orientation
    # don't draw arrows if 'fixed' flag is set (as for M4)
    if {$detFixed == 0} {
	set ptList ""
	set coordList "{0.0 0.0} {0.0 $dy}"
	foreach coord $coordList {
	    set x [lindex $coord 0]
	    set y [lindex $coord 1]
	    lappend ptList [expr $objX + ( ($x-$offsx) * $cosRot + ($y-$offsy) * $sinRot) * $scaleFactor]
	    lappend ptList [expr $objY + ( ($x-$offsx) * $sinRot - ($y-$offsy) * $cosRot) * $scaleFactor]
	}

	$itk_component(canvas) create line \
		[lindex $ptList 0] [lindex $ptList 1] \
		[lindex $ptList 2] [lindex $ptList 3] \
		-tag tagDetArrow -width 1 -fill green -arrow last
    }
}

body armDisplay::drawAdapRot {} {
    
    puts_debug "armDisplay::drawAdapRot(armLen $armLen fieldRad $fieldRad)"

    deleteItems tagAdapter tagRotator
    if {$showAdapter == 0} {
	return
    }

    # Draw adapter
    set rad [expr $armLen * $scaleFactor]   
    set x1 [expr $objX - $rad]
    set y1 [expr $objY - $rad]
    set x2 [expr $objX + $rad]
    set y2 [expr $objY + $rad]
    $itk_component(canvas) create oval $x1 $y1 $x2 $y2\
	    -tag circle -outline blue -tags tagAdapter

    puts_debug "armDisplay::drawAdapRot():Circle($x1 $y1 $x2 $y2)"
    
    # Draw center field
    set rad [expr $fieldRad * $scaleFactor]
    set x1 [expr $objX - $rad]
    set y1 [expr $objY - $rad]
    set x2 [expr $objX + $rad]
    set y2 [expr $objY + $rad]
    $itk_component(canvas) create oval $x1 $y1 $x2 $y2\
	    -tag circle -outline red -tags tagRotator

    # draw image out of field text
    set y2 [expr $objY - $rad + 15]
    if {$globalShow == 0} {
	$itk_component(canvas) create text \
		$objX $y2 -fill red -anchor center -text "Out of Field" -tags tagRotator
    }
    # Draw vignetting
    if { $nonvigRadius > 0 } {
	set vignRad [expr ($nonvigRadius * 60) / $arcsecPerPixel]
	set rad [expr $vignRad * $scaleFactor]
	set x1 [expr $objX - $rad]
	set y1 [expr $objY - $rad]
	set x2 [expr $objX + $rad]
	set y2 [expr $objY + $rad]
	$itk_component(canvas) create oval $x1 $y1 $x2 $y2\
		-tag circle -outline OliveDrab -tags tagRotator
    }


}

body armDisplay::updateAdConfig {} {
    
    #Data used for nasmyth foci
    set imageScale 0.582
    set armLenmm 560.0
#    set fieldSizeArcMin 30
    set fieldSizeArcMin 27.01
    set nonvigRadius 11
    set armShape(0) {137.0 0.0}
    set armShape(1) {64.0 314.0}
    set armShape(2) {37.0 496.0}
    set armShape(3) {37.0 594}
    switch $focusId {
	2 {
	    # Nasmyth A
	    set adx ada
	}
	3 {
	    # Nasmyth B
	    set adx adb
	}
	4 {
	    # Cassegrain
	    set adx adc
	    set imageScale 0.528
	    set armLenmm 370.0
	    set fieldSizeArcMin 15
	    # non extra vignetting zone defined for cass
	    set nonvigRadius 0

	    set armShape(0) {137.0 0.0}
            set armShape(1) {64.0 125.0}
            set armShape(2) {37.0 308.0}
            set armShape(3) {37.0 450.0}
	}
	5 {
	    # Coude
	    set adx ada
	}
    }

    # set symbolic address of attributes at lcu (does not apply for
    # default focus, none).
    if {! $simulation} {	
	
	set attrName(adSetPos)	  @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adapter:POSLOOP.posSetPoint
	set attrName(ttSetPos)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adttMotor.nextPosUser
	set attrName(rotPos)      @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:rot:POSLOOP.pos
	set attrName(adPos)       @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adapter:POSLOOP.pos
	set attrName(ttPos)       @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adttMotor.posUser
	set attrName(ttCenterPos) @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adttMotor.centerPosition   
	set attrName(armLenmm)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adapter.sensorArmLength
	set attrName(imageScale)  @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adapter.imageScale
	set attrName(fieldSize)   @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:adapter.fieldSize

	# new det data
	set attrName(detFixed)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetFixed
	set attrName(detAngle)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetAngle
	set attrName(detSizeX)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetSizeX
	set attrName(detSizeY)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetSizeY
	set attrName(detOffsX)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetOffsX
	set attrName(detOffsY)    @$tcsEnv:Appl_data:TCS:LCU:$adx:adrot:admain.vfDetOffsY

	# get actual values
	if [catch {seq_dbReadSymbolic $attrName(armLenmm)} result] {
	    puts $result
	} else {set armLenmm $result}
	
	if [catch {seq_dbReadSymbolic $attrName(imageScale)} result] {
	    puts $result
	} else {set imageScale $result}

	if [catch {seq_dbReadSymbolic $attrName(fieldSize)} result] {
	    puts $result
	} else {set fieldSizeArcMin $result}

	puts_debug "armDisplay::updateAdConfig(): $adx armLenmm $armLenmm fieldSize $fieldSizeArcMin"

    }
    
    # Apparently the cassegrain arm length in the DB is not accurate
#    if {$focusId == 4} {
#	set armLenmm [expr $armLenmm - 25]
#    }
}

body armDisplay::updateInsData {} {
    global gvar 

    if { !$simulation } {
	set insAttr $attrName(insData).selectedIns
	if [catch {set instrument [seq_dbReadSymbolic $insAttr]} ret] {
	    puts $ret
	    return
	}	
	
	# in normal operation read from adaptor
	if {!$simulation} {
	    set detFixed [seq_dbReadSymbolic -noBadQLog $attrName(detFixed)]
	    set detAngle [seq_dbReadSymbolic -noBadQLog $attrName(detAngle)]
	    set detX     [expr [seq_dbReadSymbolic -noBadQLog $attrName(detSizeX)] / $imageScale]
	    set detY     [expr [seq_dbReadSymbolic -noBadQLog $attrName(detSizeY)] / $imageScale]
	    set detXoffs [expr [seq_dbReadSymbolic -noBadQLog $attrName(detOffsX)] / $imageScale]
	    set detYoffs [expr [seq_dbReadSymbolic -noBadQLog $attrName(detOffsY)] / $imageScale]
	} else {
	# othewise read from ws instrument setup
	    set detXAttr $attrName(insData).data($instrument,vigAreaSizeX)
	    if [catch {set detX [seq_dbReadSymbolic $detXAttr]} ret] {
		puts $ret
	    }
	    set detYAttr $attrName(insData).data($instrument,vigAreaSizeY)
	    if [catch {set detY [seq_dbReadSymbolic $detYAttr]} ret] {
		puts $ret
	    }
	    
	    set detXoffsAttr $attrName(insData).data($instrument,vigAreaCntX)
	    if [catch {set detXoffs [seq_dbReadSymbolic $detXoffsAttr]} ret] {
		puts $ret
	    }
	    set detYoffsAttr $attrName(insData).data($instrument,vigAreaCntY)
	    if [catch {set detYoffs [seq_dbReadSymbolic $detYoffsAttr]} ret] {
		puts $ret
	    }
	    set detFixed 0
	    set detAngle 0
	}
	puts_debug "armDisplay::updateInsData(): $instrument detX $detX detY $detY xOffs $detXoffs yOffs $detYoffs detFixed $detFixed detAngle $detAngle"
    }
}

body armDisplay::loadImage {} { 
    global gvar ::loadImage

    # The variable gvar(pipeStatus) is set when executing a command in the background (Batch.tcl)
    if [info exists gvar(pipeStatus)] {
	# Status = 1 means that an error occurred during image load and in that case
	# the automatic image load must be deactivated
	if {$gvar(pipeStatus) == 1} {
	    set ::loadImage 0
	    set autoLoadImage 0
	    if {"$astroImage" != ""} {
		set gvar(pipeStatus) 0
	    }
	}
    }
    
    # Skip new image load if already in progress...
    if {"$astroImage" != "" && "[$astroImage get_state]" != "normal"} {
	if {!$autoLoadImage} {
	    $astroImage interrupt
	}
	return
    }

    # If the telescope is not tracking (value = 104), it is drifting, which 
    # means it is loading new sky images continously, and that must be omitted.
    if { !$simulation } {
	if [catch {seq_dbReadSymbolic $attrName(trkSubstate)} trkSubstate] {
	    puts $trkSubstate
	} elseif {$trkSubstate != 104} {return}
    } 

    if {!$autoLoadImage} {
	return
    }

    if {! $simulation} {
	if [catch {set telSubstate [seq_dbReadSymbolic $attrName(telSubstate)]} ret] {
	    puts $ret
	    return
	} 
	if [catch {set refTelRa [seq_dbReadSymbolic $attrName(refTelRa)]} ret] {
	    puts $ret
	    return
	} else {set refTelRa [addColon $refTelRa]}
	if [catch {set refTelDec [seq_dbReadSymbolic $attrName(refTelDec)]} ret] {
	    puts $ret
	    return
	} else {set refTelDec [addColon $refTelDec]}

	#Presetting has value 29
	if {$telSubstate == 29} {
	    set telRa $refTelRa
	    set telDec $refTelDec
	} else {
	    set telRa $actTelRa
	    set telDec $actTelDec
	}
    } else {
	if {$refTelRa != $oldRefTelRa || $refTelDec != $oldRefTelDec} {
	    set oldRefTelRa $refTelRa
	    set oldRefTelDec $refTelDec
	    set actTelRa $refTelRa
	    set actTelDec $refTelDec
	}
	set telRa $actTelRa
	set telDec $actTelDec
    }

    set load 0
    #Conversion impossible means telRa and telDec are outside image range
    if [catch {[getImgObj] convert coords $telRa $telDec wcs x y image} result] {
	set load 1
    } elseif {[expr $x + $fieldRad] > [[getImgObj] width] || \
	    [expr $x - $fieldRad] < 0 || \
	    [expr $y + $fieldRad] > [[getImgObj] height] || \
	    [expr $y - $fieldRad] < 0} {
	set load 1
    }
    
    if { $load } {
	set astroImage [cat::AstroImageGuider::new_imagesvr $itk_option(-image) $imageSvr]
	$astroImage getimage_from_args $telRa $telDec J2000 40 40
    }
}

body armDisplay::updateProbeData {} {

    puts_debug "armDisplay::updateProbeData()"

    if {! $simulation} {
	#Update adapter/rotator values 
	foreach i {altPos parAngle ttCenterPos rotPos adPos ttPos adSetPos ttSetPos} {
	    if [catch {set $i [seq_dbReadSymbolic $attrName(${i})]} ret] {
		set $i 0
		puts $ret
	    } elseif {[set $i] == "?" || [set $i] == "-?"} {
		puts "WARNING: $i contains invalid value: [set $i]"
		set $i 0
	    }
	}
	# correct tt angle with centerPosition offset
	set ttPos [expr $ttPos - $ttCenterPos]
	set ttSetPos [expr $ttSetPos - $ttCenterPos]

	# if no image set altPos and parAngle to 0
	if {$globalShow == 0} {
	    set altPos [expr $pi/2]
	    set parAngle 0
	}

	#Adjust adapter angle according to absolute rotator angle (alt +/- parallAngle)
	switch $focusId {
	    2 { #NA
		set fieldRotation [expr ($pi - $parAngle) + $altPos ]
		set rotOnSkySign -1
	    }
	    3 { #NB
		set fieldRotation [expr ($pi - $parAngle) - $altPos ]
		set rotOnSkySign -1
	    }
	    4 {	#CASSE
		set fieldRotation [expr $parAngle - $pi ]
		set rotOnSkySign 1
	    }
	    5 { #Coude same as NA
		set fieldRotation [expr ($pi - $parAngle) + $altPos ]
		set rotOnSkySign -1
	    }
	}

	if [catch {set adPos [expr $adPos - $fieldRotation - $pi ] }] {
	    set adPos 0
	} 
	set adapterAbs $adPos

	if [catch {set adSetPos [expr $adSetPos - $fieldRotation + $pi ] }] {
	    set adSetPos 0
	}

	# calculate rotonsky using rot ada angles, correct using adPos
	set rotOnSkyPos [expr $rotOnSkySign * ($rotPos - $fieldRotation) + $detAngle]

	# Update probe position
	if [catch {set probeRa [seq_dbReadSymbolic $attrName(probeRa)]} ret] {
	    puts $ret
	} else {set probeRa [addColon $probeRa]}
	    
	if [catch {set probeDec [seq_dbReadSymbolic $attrName(probeDec)]} ret] {
	    puts $ret
	} else {set probeDec [addColon $probeDec]}

	puts_debug "armDisplay::updateProbeData(): parAngle $parAngle altPos $altPos"
	puts_debug "armDisplay::updateProbeData(): detAngle $detAngle rotOnSkyPos $rotOnSkyPos fieldRotation $fieldRotation"
	puts_debug "armDisplay::updateProbeData(): rotPos $rotPos adPos $adPos ttPos $ttPos"
    }
}    

body armDisplay::updateTelData {} {

   if {! $simulation} {
	#Update current telescope RA and Dec
	if [catch {set actTelRa [seq_dbReadSymbolic $attrName(actTelRa)]} ret] {
	    puts $ret
	} else {set actTelRa [addColon $actTelRa]}
	if [catch {set actTelDec [seq_dbReadSymbolic $attrName(actTelDec)]} ret] {
	    puts $ret
	} else {set actTelDec [addColon $actTelDec]}
    }

    #convert RA and Dec to canvas x,y
    if [catch {[getImgObj] convert coords $actTelRa $actTelDec wcs objX objY canvas} result] {
	puts_debug "armDisplay::updateTelData():Could not get image center pixel"

	set globalShow 0
	set objX [expr [ [getImgObj] dispwidth ] / 2] 
	set objY [expr [ [getImgObj] dispheight ] / 2]

 
    } else {
	set globalShow 1
    }
    puts_debug "armDisplay::updateTelData():actTelRa $actTelRa actTelDec $actTelDec"
    puts_debug "armDisplay::updateTelData():Image center pixel ($objX $objY)"
}

body armDisplay::updateTTAngle {} {
                   
#                  /\ 
#                 /--\   
#                / tt \ 
#           arm / angle\ arm
#              /        \
#             /          \
#            /            \
#    Object *--------------* Guide Star
#                radius
    
    if [catch {[getImgObj] convert coords $starRa $starDec wcs starX starY canvas} result] {
	puts "updateTTAngle: Could not convert: $result"
	return 0
    }

    #The distance between the object and the guide star is returned in arcseconds
    if [catch {[getImgObj] wcsdist $objX $objY $starX $starY} result] {
	puts "updateTTAngle: Could not calculate distance: $result"
	return 0
    }
    
    set radPix [expr $result / $arcsecPerPixel]
    set ttPos [expr 2 * asin($radPix / (2 * $armLen)) * $fromRad2Deg]

    if {$altProbePos} {	set ttPos -$ttPos }
    set ttSetPos $ttPos
}

body armDisplay::updateAdAngle {} {

# The drawings do not have the right proportions    


# CASE: deltaX < 0
# D = 90 + B
# B = atan (deltaY/deltaX) 
# G = 90 - (tt/2)
# A = Adapter Angle = -(180 - D - G) = atan(deltaY/deltaX) - tt/2 
#
#                     |      * guide star          - 
#                     |     /|                     |
#                     |    / |                     |
#                     | D /  |                     |     
#                     |--/   |                     |  deltaY
#                     | /| B | arm                 |
#              object |/||   |                     |
#      ---------------*-|----|--------             -
#                     |\| G  |
#                     | \    |                   
#                     |--\ tt|                   
#                     | A \--|
#                     |    \ |
#                     | arm \|
#                     |      O  
#                     
#
#                     |------| deltaX   



# CASE: deltaX > 0
# B = atan (deltaY/deltaX)                    
# G = 90 - tt/2 
# A = Adapter Angle = 90 + B + G = 180 + atan (deltaX/deltaY) - tt/2 
#
#                     |
#                     | 
#                 arm |
# guide star  *-------|-------O            -
#              \| G   | tt \ /             |
#               \     |     /              | 
#                \    |    /               |
#                 \   |   / arm            | deltaY
#                  \ G|  /                 |
#                  |\---/                  |
#                B | \|/                   |
#       --------------*-------------       - 
#                     | object
#                     |
#                     |
#                     |
#
#             |-------|
#               deltaX            


    set deltaY [expr $objY - $starY]
    set deltaX [expr $objX - $starX]

    if {$deltaX != 0} {
	set B [expr atan($deltaY / $deltaX)]
	set adPos [expr $B - (($ttPos / 2) * $fromDeg2Rad)]

	if {$deltaX > 0} {
	    set adPos [expr $adPos + $pi]
	} 
    } else {set adPos 0}

    if {$altProbePos} {	set adPos [expr $adPos - $pi] }

    set adapterAbs $adPos
    set adSetPos $adPos
}

body armDisplay::updateGuideStars {} {
    
    puts_debug "armDisplay::updateGuideStar()"

    deleteItems starMark

    if [catch {set guideStarTbl [seq_dbReadSymbolic $attrName(guideStarTbl)]} ret] {
	puts $ret
	return
    }
    foreach guideStar $guideStarTbl {
	set alpha [addColon [lindex $guideStar 0]]
	set delta [addColon [lindex $guideStar 1]]
	if [catch {[getImgObj] convert coords $alpha $delta wcs x y canvas} result] {
	    puts "updateGuideStars: Could not convert: $result"
	    continue
	}
	set rad [expr 12 * $scaleFactor]
	set x1 [expr $x - $rad]
	set y1 [expr $y - $rad]
	set x2 [expr $x + $rad]
	set y2 [expr $y + $rad]
	$itk_component(canvas) create oval $x1 $y1 $x2 $y2 \
		-width 2 -tag circle -outline blue -tags starMark	
    }
}

body armDisplay::updateGraphic {} {
    global gvar 

    puts_debug ""
    puts_debug "armDisplay::updateGraphic():Begin"

    if {! $simulation} {
	if {$autoSetFocus} {
	    if [catch {set focusId [seq_dbReadSymbolic $attrName(focus)]} ret] {
		puts $ret
		# default is nasmyth A
		set focusId 2
	    } 
	}
        updateAdConfig
        updateInsData
    }
 
    #Calculate arm length in pixels
    set armLen [expr ($armLenmm / $imageScale) / $arcsecPerPixel]
    
    #Calculate field radius in pixels
    set fieldRad [expr (($fieldSizeArcMin / 2) * 60) / $arcsecPerPixel]

    #Get the zoom  / scale factor
    lassign [[getImgObj] scale] xs ys
    if {"$ys" != ""} {
	if {$xs > 0} {
	    set scaleFactor $xs
	} elseif {$xs < 0} {set scaleFactor [expr 1.0 / -$xs]}
    }

    updateProbeData
    updateTelData
    loadImage
    
    if {$dlg2Update != ""} {
	$dlg2Update updateFields\
		[expr $rotPos * $fromRad2Deg]\
		[expr $adapterAbs * $fromRad2Deg]\
		$ttPos\
		$probeRa\
		$probeDec\
		$actTelRa\
		$actTelDec\
		$refTelRa\
		$refTelDec\
		$focusId
    }

    drawAdapRot
    drawDetector
    #First draw the probe destination position...
    drawProbePoly 1 
    drawProbeBase 1

    #...and then the current position
    drawProbePoly 
    drawProbeBase 
    
    if {$globalShow == 1 && !$simulation} {
	updateGuideStars
    }

    # These calculations must be carried out simulation mode when the 
    # adapter and rotator graphics are already correctly in place.
    if {$probeSimDataFlag} {
	set probeSimDataFlag 0
	updateTTAngle
	updateAdAngle
    }
    set updateGraphicId [after $updateRateMs [code $this updateGraphic]]
    puts_debug "armDisplay::updateGraphic():End"

}

body armDisplay::sendCmd {envName procName cmd arguments {timeout 15000}} {

    if {[catch {seq_msgSendCommand -check $envName $procName $cmd $arguments} cmdId] } {
	seq_msgFlush all 3000
	puts "Send error: $cmdId"
	return 0
    } else {
	set errFlag 0
	if {[catch {seq_msgRecvReply -last -$timeout $cmdId errFlag} reply]} {
	    seq_msgFlush all 3000
	    set errFlag 1
	}
	if {$errFlag} {
	    puts "Reply error: $reply"
	    return 0
	}
    }
   return 1  
}

body armDisplay::probe2Star {} {
    global gvar

    if [catch {set agSubstate [seq_dbReadSymbolic $attrName(agSubstate)]} ret] {
	puts $ret
	return
    }

    #Stop autoguiding if active
    if {$agSubstate == 122} {
        if {[sendCmd $tcsEnv agwsControl STOPAG ""] == 0} {
	    seq_errDisplay
	    seq_errResetStack
	    return
        }
    } 
 
    if {"$vigLim" == ""} {
	if [catch {set vigLim [seq_dbReadSymbolic $attrName(vigLim)]} ret] {
	    puts $ret"
	    return
	}
    }
    if {"$vigLim" == "NONE"} {
	set vigParam "FALSE"
    } elseif {"$vigLim" == "INSTRUMENT"} {
	set vigParam "TRUE"
    }

    # type, alpha, delta, X, Y, checkVignettingPos 
    set buf "AD, [removeColon $starRa], [removeColon $starDec], , , $vigParam"
    if {[sendCmd $tcsEnv agwsControl PRABS $buf] == 0} {
	seq_errDisplay
	seq_errResetStack
    }
}


body armDisplay::puts_debug { str } {
    if {$debug_flag} {
	puts $str
    }
}


#guidecam_block end

# ___oOo___
