#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: armDisplayDlg.tcl,v 1.25.1.6 2005/12/09 18:38:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# tcsmgr1 2002-02-03 added coude focus support
# sskole  31/01/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#guidecam_block begin

itk::usual armDisplayDlg {}

class armDisplayDlg {
    inherit util::TopLevelWidget

    constructor {args} {}
    destructor {}

    public method prepareCatalog {}
    public method starClicked {}
    public method close {} {}
    public method apply {} {}
    public method load  {} {}
    public method createWidgets {w} {}
    public method create3Columns {widgetList leftF middleF rightF} {}
    public method create2Columns {widgetlist leftF middleF rightF} {}
    public method getInstance {}
    public method selectionOff {}
    public method selectionOn {}
    public method autoFocusSet {} 

    # updateFields is called by armDisplay to update label values
    public method updateFields {\
	    rotator\
	    adaptor\
	    ttable\
	    starRa\
	    starDec\
	    actTelRa\
	    actTelDec\
	    refTelRa\
	    refTelDec\
	    focusId} {}

    itk_option define -image image Image {}

    # fonts used
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

    # set the width for displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 22
    itk_option define -outputwidth outputWidth OutputWidth 11
    itk_option define -entrywidth entryWidth EntryWidth 12

    private common arm           ""
    private common showArmLast   1 
    private common showDetLast   1
    private common showAdapLast  1
    #initially in simulation mode
    private common simulation    0
    private common imageSvrLast  "dss@pl.eso"
    private common loadImageLast 1
    private common pickTypeLast  0
    private common instance      ""
    private variable starRa      ""
    private variable starDec     ""
    private variable probeRa     ""
    private variable probeDec    ""
    private variable origBindTags    ""
    private variable updateRate  3
}

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body armDisplayDlg::constructor { args } {
    global ::showArm ::showDet ::showAdap ::focus\
	    ::loadImage ::imageSvr ::env ::pickType ::autoFocus

    eval itk_initialize $args 
    
    wm title $w_ "Guide Probe Settings"
    
    # main controls frame
    pack [set mainf [frame $w_.mainf]] -side top -fill both \
	    -expand 1 -padx 1m -pady 1m -ipadx 1m -ipady 1m
    
    pack [set topf [frame $mainf.topf]] -side top -fill both -expand 1
    
    createWidgets $topf

    if {$arm == ""} {
	set arm [armDisplay $w_.armdisplay -image $itk_option(-image)]
    } 
    set showArm $showArmLast
    set showDet $showDetLast
    set showAdap $showAdapLast   
    set imageSvr $imageSvrLast
    set loadImage $loadImageLast
    set pickType $pickTypeLast
    set instance $this
    set focus 2
    set autoFocus 1

    apply
}

body armDisplayDlg::destructor {} {

}

body armDisplayDlg::selectionOn {} {

    set canvas [$itk_option(-image) get_canvas]
    set origBindTags [bindtags $canvas]
    set pickTag pick$w_
    bind $pickTag <ButtonRelease-1> [code $this starClicked]
    bindtags $canvas $pickTag
}

body armDisplayDlg::selectionOff {} {

    if {"$origBindTags" != ""} {
	bindtags [$itk_option(-image) get_canvas] $origBindTags
	set origBindTags ""
    }
}

proc armDisplayDlg::getInstance {} {return $instance}

body armDisplayDlg::starClicked {} {
    global objRa objDec

    set msg "Move to the selected guide star?"
    set result [tk_dialog .dialog "Guide Probe" $msg "" 0 Accept Cancel]
    if {$result == 0} {
	set starRa $objRa
	set starDec $objDec
	$itk_component(starRaValue) config -value $starRa
	$itk_component(starDecValue) config -value $starDec
	apply
    }
}

body armDisplayDlg::apply {} {
    global showArm showDet showAdap loadImage imageSvr vigLim pickType pickDlg focus autoFocus

    if {$autoFocus} {
	set setFocus ""
    } else {
	set setFocus $focus
    }

    $arm updateArmData\
	    $instance\
	    ""\
	    ""\
	    ""\
	    $showArm\
	    $showDet\
	    $showAdap\
	    $setFocus\
	    $simulation\
	    $updateRate\
	    $starRa\
	    $starDec\
	    ""\
	    ""\
	    ""\
	    ""\
	    $imageSvr\
	    $loadImage\
	    $vigLim\
	    0 0 0 \
	    $autoFocus

    set showArmLast $showArm
    set showDetLast $showDet
    set showAdapLast $showAdap
    set imageSvrLast $imageSvr
    set loadImageLast $loadImage

    if {$pickType == 0} {
	AstroCat::hideCatalogs
	if [info exists pickDlg] { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
    } elseif {$pickType == 1} {
	if [info exists pickDlg] { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
	after 2000 [code $this prepareCatalog]
	
    } elseif {$pickType == 2} {
	if {![info exists pickDlg]} {
	    AstroCat::hideCatalogs
	    selectionOff
	    set parentWin [winfo parent $w_]
	    set parentObj "::$parentWin"
	    guidecamPickObject $itk_option(-image) $parentWin $parentObj
	    while {1} {
		if [info exists pickDlg] {
		    $pickDlg pick_object
		} else {break}
	    }
	}
    }
}

body armDisplayDlg::prepareCatalog {} {
    
    set catalog [AstroCat::new_catalog {USNO at ESO} "::$itk_option(-image)" 0]
    update
    selectionOn
    if {[$catalog getListCount] == 0} {
	$catalog search
    }
}

body armDisplayDlg::close {} {

    wm withdraw $w_
}

body armDisplayDlg::load {} {
    global ::imageSvr
    
    set RA [ $itk_component(actTelRaValue) cget -value ]
    set DEC [ $itk_component(actTelDecValue) cget -value ]

    set astroImage [cat::AstroImageGuider::new_imagesvr $itk_option(-image) $imageSvr]
    $astroImage getimage_from_args  $RA $DEC J2000 40 40

}

body armDisplayDlg::create3Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	#Create  widgets
	set text [lindex $widget 0]
	set name [lindex $widget 1]

	itk_component add ${name}Label {
	    label $leftF.${name}Label \
		    -text $text\
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Entry {
	    entry $middleF.${name}Entry\
		    -textvariable ${name}Entry\
		    -width $itk_option(-entrywidth)\
		    -font $itk_option(-valuefont)
	}

	#bind
	bind $middleF.${name}Entry <KeyPress-Return> [code $this apply]

	itk_component add ${name}Value {
	    LabelValue $rightF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -valuewidth $itk_option(-outputwidth)\
		    -relief groove
	} 
	pack $itk_component(${name}Label) -side top -padx 1.0m -pady 1.5m -anchor w
	pack $itk_component(${name}Entry) -side top -padx 0.5m -pady 0.5m -anchor w
	pack $itk_component(${name}Value) -side top -padx 0.5m -pady 0.5m -anchor w
    }
}

body armDisplayDlg::create2Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	set text [lindex $widget 0]
	set name [lindex $widget 1]
	
	itk_component add ${name}Label {
	    label $leftF.${name}Label\
		    -text $text \
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Value {
	    LabelValue $middleF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -valuewidth $itk_option(-outputwidth)\
		    -relief groove
	} 
	itk_component add ${name}Dummy {
	    label $rightF.$name -text "" 
	}
	pack $itk_component(${name}Label) -side top -padx 0.5m -pady 1.5m -anchor w
	pack $itk_component(${name}Value) -side top -pady 0.5m -anchor w
	pack $itk_component(${name}Dummy) -side top -padx 1.0m -pady 1.5m -anchor w
    }
}

body armDisplayDlg::createWidgets {w} {
    global ::showArm ::showDet ::showAdap ::focus\
	    ::loadImage ::imageSvr ::vigLim ::pickType ::autoFocus ::radioPath
    
    pack [set topFrame [frame $w.topFrame -bd 2]] \
	    -side top -fill both -expand 0

    pack [set lowFrame [frame $w.lowFrame -bd 2]] \
	    -side top -fill both -expand 0

    #============================================================================
    #Image Server Frame
    #===========================================================================
    pack [set serverF [frame $topFrame.serverF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 
    pack [label $serverF.label -text "Image Server"] \
	    -side top -padx 1m -pady 2m
 
    pack [radiobutton $serverF.garching -text "Garching" -variable imageSvr -value "dss@eso"] \
	    -side top -padx 0.5m -pady 1.0m -anchor w
    pack [radiobutton $serverF.paranal -text "Paranal" -variable imageSvr -value "dss@pl.eso"] \
	    -side top -padx 0.5m -pady 1.0m -anchor w

    #============================================================================
    #Show State frame
    #==========================================================================
    pack [set visibleF [frame $topFrame.visibleF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 
    pack [label $visibleF.label -text "Show State"] \
	    -side top -padx 1m -pady 2m

    #Create visibility state radio buttons and check box
    pack [checkbutton $visibleF.loadImage -text "Automatic Image Load" -variable loadImage] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showArm	-text "Show Probe" -variable showArm] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showDet -text "Show Detector" -variable showDet] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showAdap -text "Show Fields" -variable showAdap] \
	    -side top -padx 0.5m -pady 0.5m -anchor w  
    
    #=============================================================================
    #Guide Star frame
    #============================================================================
    pack [set starF [frame $topFrame.starF -bd 2 -relief groove]] \
	   -side left -fill both -expand 1
    pack [label $starF.label -text "Guide Star"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $starF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $starF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $starF.rightF -bd 2]] \
	    -side left -fill both -expand 0

    pack [radiobutton $leftF.off -text "No Selection" -variable pickType -value 0] \
	    -side top -padx 0.5m -pady 1.0m -anchor w
    pack [radiobutton $leftF.catalog -text "Catalog Selection" -variable pickType -value 1] \
	    -side top -padx 0.5m -pady 1.0m -anchor w
    pack [radiobutton $leftF.fit -text "Fit Selection" -variable pickType -value 2] \
	    -side top -padx 0.5m -pady 1.0m -anchor w

    itk_component add starRaValue {
	LabelValue $middleF.starRaValue \
		-text "RA: " \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    itk_component add starDecValue {
	LabelValue $middleF.starDecValue \
		-text "Dec:" \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    pack $itk_component(starRaValue) -side top -padx 1m -pady 1.0m -anchor w
    pack $itk_component(starDecValue) -side top -padx 1m -pady 1.0m -anchor w

    #=============================================================================
    #Telescope Frame
    #============================================================================
    pack [set telF [frame $lowFrame.telF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1
    pack [label $telF.label -text "Telescope"] -side top -padx 1m -pady 2m
    pack [set leftF [frame $telF.leftF -bd 2]] -side left -fill both -expand 0
    pack [set middleF [frame $telF.middleF -bd 2]] -side left -fill both -expand 0
    pack [set rightF [frame $telF.rightF -bd 2]] -side left -fill both -expand 0

    #set radiopath for later use
    set radioPath $leftF

    # NaA Radio Button
    pack [radiobutton $leftF.nasa -text "Nasmyth A" -variable focus -value 2 -state disabled] \
	    -side top -padx 0.5m -pady 1m -anchor w

    # Auto set focus
    pack [checkbutton $middleF.autoFocus -text "Automatic Focus Set" -variable autoFocus \
	    -command [code $this autoFocusSet ] ] \
	    -side top -padx 0.5m -pady 0.5m -anchor w 

    # pack [label $middleF.dummy1 -text ""] -side top -pady 2m -anchor w
    pack [label $rightF.dummy1 -text "" ] -side top -pady 2m -anchor w 

    # NaB Radio Button
    pack [radiobutton $leftF.nasb -text "Nasmyth B" -variable focus -value 3 -state disabled] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy2 -text ""] -side top -pady 2m -anchor w
    pack [label $rightF.dummy2 -text ""] -side top -pady 2m -anchor w

    # Cas Radio Button
    pack [radiobutton $leftF.casse -text "Cassegrain" -variable focus -value 4 -state disabled] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy3 -text ""] -side top -pady 2m -anchor w
    pack [label $rightF.dummy3 -text ""] -side top -pady 2m -anchor w

    # Coude Radio Button
    pack [radiobutton $leftF.coude -text "Coude" -variable focus -value 5 -state disabled] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy4 -text ""] -side top -pady 1.5m -anchor w
    pack [label $rightF.dummy4 -text ""] -side top -pady 1.5m -anchor w

    #pack [label $leftF.dummy -text ""] -side top -pady 2m
    #pack [label $middleF.dummy4 -text ""] -side top -pady 2m

    # Create telescope fields
    set widgetList {{"Actual RA:" actTelRa} {"Actual Dec:" actTelDec} \
	    {"Reference RA:" refTelRa} {"Reference Dec:" refTelDec}}


    create2Columns $widgetList $leftF $middleF $rightF
   
    #=============================================================================
    #Guide Probe Frame
    #============================================================================
    pack [set probeF [frame $lowFrame.probeF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1
    pack [label $probeF.label -text "Guide Probe"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $probeF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $probeF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $probeF.rightF -bd 2]] \
	    -side left -fill both -expand 0

    # Create vignetting radio buttons
    pack [radiobutton $leftF.allowVign -text "Vign. Allowed" -variable vigLim -value "NONE"] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy1 -text ""] -side top -pady 2m -anchor w
    pack [label $rightF.dummy1 -text ""] -side top -pady 2m -anchor w

    pack [radiobutton $leftF.allowNoVign -text "Vign. not Allowed" -variable vigLim -value "INSTRUMENT"] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy2 -text ""] -side top -pady 2m -anchor w
    pack [label $rightF.dummy2 -text ""] -side top -pady 2m -anchor w

    pack [radiobutton $leftF.vignSetup -text "Vign. DB Setup" -variable vigLim -value ""] \
	    -side top -padx 0.5m -pady 1m -anchor w
    pack [label $middleF.dummy3 -text ""] -side top -pady 1.5m -anchor w
    pack [label $rightF.dummy3 -text ""] -side top -pady 1.5m -anchor w

    # Create status and entry fields
    set widgetList {{"RA:" probeRa} {"Dec:" probeDec}}
    create2Columns $widgetList $leftF $middleF $rightF
   
    set widgetList {{"Rotator (deg):" rot} {"Adapter (deg):" adap} \
	     {"Turn Table (deg):" ttable}}
    create2Columns $widgetList $leftF $middleF $rightF

    #======================================================================
    #Create buttons
    #=========================================================================
    itk_component add buttons {frame $w_.buttons -borderwidth 2 -relief groove}
    
    pack $itk_component(buttons) -side top -fill x 
    
    itk_component add close {
	button $w_.close -text "Close" -command [code $this close]
    }
    itk_component add load {
	button $w_.load -text "Load Image" -command [code $this load]
    }
    itk_component add apply {
	button $w_.apply -text "Apply" -command [code $this apply]
    }

    pack $itk_component(apply) $itk_component(load) $itk_component(close) \
	    -side left -expand 1 \
	    -padx 2m -pady 2m -ipadx 2m -ipady 1m -in $itk_component(buttons)
}

body armDisplayDlg::updateFields {\
	rotator\
	adapter\
	ttable\
	probeRa\
	probeDec\
	actTelRa\
	actTelDec\
	refTelRa\
	refTelDec\
        focusId} {

    global ::focus ::autoFocus

    #Guide probe
    $itk_component(probeRaValue) config -value $probeRa
    $itk_component(probeDecValue) config -value $probeDec
    
    $itk_component(rotValue) config -value [format "%.2f" $rotator]
    $itk_component(adapValue) config -value [format "%.2f" $adapter]
    $itk_component(ttableValue) config -value [format "%.2f" $ttable]

    #Telescope
    $itk_component(actTelRaValue) config -value $actTelRa
    $itk_component(actTelDecValue) config -value $actTelDec
    $itk_component(refTelRaValue) config -value $refTelRa
    $itk_component(refTelDecValue) config -value $refTelDec

    #Focus
    if { $autoFocus } {
	set focus $focusId
    }
}

body armDisplayDlg::autoFocusSet {} {
    global ::autoFocus ::radioPath
    
    if {$autoFocus} {
	set state disabled
    } else {
	set state normal
    }

    $radioPath.nasa configure -state $state
    $radioPath.nasb configure -state $state
    $radioPath.casse configure -state $state
    $radioPath.coude configure -state $state
}


#guidecam_block end

# ___oOo___
