#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: armDisplayDlgLite.tcl,v 1.25.1.6 2005/12/09 18:38:47 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# sskole  31/01/00  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#guidecam_block begin

itk::usual armDisplayDlgLite {}

class armDisplayDlgLite {
    inherit util::TopLevelWidget

    constructor {args} {}
    destructor {}
    
    public method prepareCatalog {}
    public method starClicked {}
    public method close {} {}
    public method apply {} {}
    public method showCoords {} {}
    public method createWidgets {w} {}
    public method create3Columns {widgetList leftF middleF rightF} {}
    public method create2Columns {widgetlist leftF middleF rightF} {}
    public method getInstance {}
    public method selectionOff {}
    public method selectionOn {}
    public method selectInstrumentMode {instrument mode}

    # updateFields is called by armDisplay to update label values
    public method updateFields {\
	    rotator\
	    adaptor\
	    ttable\
	    starRa\
	    starDec\
	    actTelRa\
	    actTelDec\
	    refTelRa\
	    refTelDec\
	    focusId} {}

    itk_option define -image image Image {}

    # fonts used
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*

    # set the width for displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 22
    itk_option define -outputwidth outputWidth OutputWidth 11
    itk_option define -entrywidth entryWidth EntryWidth 12

    private common arm            ""
    private common showArmLast    1 
    private common showDetLast    1
    private common showAdapLast   1
    private common focus          2
    private common simulation     1
    private common imageSvr       "dss@eso"
    private common loadImageLast  1
    private common pickTypeLast   1
    private common instance       ""
    private common instrumentLast "VISIR"
    private common modeLast       "Visir"
    private variable starRa       ""
    private variable starDec      ""
    private variable probeRa      ""
    private variable probeDec     ""
    private variable origBindTags ""
    private variable updateRate   1
    private variable vigLim       "NONE"
    private variable detX         0
    private variable detY         0
}

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body armDisplayDlgLite::constructor { args } {
    global gvar ::showArm ::showDet ::showAdap ::loadImage ::env ::pickType 

    eval itk_initialize $args 
    
    wm title $w_ "Guide Probe Settings"
    
    # main controls frame
    pack [set mainf [frame $w_.mainf]] -side top -fill both \
	    -expand 1 -padx 1m -pady 1m -ipadx 1m -ipady 1m
    
    pack [set topf [frame $mainf.topf]] -side top -fill both -expand 1
    
    createWidgets $topf

    if {$arm == ""} {
	set arm [armDisplay $w_.armdisplay -image $itk_option(-image)]
    } 
    set showArm $showArmLast
    set showDet $showDetLast
    set showAdap $showAdapLast   
    set loadImage $loadImageLast
    set pickType $pickTypeLast
    set instance $this
    
    selectInstrumentMode $instrumentLast $modeLast
#   apply
}

body armDisplayDlgLite::destructor {} {

}

body armDisplayDlgLite::selectionOn {} {
    global  objRa objDec

    set canvas [$itk_option(-image) get_canvas]
    set origBindTags [bindtags $canvas]
    set pickTag pick$w_

    bind $pickTag <1> "set objRa \"\" ; set objDec \"\" "
    bind $pickTag <ButtonRelease-1> " [code $this starClicked] "
    bindtags $canvas $pickTag
}

body armDisplayDlgLite::selectionOff {} {

    if {"$origBindTags" != ""} {
	bindtags [$itk_option(-image) get_canvas] $origBindTags
	set origBindTags ""
    }
}

proc armDisplayDlgLite::getInstance {} {return $instance}

body armDisplayDlgLite::starClicked {} {
    global objRa objDec

    if {"$objRa" != "" && "$objDec" != "" } {
	set msg "Move to the selected guide star?"
	set result [tk_dialog .dialog "Guide Probe" $msg "" 0 Accept Cancel]
	if {$result == 0} {
	    set starRa $objRa
	    set starDec $objDec
	    $itk_component(starRaValue) config -value $starRa
	    $itk_component(starDecValue) config -value $starDec
	    apply
	}
    }
}

body armDisplayDlgLite::apply {} {
    global rotEntry showArm showDet showAdap\
	    refTelRaEntry refTelDecEntry\
	    loadImage pickType altPos pickDlg

    if { $rotEntry != ""} {
	set actualRot [expr $rotEntry * -1.0]
    } else {
	set actualRot ""
    }
    
    $arm updateArmData\
	    $instance\
	    $actualRot\
	    ""\
	    ""\
	    $showArm\
	    $showDet\
	    $showAdap\
	    $focus\
	    $simulation\
	    $updateRate\
	    $starRa\
	    $starDec\
	    ""\
	    ""\
	    $refTelRaEntry\
	    $refTelDecEntry\
	    $imageSvr\
	    $loadImage\
	    $vigLim\
	    $altPos\
	    $detX\
	    $detY

    set rotEntry "" 
    set refTelRaEntry ""
    set refTelDecEntry ""

    set showArmLast $showArm
    set showDetLast $showDet
    set showAdapLast $showAdap
    set loadImageLast $loadImage

    if {$pickType == 0} {
	AstroCat::hideCatalogs
	if [info exists pickDlg] { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
    } elseif {$pickType == 1} {
	if [info exists pickDlg] { 
	    $pickDlg close 
	} else {
	    selectionOff
	}
	after 1000 [code $this prepareCatalog]
	
    } elseif {$pickType == 2} {
	if {![info exists pickDlg]} {
	    AstroCat::hideCatalogs
	    selectionOff
	    set parentWin [winfo parent $w_]
	    set parentObj "::$parentWin"
	    guidecamPickObject $itk_option(-image) $parentWin $parentObj
	    while {1} {
		if [info exists pickDlg] {
		    $pickDlg pick_object
		} else {break}
	    }
	}
    }
}

body armDisplayDlgLite::prepareCatalog {} {
    
    set catalog [AstroCat::new_catalog {USNO at ESO} "::$itk_option(-image)" 0]
    update
    selectionOn
    if {[$catalog getListCount] == 0} {
	$catalog search
    }
}

body armDisplayDlgLite::close {} {

    wm withdraw $w_
}

body armDisplayDlgLite::showCoords {} {
    global probeRaValue probeDecValue ttableValue

    if { $ttableValue == "" } {
	set pposStr ""
    } else {
	if { $ttableValue >= 0 } {
	    set pposStr "PPOS: POS"
	} else {
	    set pposStr "PPOS: NEG"
	}
    }

    set msg "RA: $probeRaValue DEC: $probeDecValue $pposStr"
    set result [tk_dialog .dialog "Guide Star Coords" $msg "" 0 Close]
}

body armDisplayDlgLite::create3Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	#Create  widgets
	set text [lindex $widget 0]
	set name [lindex $widget 1]

	itk_component add ${name}Label {
	    label $leftF.${name}Label \
		    -text $text\
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Entry {
	    entry $middleF.${name}Entry\
		    -textvariable ${name}Entry\
		    -width $itk_option(-entrywidth)\
		    -font $itk_option(-valuefont)
	}

	#bind
	bind $middleF.${name}Entry <KeyPress-Return> [code $this apply]

	itk_component add ${name}Value {
	    LabelValue $rightF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -textvariable ${name}Value\
		    -valuewidth $itk_option(-outputwidth)\
		    -relief groove
	} 
	pack $itk_component(${name}Label) -side top -padx 1.0m -pady 1.2m -anchor w
	pack $itk_component(${name}Entry) -side top -padx 0.5m -pady 0.5m -anchor w
	pack $itk_component(${name}Value) -side top -padx 0.5m -pady 0.5m -anchor w
    }
}

body armDisplayDlgLite::create2Columns {widgetList leftF middleF rightF} {

    foreach widget $widgetList {
	set text [lindex $widget 0]
	set name [lindex $widget 1]
	
	itk_component add ${name}Label {
	    label $leftF.${name}Label\
		    -text $text \
		    -font $itk_option(-labelfont)
	}
	itk_component add ${name}Value {
	    LabelValue $middleF.${name}Value \
		    -valuefont $itk_option(-valuefont)\
		    -valuewidth $itk_option(-outputwidth)\
		    -textvariable ${name}Value\
		    -relief groove
	} 
	itk_component add ${name}Dummy {
	    label $rightF.$name -text "" 
	}
	pack $itk_component(${name}Label) -side top -padx 0.4m -pady 1.2m -anchor w
	pack $itk_component(${name}Value) -side top -pady 0.5m -anchor w
	pack $itk_component(${name}Dummy) -side top -padx 1.0m -pady 1.2m -anchor w
    }
}

body armDisplayDlgLite::createWidgets {w} {
    global ::showArm ::showDet ::showAdap ::loadImage \
	    ::pickType ::altPosition ::altPos ::gvar ::insMode \
	    rotEntry refTelRaEntry refTelDecEntry \
	    rotValue refTelRaValue refTelDecValue
    
    pack [set topFrame [frame $w.topFrame -bd 2]] \
	    -side top -fill both -expand 0

    pack [set lowFrame [frame $w.lowFrame -bd 2]] \
	    -side top -fill both -expand 0

    #============================================================================
    #Show State frame
    #==========================================================================
    pack [set visibleF [frame $topFrame.visibleF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 
    pack [label $visibleF.label -text "Show State"] \
	    -side top -padx 1m -pady 2m

    #Create visibility state radio buttons and check box
    pack [checkbutton $visibleF.loadImage -text "Automatic Image Load" -variable loadImage] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showArm	-text "Show Probe" -variable showArm] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showDet -text "Show Detector" -variable showDet] \
	    -side top -padx 0.5m -pady 0.5m -anchor w
    pack [checkbutton $visibleF.showAdap -text "Show Fields" -variable showAdap] \
	    -side top -padx 0.5m -pady 0.5m -anchor w  

    #=============================================================================
    #Guide Star frame
    #============================================================================
    pack [set starF [frame $topFrame.starF -bd 2 -relief groove]] \
	   -side left -fill both -expand 1
    pack [label $starF.label -text "Guide Star"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $starF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $starF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $starF.rightF -bd 2]] \
	    -side left -fill both -expand 0

#    pack [radiobutton $leftF.off -text "No Selection" -variable pickType -value 0] \
#	    -side top -padx 0.5m -pady 1.0m -anchor w
    pack [radiobutton $leftF.catalog -text "Catalog Selection" -variable pickType -value 1] \
	    -side top -padx 0.5m -pady 1.0m -anchor w
#    pack [radiobutton $leftF.fit -text "Fit Selection" -variable pickType -value 2] \
#	    -side top -padx 0.5m -pady 1.0m -anchor w

    itk_component add starRaValue {
	LabelValue $middleF.starRaValue \
		-text "RA: " \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    itk_component add starDecValue {
	LabelValue $middleF.starDecValue \
		-text "Dec:" \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-outputwidth) \
		-relief groove
    }
    pack $itk_component(starRaValue) -side top -padx 1m -pady 1.0m -anchor w
    pack $itk_component(starDecValue) -side top -padx 1m -pady 1.0m -anchor w

    pack [checkbutton $middleF.altPosition -text "Alternate Position" -variable altPos] \
	    -side top -padx 1m -pady 2m -anchor w  

    #=============================================================================
    #Instrument Frame
    #============================================================================
    pack [set insF [frame $lowFrame.insF -bd 2 -relief groove]] \
	    -side top -fill both -expand 1
    pack [label $insF.label -text "Instrument"] \
	    -side top -padx 1m -pady 2m

    pack [menubutton $insF.selection -text "Instrument Selection" -menu $insF.selection.menu] \
	    -side left -padx 3m -pady 1m
    menu $insF.selection.menu
#    $insF.selection.menu add cascade -label ISAAC@NaB@UT1       -menu $insF.selection.menu.isaac
#    $insF.selection.menu add cascade -label FORS1@Cas@UT1       -menu $insF.selection.menu.fors1
#    $insF.selection.menu add cascade -label CRIRES@NaA@UT1      -menu $insF.selection.menu.crires
#    $insF.selection.menu add cascade -label FLAMES@NaA@UT2      -menu $insF.selection.menu.flames 
#    $insF.selection.menu add cascade -label FORS2@Cas@UT2       -menu $insF.selection.menu.fors2
#    $insF.selection.menu add cascade -label UVES@NaB@UT2        -menu $insF.selection.menu.uves 
#    $insF.selection.menu add cascade -label VISITOR@NaA@UT3     -menu $insF.selection.menu.visitor 
    $insF.selection.menu add cascade -label VISIR@Cas@@UT3      -menu $insF.selection.menu.visir 
#    $insF.selection.menu add cascade -label VIMOS@NaB@UT3       -menu $insF.selection.menu.vimos 
#    $insF.selection.menu add cascade -label NIRMOS@NaA@UT4      -menu $insF.selection.menu.nirmos 
#    $insF.selection.menu add cascade -label SINFONI@Cas@UT4     -menu $insF.selection.menu.sinfoni 
#    $insF.selection.menu add cascade -label NAOS-CONICA@NaB@UT4 -menu $insF.selection.menu.naos-conica

    #ISAAC menu
#    menu $insF.selection.menu.isaac
#    $insF.selection.menu.isaac add radiobutton \
#	    -label IsaacSw -command [code $this selectInstrumentMode ISAAC IsaacSw]
#    $insF.selection.menu.isaac add radiobutton \
#	    -label IsaacLw -command [code $this selectInstrumentMode ISAAC IsaacLw]
    
#    #FORS1 menu
#    menu $insF.selection.menu.fors1
#    $insF.selection.menu.fors1 add radiobutton \
#	    -label FORS1+fors -command [code $this selectInstrumentMode FORS1 FORS1+fors]

    #CRIRES menu
#    menu $insF.selection.menu.crires
#    $insF.selection.menu.crires add radiobutton \
#	    -label Crires -command [code $this selectInstrumentMode CRIRES Crires]

     #FLAMES menu
#    menu $insF.selection.menu.flames
#    $insF.selection.menu.flames add radiobutton \
#	    -label Flames -command [code $this selectInstrumentMode FLAMES Flames]
 
    #FORS2 menu
#    menu $insF.selection.menu.fors2
#    $insF.selection.menu.fors2 add radiobutton \
#	    -label FORS2+fors2 -command [code $this selectInstrumentMode FORS2 FORS2+fors2]

    #UVES menu
#    menu $insF.selection.menu.uves
#    $insF.selection.menu.uves add radiobutton \
#	    -label UVESslit -command [code $this selectInstrumentMode UVES UVESslit]
#    $insF.selection.menu.uves add radiobutton \
#	    -label UVESslicer -command [code $this selectInstrumentMode UVES UVESslicer]

#    #VISITOR menu
#    menu $insF.selection.menu.visitor
#    $insF.selection.menu.visitor add radiobutton \
#	    -label Visitor -command [code $this selectInstrumentMode VISITOR Visitor]

    #VISIR menu
    menu $insF.selection.menu.visir
    $insF.selection.menu.visir add radiobutton \
	    -label Visir -command [code $this selectInstrumentMode VISIR Visir]

    #VIMOS menu
#    menu $insF.selection.menu.vimos
#    $insF.selection.menu.vimos add radiobutton \
#	    -label Vimos -command [code $this selectInstrumentMode VIMOS Vimos]

    #NIRMOS menu
#    menu $insF.selection.menu.nirmos
#    $insF.selection.menu.nirmos add radiobutton \
#	    -label Nirmos -command [code $this selectInstrumentMode NIRMOS Nirmos]

    #SINFONI menu
#    menu $insF.selection.menu.sinfoni
#    $insF.selection.menu.sinfoni add radiobutton \
#	    -label Sinfoni -command [code $this selectInstrumentMode SINFONI Sinfoni]

    #NAOS-CONICA menu
#    menu $insF.selection.menu.naos-conica
#    $insF.selection.menu.naos-conica add radiobutton \
#	    -label Naos-Conica -command [code $this selectInstrumentMode NAOS-CONICA Naos-Conica]

    pack [label $insF.insSelectionLabel -text "Currently Selected:" \
	    -font $itk_option(-labelfont)] \
	    -side left -padx 3m -pady 1m

    itk_component add insSelectionValue {
	LabelValue $insF.insSelectionValue \
		-valuefont $itk_option(-valuefont)\
		-valuewidth $itk_option(-labelwidth)\
		-relief groove
    }
    pack $itk_component(insSelectionValue) -side left -padx 3m -pady 1m
    
    #=============================================================================
    #Telescope Frame
    #============================================================================
    pack [set telF [frame $lowFrame.telF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1
    pack [label $telF.label -text "Telescope"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $telF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $telF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $telF.rightF -bd 2]] \
	    -side left -fill both -expand 0
     
    # Create status and entry fields
    set widgetList {{"Preset RA:" refTelRa} {"Preset Dec:" refTelDec} \
	{"Rot on sky (deg):" rot} }
    create3Columns $widgetList $leftF $middleF $rightF
    pack [label $leftF.space -text ""] -side top -padx 1m -pady 2m -anchor w
    pack [label $middleF.space -text "nn:nn:nn.nnn"] -side top -padx 1m -pady 2m -anchor w
    pack [label $rightF.space -text "J2000"] -side top -padx 1m -pady 2m -anchor w
    #set widgetList {{"Offset RA:" actTelRa} {"Offset Dec:" actTelDec}}
    #create3Columns $widgetList $leftF $middleF $rightF

    #=============================================================================
    #Guide Probe Frame
    #============================================================================


    pack [set probeF [frame $lowFrame.probeF -bd 2 -relief groove]] \
	    -side left -fill both -expand 1 -anchor w
    pack [label $probeF.label -text "Guide Probe"] \
	    -side top -padx 1m -pady 2m
    pack [set leftF [frame $probeF.leftF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set middleF [frame $probeF.middleF -bd 2]] \
	    -side left -fill both -expand 0
    pack [set rightF [frame $probeF.rightF -bd 2]] \
	    -side left -fill both -expand 0

    # Create status and entry fields
    set widgetList {{"RA:" probeRa} {"Dec:" probeDec} \
	{"Adapter (deg):" adap} {"Turn Table (deg):" ttable} }
    create2Columns $widgetList $leftF $middleF $rightF
    
    #set widgetList {{"Rot on sky (deg):" rot}}
    #create3Columns $widgetList $leftF $middleF $rightF

    set rotEntry "90.0"  
    set refTelRaEntry "0:00:00.0"
    set refTelDecEntry "0:00:00.0"
    set rotValue "90.0"  
    set refTelRaValue "0:00:00.0"
    set refTelDecValue "0:00:00.0"


    #======================================================================
    #Create buttons
    #=========================================================================
    itk_component add buttons {frame $w_.buttons -borderwidth 2 -relief groove}
    
    pack $itk_component(buttons) -side top -fill x 
    
    itk_component add close {
	button $w_.close -text "Close" -command [code $this close]
    }
    itk_component add showCoords {
	button $w_.load -text "Get Guide Star Coords" -command [code $this showCoords]
    }
    itk_component add apply {
	button $w_.apply -text "Apply" -command [code $this apply]
    }
    
    pack $itk_component(apply) $itk_component(showCoords) $itk_component(close) \
	    -side left -expand 1 \
	    -padx 2m -pady 2m -ipadx 2m -ipady 1m -in $itk_component(buttons)
}

body armDisplayDlgLite::selectInstrumentMode {instrument mode} {
    global ::gvar

    
    switch $instrument {
	ISAAC {
	    #ISAAC@NaB@UT1
	    set focus 3
	    switch $mode {
		IsaacSw {
		    set detX 150
		    set detY 150
		}
		IsaacLw {
		    set detX 72
		    set detY 72
		}
	    }
	}
	FORS1 {
	    #FORS1@NaA@UT1
	    set focus 4
	    set detX 408
	    set detY 408
	}
	CRIRES {
	    #CRIRES@NaA@UT1
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	FLAMES {
	    #FLAMES@NaA@UT2
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	FORS2 {
	    #FORS2@Cas@UT2
	    set focus 4
	    set detX 120
	    set detY 408
	}
	UVES {
	    #UVES@NaB@UT2
	    set focus 3
	    switch $mode {
		UVESslit {
		    set detX 60
		    set detY 60
		}
		UVESslicer {
		    set detX 60
		    set detY 60
		}
	    }
	}
	VISITOR {
	    #VISITOR@NaA@UT3
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	VISIR {
	    #VISIR@Cas@UT3
	    set focus 4
	    set detX 60
	    set detY 60
	}
	VIMOS {
	    #VIMOS@NaB@UT3
	    set focus 3
	    set detX 960
	    set detY 1080
	}
	NIRMOS {
	    #NIRMOS@NaA@UT4
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	SINFONI {
	    #SINFONI@Cas@UT4
	    set focus 4
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
	NAOS-CONICA {
	    #NAOS-CONICA@NaB@UT4
	    set focus 2
	    warning_dialog "\nModes not yet implemented for instrument $instrument!!"
	}
    }

    $itk_component(insSelectionValue) config -value "$mode@$instrument"
    set $instrumentLast $instrument
    set $modeLast $mode
    apply
}

body armDisplayDlgLite::updateFields {\
	rotator\
	adapter\
	ttable\
	probeRa\
	probeDec\
	actTelRa\
	actTelDec\
	refTelRa\
	refTelDec\
        focusId} {

    #Guide probe
    $itk_component(probeRaValue) config -value $probeRa
    $itk_component(probeDecValue) config -value $probeDec
    $itk_component(rotValue) config -value [format "%.2f" [expr $rotator * -1.0] ]
    $itk_component(adapValue) config -value [format "%.2f" [expr $adapter * -1.0] ]
    $itk_component(ttableValue) config -value [format "%.2f" [expr $ttable * -1.0] ]

    #Telescope
    #$itk_component(actTelRaValue) config -value $actTelRa
    #$itk_component(actTelDecValue) config -value $actTelDec
    $itk_component(refTelRaValue) config -value $refTelRa
    $itk_component(refTelDecValue) config -value $refTelDec
}

#guidecam_block end

# ___oOo___
