#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: guidecam.tcl,v 1.25.1.6 2005/12/09 18:38:43 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  25/11/96  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

set skycat_usage ""

set about_skycat "\
guidecam version $par(RELEASE)
Copyright (C) 1996 ESO - European Southern Observatory 
"

itk::usual guidecam {}

# create a class for the application

class guidecam {
    inherit rtd::Rtd

    # constructor: create a toplevel window

    constructor {args} {
       eval itk_initialize $args 
	
	#guidecam_block begin
	# add a progress bar at the bottom
	ProgressBar $w_.progress
	pack $w_.progress -side bottom -fill x
	#guidecam_block end
    }

    # These options define which menus to activate ( by default all )

    itk_option define -rtd rtd Rtd 0 
    itk_option define -cat cat Cat 0    

    # url to use for the help menu - link to skycat WWW page
    # itk_option define -help_url help_url Help_url \
                        {http://arch-http.hq.eso.org/skycat}

    public method openTcsFile {} {

       if {[winfo exists $w_.pick]} {
          $w_.pick deleteTcsWindow
       }

       if {$mode == "Camera"} {
          $this detach_camera
          $image_ clear
       }
       set mode "file"
       $image_ open
    }


    #guidecam_block begin
    public method add_guideprobe_menu {}
    #guidecam_block end

    public method tcsAttachCamera {} {
       set mode "Camera"
       $this attach_camera
    }

    public method tcsDetachCamera {} {
       set mode ""
       $this detach_camera
    }

    public method getMode {} { return $mode }
 
    public method setXdefaults     {} 
    public method add_catalog_menu {} 
    public method add_help_menu    {}
    public method add_tcs_menu     {}
    public method clone            {}
    public method make_rtdimage    {}
    public method make_init_window {}
    public method init             {}
    public method feedback         {msg}

    protected variable percent_done_ 0
    protected variable mode          "open"
}

#guidecam_block begin
body guidecam::add_guideprobe_menu {} {

   set m [add_menubutton "Guide-Probe" left]

   add_menuitem $m command "Settings ..." \
	{Settings for the guide probe.} \
	-command "guideProbeSettings $image_ $w_"

   $m add separator
}
#guidecam_block end

#------------------------------------------------------
# METHOD : init
# This method is automatically called by TopLevelWidget class
#------------------------------------------------------

body guidecam::init {} {
    global par gvar

    Rtd::init

    # w_ is a protected variable of TopLevelWidget class

    wm title $w_ $par(TITLE)
    wm iconname $w_ guidecam
    wm geometry $w_ +450+10

    feedback "catalog and help menu..."
    if { $gvar(lite) } {
	pack forget $itk_component(menubar).real-time
    } else {
	add_catalog_menu
	add_tcs_menu
    }
    add_help_menu
    #guidecam_block begin
    add_guideprobe_menu

    #set w_.autoscale 1
    #$itk_component(image) autoscale $w_.autoscale

    #guidecam_block end



    if {[winfo exists $w_.init]} {
       destroy $w_.init ;# destroy init window

       update   ;# the logo uses up colors: the update forces 
                 # the destroy and frees the colors
       $itk_component(image) alloccolors 60
       wm deiconify $w_
    }
}

#---------------------------------------------------------------
# METHOD : setXdefaults
#
# Set default X resources for colors and fonts, and set some default key
# bindings. This method is called from the parent class and overridden 
# here.
# These are built-in defaults that the user can also override in 
# the ~/.Xdefaults file.
#---------------------------------------------------------------

body guidecam::setXdefaults {} {

   Rtd::setXdefaults
   cat::setXdefaults ;# initialize the catalog widgets

   # since we know this method gets called early, this is a good place to 
   # create window to display while starting up
   make_init_window
}

#---------------------------------------------------------------
# METHOD : add_catalog_menu
#---------------------------------------------------------------

body guidecam::add_catalog_menu {} {

   if (!$itk_option(-rtd)) {
      pack forget $itk_component(menubar).real-time
   }

   if ($itk_option(-cat)) {
      set m [add_menubutton "Data-Servers"]

      add_menuitem $m command "Catalogs..." \
          {Display the catalog window with the default catalog (GSC)} \
          -command [code $itk_component(image) catalogs]

      add_menuitem $m command "Image Servers..." \
         {Display the image server window with the default image server (DSS)}\
         -command [code $itk_component(image) image_servers]

      add_menuitem $m command "Archives..." \
          {Display the catalog window with the default archive} \
          -command [code $itk_component(image) archives]

      add_menuitem $m command "Local Catalog..." \
          {Open a local catalog from a tab table format file} \
          -command [code $itk_component(image) local_catalog]

      $m add separator

      add_menuitem $m command "Simbad..."  \
          {Display the catalog window for Simbad} \
          -command [code $itk_component(image) simbad]

      add_menuitem $m command "NED..." \
          {Display the catalog window for NED} \
          -command [code $itk_component(image) ned]

      add_short_help $itk_component(menubar).data-servers \
          "Data Servers menu: plot or display information from astronomical \
           catalogs or image servers"
   }
}

#---------------------------------------------------------------
# METHOD : add_help_menu 
#---------------------------------------------------------------

body guidecam::add_help_menu {} {

   set m [add_menubutton "Help" right]
   add_menuitem $m command "About ..." \
       {Display a window with information about this version} \
       -command [code $itk_component(image) about]

   add_menuitem $m command "Help..." \
       {Display information in netscape } \
       -command ""

    add_short_help $itk_component(menubar).help \
       {Help menu: display information about this application}
}




#---------------------------------------------------------------
# METHOD : add_tcs_menu
#---------------------------------------------------------------

body guidecam::add_tcs_menu {} {

   set m [add_menubutton "TCS" left]

   add_menuitem $m command "Open ..." \
	{Open and display a (FITS) image file} \
	-command [code $this openTcsFile]

   $m add separator

   add_menuitem $m command "Attach Camera" \
       {Attach the real-time camera - start receiving images} \
	    -command [code $this tcsAttachCamera]

   add_menuitem $m command "Detach Camera" \
	    {Detach the real-time camera - stop receiving images} \
	    -command [code $this tcsDetachCamera]

    add_menuitem $m command "Set Camera..." \
	    {Set the real-time camera name} \
	    -command [code $this set_camera]

   $m add separator
   $m add separator

   add_menuitem $m command "Pick Reference Star ..." \
       {Display a window to select the reference star} \
       -command "guidecamPickObject $image_ $w_ $this"

   $m add separator
   $m add separator

   add_menuitem $m command "Display Setup File (DEBUG)" {} \
       -command "displayAgSetupFile ccdAgRtdWindow.det"

   add_menuitem $m cascade "Debug Flags" {} \
	    -menu [set dbg_menu [menu $m.tcsDebug]]

   add_menuitem $dbg_menu checkbutton "Debug msgSendCommand" {} \
	    -variable seq_debug(seq_msgSendCommand) -onvalue 1 -offvalue 0

   add_menuitem $dbg_menu checkbutton "Debug msgRecvReply" {} \
	    -variable seq_debug(seq_msgRecvReply) -onvalue 1 -offvalue 0

   add_menuitem $dbg_menu checkbutton "Command Checking" {} \
            -variable tcs(checkCmd) -onvalue CHECK_ALL -offvalue NO_CHECK
}



#---------------------------------------------------------------
# METHOD : clone
# make a new main window (redefined from parent class)
#---------------------------------------------------------------

body guidecam::clone {} {
   global clone_cnt ::skycat_usage

   if {![info exists clone_cnt]} { set clone_cnt 0 }
   incr clone_cnt
   after 0 [code TopLevelWidget::start guidecam "" "" .guidecam$clone_cnt]
}

#---------------------------------------------------------------
# METHOD : make_rtdimage
# make a new main window (redefined from parent class)
# create the rtd image widget with catalog extensions
# (redefined from parent class to use class with catalog
# features added)
#---------------------------------------------------------------

body guidecam::make_rtdimage {} {
   set image_ $w_.image

   itk_component add image { 
       guidecamCtrl $image_ \
		-file $itk_option(-file) \
		-usexshm $itk_option(-usexshm) \
		-shm_header $itk_option(-shm_header) \
		-shm_data $itk_option(-shm_data) \
		-min_colors $itk_option(-min_colors) \
		-max_colors $itk_option(-max_colors) \
		-drag_scroll $itk_option(-drag_scroll) \
		-scrollbars $itk_option(-scrollbars) \
		-verbose $itk_option(-verbose) \
		-subsample $itk_option(-subsample) \
		-xscale -2 -yscale -2 \
		-use_zoom_view $itk_option(-use_zoom_view) \
		-with_zoom_window $itk_option(-with_zoom_window) \
		-with_pan_window $itk_option(-with_pan_window) \
		-zoom_factor $itk_option(-zoom_factor) \
		-colorramp_height $itk_option(-colorramp_height) \
		-default_cmap $itk_option(-default_cmap) \
		-default_itt $itk_option(-default_itt) \
		-with_colorramp $itk_option(-with_colorramp) \
		-rapid_frame_command [code $this rapid_frame_command] \
		-feedback [code $this feedback] \
		-port $itk_option(-port) \
		-shorthelpwin $this \
		-debug $itk_option(-debug)\
                -canvaswidth 520 -canvasheight 520
   }
   # keep a list of skycat instances
   global ::skycat_images
   lappend skycat_images $itk_component(image)

   # -xscale $itk_option(-xscale) \
   # -yscale $itk_option(-yscale) \
}

#---------------------------------------------------------------
# METHOD : make_init_window
# display a window while the application is starting up
#---------------------------------------------------------------
    
body guidecam::make_init_window {} {
   global ::skycat_logo ::about_skycat

#   if {! [info exists skycat_logo] || "$skycat_logo" == ""} {
#      puts "DBG> skycat_logo not exists .."
#      return
#   } 

   set w [TopLevelWidget $w_.init -center 1]
   wm overrideredirect $w 1
   wm withdraw $w_

#   label $w.logo -image $skycat_logo -borderwidth 2 -relief groove
   message $w.msg -text $about_skycat -width 6i -justify center \
                  -borderwidth 2 -relief groove
   ProgressBar $w.progress -from 0 -to 10 -value 0 -borderwidth 2 \
               -relief groove

   pack $w.msg $w.progress -side top -fill x -padx 1m -pady 2m

   tkwait visibility $w
}

#---------------------------------------------------------------
# METHOD : feedback
# this method is redefined here to get feedback during startup
#---------------------------------------------------------------

body guidecam::feedback {msg} {
   if {[winfo exists $w_.init.progress]} {
      $w_.init.progress config -text $msg -value [incr percent_done_]
      update idletasks
   }
}

#---------------------------------------------------------------
# OPTION : -rtd     
# Flag to display the real-time menu (VLT features)
#---------------------------------------------------------------  

configbody guidecam::rtd {
   if {$itk_option(-rtd) != 0 && $itk_option(-rtd) != 1} {
      set itk_option(-rtd) 1
      puts "The -rtd option requires a value of 1 (true) or 0 (false)"
   }
}

#---------------------------------------------------------------
# OPTION : -cat  
# Flag to display the catalog-menu
#---------------------------------------------------------------  

configbody guidecam::cat {
   if {$itk_option(-cat) != 0 && $itk_option(-cat) != 1} {
      set itk_option(-cat) 1
      puts "The -cat option requires a value of 1 (true) or 0 (false)"
   }
}
