#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: guidecamCtrl.tcl,v 1.25.1.6 2005/12/09 18:38:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  25/11/96  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual guidecamCtrl {}

# create a class for the application

class guidecamCtrl {
    inherit rtd::RtdImageCtrl

    # constructor

    constructor {args} {
	eval itk_initialize $args
    }

    itk_option define -debug debug Debug 0

    public method display_logo     {}
    public method about            {}
    public method send_to_netscape {url}
    public method list_windows   {}
    public method local_catalog  {}
    public method catalogs  {}
    public method simbad    {}
    public method ned       {}
    public method archives  {}
    public method image_servers {} 
    public method mark_image {ra dec width height}
#    public method pick_dialog {cmd}

    # Remove the given mark from the image

    public method unmark_image {id} { $canvas_ delete $id }
}


#------------------------------------------------------
# METHOD : display_logo
# 
# display the skycat logo in the center of the image window
# with some copywrite text
# Note that we assume here that the logo was created previously
# and the name of the image is in the global var skycat_logo.
#------------------------------------------------------

body guidecamCtrl::display_logo {} {
   global ::skycat_logo ::about_skycat

   if {! [info exists skycat_logo] || "skycat_logo" == ""} {
      return
   }
	
   # center logo
   $canvas_ config -scrollregion "0 0 1 1"
   $canvas_ create image 0 0 -image $skycat_logo \
	    -tags "objects logo logo_image" -anchor c

   # place text under logo
   lassign [$canvas_ bbox logo_image] x0 y0 x1 y1
   set y0 $y1
   set iw [expr $x1-$x0]
   set ih [expr $y1-$y0]
   set x [expr ($x0+$x1)/2]
   set y [expr $y1+10]

   $canvas_ create text $x $y -tags "objects logo logo_text" \
	    -justify center -anchor n -fill lightblue \
	    -font -Adobe-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-* \
	    -text $about_skycat
	
   # center the whole thing
   lassign [$canvas_ bbox logo] x0 y0 x1 y1
   set w [expr $x1-$x0]
   set h [expr $y1-$y0]
   $canvas_ move logo 0 [expr -($h/2-$ih/2)] 
	
   # clicking in it deletes it
   $canvas_ bind logo <1> "$canvas_ delete logo"
}

#------------------------------------------------------
# METHOD : about
#  
# Display a popup window with information about this application
#------------------------------------------------------

body guidecamCtrl::about {} {
   global ::skycat_logo ::about_skycat

   if {! [winfo exists $w_.about]} {
      if {! [info exists skycat_logo] || "skycat_logo" == ""} {
         set bitmap info
         set logo ""
      } else {
         set bitmap ""
         set logo $skycat_logo
      }

      DialogWidget $w_.about -image $logo -bitmap $bitmap \
		-messagewidth 6i -justify center -text $about_skycat
   }

   $w_.about activate
}

#------------------------------------------------------
# METHOD : send_to_netscape
#  
# Send a URL to be displayed by netscape
#------------------------------------------------------

body guidecamCtrl::send_to_netscape {url} {
   if {[catch {exec netscape -remote "openURL($url)"} msg1]} {
      if {! [string match $msg1 "netscape: warning"]} {
         if {[catch {exec netscape $url &} msg2]} {
            warning_dialog "couldn't start netscape: $msg1"
         }
      }
    }
}
    
#------------------------------------------------------
# METHOD : list_windows
#  
# Return a list of top level windows that this class might create
#------------------------------------------------------

body guidecamCtrl::list_windows {} {
   return "[RtdImageCtrl::list_windows] $w_.ai1 [AstroCat::list_windows]"
}


#------------------------------------------------------
# METHOD : local_catalog 
#  
# Pop up a file select dialog to access a local catalog
#------------------------------------------------------ 
   
body guidecamCtrl::local_catalog {} {

   set file [filename_dialog [pwd] * $w_]

   if {"$file" != ""} {
      if {[file isfile $file]} {
         if {! [info exists $w_.cat]} {
            astrocat $w_.cat
         }
         if {[catch {$w_.cat check $file} msg]} {
            error_dialog $msg
         } else {
            busy { AstroCat::new_catalog $file $this \
                $itk_option(-debug) [AstroCat::is_tcs_catalog $file] }
         }
      } else {
         error_dialog "There is no file named '$file'" $w_
      }
    }
}

#------------------------------------------------------
# METHOD : catalogs
#  
# Pop up a dialog for the astronomical catalogs (GSC, etc...)
#------------------------------------------------------     
    
body guidecamCtrl::catalogs {} {
   busy { AstroCat::new_catalog {Guide Star Catalog at ESO} \
          $this $itk_option(-debug) }
}

#------------------------------------------------------
# METHOD : simbad 
#  
# Pop up a dialog for the astronomical catalogs (GSC, etc...)
#------------------------------------------------------     
    
body guidecamCtrl::simbad {} {
   busy { AstroCat::new_catalog SIMBAD $this $itk_option(-debug) }
}

#------------------------------------------------------
# METHOD : ned 
#  
# Pop up a dialog for the NED catalog server
#------------------------------------------------------     

body guidecamCtrl::ned {} {
   busy { AstroCat::new_catalog NED $this $itk_option(-debug) }
}

#------------------------------------------------------
# METHOD : archives 
#  
# Pop up a dialog for the astronomical Archives (GSC, etc...)
#------------------------------------------------------    

body guidecamCtrl::archives {} {
   busy { AstroCat::new_catalog {NTT Archive at ESO} $this $itk_option(-debug)}
}

#------------------------------------------------------
# METHOD : image_servers 
#  
# Pop up a dialog for the image servers (DSS, etc...)
#------------------------------------------------------    

body guidecamCtrl::image_servers {} {
   busy { cat::AstroImage::new_imagesvr {Digitized Sky at ESO} $this \
          $itk_option(-debug) }
}

#------------------------------------------------------
# METHOD : mark_image 
#    
# Display a rectangle on the image at the given center coords 
# with the given width and height and return the items tag or id.
#------------------------------------------------------   

body guidecamCtrl::mark_image {ra dec width height} {
   # get radius from width and height
   set a $width/2.0
   set radw [expr sqrt(2.0*$a*$a)]
   set a $height/2.0
   set radh [expr sqrt(2.0*$a*$a)]
	
   # combine 2 square boxes to get the rectangle
   lassign [$image_ radecbox $ra $dec $radw] wr0 wd0 wr1 wd1
   lassign [$image_ radecbox $ra $dec $radh] hr0 hd0 hr1 hd1
	
   set eq [$image_ wcsequinox]
   if {[catch {
       $image_ convert coords $wr0 $hd0 "wcs $eq" x0 y0 canvas
       $image_ convert coords $wr1 $hd1 "wcs $eq" x1 y1 canvas
   } msg]} {
       error_dialog $msg $w_
   return
   }

   set id [$canvas_ create rectangle $x0 $y0 $x1 $y1 \
		    -width 2 -tags objects -fill yellow \
		    -outline white -stipple pat6]

   $itk_component(draw) add_object_bindings $id
   return $id
}
