#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: guidecamImagePick.tcl,v 1.25.1.6 2005/12/09 18:38:42 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    25/11/96  created
#           21/10/97  Add entry widgets to send SETUP command
#

#***********************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual guidecamImagePick {}

class guidecamImagePick {
    inherit util::TopLevelWidget

    #  create a new RtdImagePick widget

    constructor {args} {}

    # destructor

    destructor {
	catch {cancel}
	catch {$target_image_ view remove $image_}
        catch {deleteTcsWindow}
    }

    #guidecam_block begin
 
    public method close {} { 
	global pickDlg
	unset pickDlg
	cancel
	# Give tkwait in pick_object_in_image time to react. update does not do it
	after 1000 delete object $this
    }
    #guidecam_block end
    
    # this method can be called when the user has selected an object 
    # or star for the "pick_object" method. 
    # The argument should be the value returned from the rtdimage
    # "statistics" subcommand

    public method picked_wcs_object {retval} {
	global ::$w_.picked
	set $w_.picked $retval
    }

    # cancel the wait for the pick_object method and reset the cursor

    public method cancel_pick {} {
	global ::$w_.picked
	set $w_.picked ""
    } 
  
    public method deleteTcsWindow {} {
       if {[set id [$target_canvas_ find withtag tcsRefStar]] != ""} {
            $target_canvas_ delete $id
        }
    }

    # Update Image Size and Ref. Star Window in canvas

    public method setTcsWindowSize {} {
        if {[catch {set list [$image_ statistics]}] == 0 } {
           $canvas_ delete mark
           $target_canvas_ delete mark
           set xref [lindex $list 0]
           set yref [lindex $list 1]
           setTcsTargetImage
	   set_values $list
        }
    }   

    # format a floating point value (which may also be empty)

    public method format_val {val} {
        if {"$val" == ""} { return }
        return [format {%.1f} $val]
    }

    public method make_labels {w}
    public method make_rect   {w}
    public method add_buttons {}
    public method update_rect { size }
    public method update_scale {{fx 1} {fy 1}}
    public method pick_object  {}
    public method set_values   {list}
    public method mark_spot {imagex imagey image canvas angle width height}
    public method cancel       {}
    public method pick_object_in_image {width height}
    public method picked_object {} 
    public method picked_special_object {x y ra dec equinox}
    public method makeTcsWindow {} 
    public method readSetupFile {} 
    public method setTcsTargetImage {}
    public method force_pick {}

    itk_option define -rtdTop rtdTop RtdTop "."

     # target (main) RtdImage itcl widget
    itk_option define -target_image target_image Target_image {} {
	set target_image_ [$itk_option(-target_image) get_image]
	set target_canvas_ [$itk_option(-target_image) get_canvas]
    }

    # X shared memory option
    itk_option define -usexshm useXshm UseXshm 1

    # flag: if true, print diagnostic messages
    itk_option define -verbose verbose Verbose {0}

    # fonts used
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
    itk_option define -valuefont valueFont ValueFont -Adobe-helvetica-medium-r-normal--12*
    itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

    # set the width for  displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 15
    itk_option define -valuewidth valueWidth ValueWidth 13

    # set the default size of the image sample area (image dist)
    itk_option define -samplesize sampleSize SampleSize 50

    # set the min,max size of the image sample area (screen dist)

    itk_option define -minsize minSize MinSize 10
    itk_option define -maxsize maxSize MaxSize 100

    # command to evaluate when a selection is made or canceled
    itk_option define -command command Command {}

    # debugging flag
    itk_option define -debug debug Debug 0
   
    protected variable target_image_  "" ;# Canvas of the dialog
    protected variable target_canvas_ "" ;# Canvas with real image
    protected variable canvas_        "" ;# internal canvas widget
    protected variable tcsWindowId    "" ;# Id of telescope window
    protected variable image_         "" ;# internal rtd image
    protected variable sampleSize_    50 ;# current sample size
    protected variable xref           "" ;# X coord tcs ref. star
    protected variable yref           "" ;# Y coord tcs ref. star

    # label for scale widget
    protected variable scale_label_ "Sample size (in image pixels): "   
}

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body guidecamImagePick::constructor { args } {
   global agProcName envName

   eval itk_initialize $args 
   
   wm title $w_ "Pick Reference Star"
   wm geometry $w_ +10+600
   # main controls frame
    
   pack [set mainf [frame $w_.mainf]] -side top -fill both \
         -expand 1 -padx 1m -pady 1m -ipadx 1m -ipady 1m

   pack [set topf [frame $mainf.topf]] -side top -fill both -expand 1

   make_labels $topf

   # make scale

   pack [set scalef [frame $mainf.scalef -bd 2 -relief groove]] \
	-side top -fill x -expand 1 -padx 1m -pady 1m

   itk_component add scale {
       scale $scalef.scale -label $scale_label_ \
             -orient horizontal -from $itk_option(-minsize) \
             -to $itk_option(-maxsize) -showvalue 0 \
             -highlightthickness 0 -command [code $this update_rect ]
   } 

   pack $itk_component(scale) -side top -fill x -expand 1
   $itk_component(scale) set $itk_option(-samplesize)
   add_short_help $itk_component(scale) \
   {Scale: set the size of the image area to examine (in FITS image pixels)}

   # Update the window size and re-calculate statistics when the
   # user finished to set the new window size

   bind $itk_component(scale) <ButtonRelease-1> "$this setTcsWindowSize"

   make_rect $topf
   add_buttons
   update_scale
}



#------------------------------------------------------
# METHOD : readSetupFile  
#------------------------------------------------------
 
body guidecamImagePick::readSetupFile {}  {
   global ::gvar ::env

   set ret [slxtclLoadSetup ccdAgRtdWindow.det]

   # Set current window size
   $itk_component(scale) set $gvar(setup.4.DET.WIN1.NX)
   set sampleSize_ $gvar(setup.4.DET.WIN1.NX)
   
   set xref $gvar(setup.0.DET.WIN1.REFX)
   set yref $gvar(setup.1.DET.WIN1.REFY)

   $itk_component(x)          config -value $xref
   $itk_component(y)          config -value $yref
   $itk_component(ra)         config -value ""
   $itk_component(dec)        config -value ""
   $itk_component(equinox)    config -value ""
   $itk_component(angle)      config -value ""
   $itk_component(object)     config -value ""
   $itk_component(background) config -value ""
   $itk_component(fwhm)       config -value ""

   setTcsTargetImage
   makeTcsWindow
}

#------------------------------------------------------
# METHOD : setTcsTargetImage 
#
# For the cumputation of xof,yof the origin point is the upper left corner
#
#------------------------------------------------------
 
body guidecamImagePick::setTcsTargetImage {}  {
   
#   puts "DBG> guidecamImagePick::setTcsTargetImage .."
#   puts "DBG> xref = $xref  yref = $yref  height = [$target_image_ height] "

   set s $sampleSize_
   set s2 [expr $s/2]
   set xof [expr $xref - $s2 + 0.5]
   set yof [expr [$target_image_ height] - $yref - $s2 + 0.5]

#   puts "DBG> ref ($xref,$yref)  off ($xof,$yof)  size = $s "
   $target_image_ view update $image_ $xof $yof $s $s 0 0 0 0 image
}

#------------------------------------------------------
# METHOD : update_rect  
#  
# update the size of the square in the canvas from the scale widget.
# "size" is the size of the sample image in image pixels.
#------------------------------------------------------
 
body guidecamImagePick::update_rect { size }  {

   set sampleSize_ $size
   $target_image_ convert dist 0 $size canvas 0 num_pixels image
   
   if {$num_pixels < 5} { return }
   $itk_component(scale) config -label "$scale_label_ [expr int($num_pixels+0.5)]"
   $itk_component(image) config -canvaswidth $size -canvasheight $size
   $target_image_ view update $image_ $size $size image
}


#------------------------------------------------------
# METHOD : update_scale 
#
# this method is called when the scale of the image is changed to update the
# scale widget and display to keep the same number of image pixels.
# 
# fx and fy are the scale factors for the canvas (the image is already scaled)
#
# Note: Don't allow statistics on area of image less than about 5x5 pixels
#------------------------------------------------------

body guidecamImagePick::update_scale {{fx 1} {fy 1}} {

   $canvas_ scale mark 0 0 $fx $fy

   # set lower bound on scale widget
   $itk_component(scale) config -from [expr $fx*5]

   # update the sample image
   update_rect $sampleSize_
}

body guidecamImagePick::makeTcsWindow {} {

   if { $xref == "" || $yref == ""} { return }

   deleteTcsWindow

   set size [expr $sampleSize_/2]
   $target_image_ convert coords $xref $yref image xc yc canvas
   $target_image_ convert dist   $size $size image h  w  canvas

   set x1 [expr $xc - $h] ; set y1 [expr $yc - $h] 
   set x2 [expr $xc + $h] ; set y2 [expr $yc + $h]

   $target_canvas_ create rectangle $x1 $y1 $x2 $y2 \
                   -tags tcsRefStar -outline white

   $target_image_ view update $image_ $sampleSize_ $sampleSize_ image
}

#------------------------------------------------------
# METHOD : pick_object  
#
# let the user select a point in the image and get the statistics on
# the area
#------------------------------------------------------

body guidecamImagePick::pick_object {} {
   global gvar

   if {[$itk_option(-rtdTop) getMode] == "Camera"} {
      $itk_option(-rtdTop) detach_camera          ;# Detach camera
   }

   $itk_component(pick) config -state disabled ;# Disable pick button

   cancel

   # The target_image_ in the pick dialog follows the movements
   # of the cursor on the 'real' image.

   $target_image_ zoomview start $image_ 1 1 2

   update_rect $sampleSize_
   $w_ configure -cursor cross

   # wait for user to click in image and check if operation 
   # has been 'cancelled'

   set list [pick_object_in_image $sampleSize_ $sampleSize_]

   $itk_component(pick) config -state normal
   $w_ configure -cursor {}
   $target_image_ zoomview stop 2

   set_values $list

   puts "guidecamImagePick::pick_object $list"

   #guidecam_block begin
   if ![lempty $list] {
       global objRa objDec
       set objRa [lindex $list 2]
       set objDec [lindex $list 3]
       if {$gvar(lite)} {
	   set instance [armDisplayDlgLite::getInstance]
       } else {
	   set instance [armDisplayDlg::getInstance]
       }
       $instance starClicked
   }
  
   #utilRaiseWindow $w_

   if {![lempty $list]} { 
       if { !$gvar(lite) } {
	   makeAgSetupFile
	   guidecamSendSetup
       }
   }
   #guidecam_block end

   if {[$itk_option(-rtdTop) getMode] == "Camera"} {
      $itk_option(-rtdTop) attach_camera          ;# Detach camera
   }
}

#------------------------------------------------------
# METHOD : format_val  
# format a floating point value (which may also be empty)
#------------------------------------------------------

body guidecamImagePick::format_val {val} {


   if {"$val" == ""} { return }
   return [format {%.1f} $val]
}


#------------------------------------------------------
# METHOD : set_values    
#
# set the values of the labels from the list (results of "pick_object" call).
# If list is empty the labels are cleared.
# If the list is not empty, mark the ra,dec spot in the image.
# format a floating point value (which may also be empty)
#------------------------------------------------------

body guidecamImagePick::set_values {list} {
    global gvar pickMode

    if {[lempty $list]}        { return }
    if {[llength $list] != 10} { set list [linsert $list 2 ""] }

#   puts "DBG> guidecamImagePick::set_values .. ($list)"
   lassign $list xref yref ra dec equinox fwhmX fwhmY angle object background
   set offset [expr $sampleSize_/2 ]

   if {$pickMode == "pickCursor"} {
       set w [$itk_option(-target_image) get_image]

       # Image coordinates are already stored in a structure
       set xref [uplevel #0 set ${w}(X)]  
       set yref [uplevel #0 set ${w}(Y)]
#       puts "DBG> image(x,y) = $xref $yref"   
   }

   set gvar(setup.0.DET.WIN1.REFX) $xref
   set gvar(setup.1.DET.WIN1.REFY) $yref
   set gvar(setup.2.DET.WIN1.ST)  T
   set gvar(setup.3.DET.WIN1.STRX) [expr $xref - $offset]
   set gvar(setup.4.DET.WIN1.NX)   $sampleSize_
   set gvar(setup.5.DET.WIN1.STRY) [expr $yref - $offset]
   set gvar(setup.6.DET.WIN1.NY)   $sampleSize_
   set gvar(setup.7.DET.WIN1.BINX) 1
   set gvar(setup.8.DET.WIN1.BINY) 1

   $itk_component(x) config -value $xref
   $itk_component(y) config -value $yref
   $itk_component(ra) config -value $ra
   $itk_component(dec) config -value $dec
   $itk_component(equinox) config -value $equinox
   $itk_component(angle) config -value [format_val $angle]
   $itk_component(object) config -value [format_val $object]
   $itk_component(background) config -value [format_val $background]
   $itk_component(fwhm) config -value \
                        "[format_val $fwhmX] : [format_val $fwhmY]"

   if {"$xref" != "" && "$yref" != ""} {
      mark_spot $xref $yref $image_ $canvas_ $angle $fwhmX $fwhmY
      mark_spot $xref $yref $target_image_ $target_canvas_ $angle $fwhmX $fwhmY
   }

   makeTcsWindow
}


#------------------------------------------------------
# METHOD : mark_spot  
#
# mark the given x,y image coordinate point in the given image/canvas with
# a cross with the given width, height (image pixels) and angle (deg).
#------------------------------------------------------


body guidecamImagePick::mark_spot {imagex imagey image canvas angle width height} {
   # get canvas coords from image coords
   $image convert coords $imagex $imagey image xc yc canvas
   $image convert dist $width $height image w h canvas

   # get bounding box

   set dx [expr $w/2] ;     set dy [expr $h/2]

   set x1 [expr $xc+$dx] ; set y1 $yc
   set x2 $xc;             set y2 [expr $yc-$dy]
   set x3 [expr $xc-$dx] ; set y3 $yc
   set x4 $xc ;            set y4 [expr $yc+$dy]

   # rotate by angle
   # convert to radian
   set rad [expr $angle/57.2958]
   set cosa [expr cos($rad)]
   set sina [expr sin($rad)]

   set rx1 [expr $xc+$cosa*$dx]
   set ry1 [expr $y1-$sina*$dx]

   set rx3 [expr $xc-$cosa*$dx]
   set ry3 [expr $y1+$sina*$dx]

   set rx2 [expr $x2-$sina*$dy]
   set ry2 [expr $yc-$cosa*$dy]

   set rx4 [expr $x2+$sina*$dy]
   set ry4 [expr $yc+$cosa*$dy]

   set bg black
   set fg white
   set tags "mark objects"
   $canvas delete $tags
	
   $canvas create line $rx3 $ry3 $rx1 $ry1 \
	    -fill $bg -width 5 -tags $tags

   $canvas create line $rx2 $ry2 $rx4 $ry4 \
	    -fill $bg -width 5 -tags $tags

   $canvas create line $rx3 $ry3 $rx1 $ry1 \
	    -fill $fg -width 2 -tags $tags

   $canvas create line $rx2 $ry2 $rx4 $ry4 \
	    -fill $fg -width 2 -tags $tags
}

#------------------------------------------------------
# METHOD : cancel 
#------------------------------------------------------

body guidecamImagePick::cancel {} {
   $canvas_ delete mark
   $target_canvas_ delete mark
   $target_image_ zoomview stop 2
   cancel_pick
}


#------------------------------------------------------
# METHOD : pick_object_in_image
#
# this method is called to allow the user to pick an object in the 
# main image.
# The arguments indicate the size of the area around the mouse ptr 
# to examine.
# The return value is a list of:
# "ra dec equinox fwhmX fwhmY angle objectPeak meanBackground" 
# as returned from the rtdimage "statistics" subcommand,
# or an error. 
#------------------------------------------------------

body guidecamImagePick::pick_object_in_image {width height} {

    set cursor [$target_canvas_ cget -cursor]
    $target_canvas_ configure -cursor cross

    set bindtags [bindtags $target_canvas_]
    set tag pick$w_
    bind $tag <ButtonRelease-1> [code $this picked_object]
    bindtags $target_canvas_ $tag

    global ::$w_.picked
    set $w_.picked {}
    tkwait variable $w_.picked

    bindtags $target_canvas_ $bindtags

    $target_canvas_ configure -cursor $cursor
    set ret [set $w_.picked]
	
    if {"$itk_option(-command)" != ""} {
       eval $itk_option(-command) [list $ret]
    }

    return $ret
}

#------------------------------------------------------
# METHOD : picked_object
#   
# this method is called when the user clicks in the image to 
# select an object or star for the "pick_object" method.
#------------------------------------------------------

body guidecamImagePick::picked_object {} {

   # display busy cursor in image and pick window...
   $itk_option(-target_image) busy {
      busy {

        if {[catch {set list [$image_ statistics]} msg]} {
           error_dialog $msg
           cancel_pick
        } else {
	    picked_wcs_object $list
        }
      } ;# end busy
   }
}


#------------------------------------------------------
# METHOD : picked_special_object
#
# this method is called when the user clicks in the image to select 
# an object or star for the "pick_object" method. 
# In this case, the x,y and ra,dec position are fixed and only the other 
# info should be calculated (used).
#------------------------------------------------------

body guidecamImagePick::picked_special_object {x y ra dec equinox} {

   # display busy cursor in image and pick window...
   $itk_option(-target_image) busy {
	busy {
              if {[catch {set list [$image_ statistics]} msg]} {
		 error_dialog $msg
		 cancel_pick
              } else {
		 lassign $list {} {} {} {} {} fwhmX fwhmY angle object background
		 picked_wcs_object \
			[list $x $y $ra $dec $equinox $fwhmX $fwhmY $angle $object $background]
              }
	    }
	}
}

###################################################################
#                   METHOD USED TO BUILD THE PANEL
###################################################################


#------------------------------------------------------
# METHOD : make_labels
#------------------------------------------------------

body guidecamImagePick::make_labels {w} {

   # frame for labels
   pack [set labelf [frame $w.labelf -bd 2 -relief groove]] \
        -side left -fill both -expand 0

   # title for label area
   pack [label $labelf.title -text "Image Statistics:"] \
        -side top -padx 1m -pady 1m

   # Add additional frames to optimize packing

   pack [frame $labelf.f1 ] -side top -padx 0 -pady 0
   pack [frame $labelf.f2 ] -side top -padx 0 -pady 0
	
	itk_component add x {
	    LabelValue $labelf.f1.x \
		-text "Image X,Y:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-labelwidth 8\
		-valuewidth $itk_option(-valuewidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add y {
	    LabelValue $labelf.f1.y \
		-text "" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-labelwidth 0 \
		-valuewidth $itk_option(-valuewidth) \
		-relief groove \
		-anchor e
	} 

	itk_component add ra {
	    LabelValue $labelf.f2.ra \
		-text "a,d:" \
		-labelfont $itk_option(-wcsfont) \
		-valuefont $itk_option(-valuefont) \
		-labelwidth 8 \
		-valuewidth $itk_option(-valuewidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add dec {
	    LabelValue $labelf.f2.dec \
		-text "" \
		-labelfont $itk_option(-wcsfont) \
		-valuefont $itk_option(-valuefont) \
		-labelwidth 0 \
		-valuewidth $itk_option(-valuewidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add equinox {
	    LabelValue $labelf.equinox \
		-text "Equinox:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-valuewidth) \
		-labelwidth $itk_option(-labelwidth)  \
		-relief groove \
		-anchor e
	} 
	itk_component add object {
	    LabelValue $labelf.object \
		-text "Level above bg:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-valuewidth) \
		-labelwidth $itk_option(-labelwidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add background {
	    LabelValue $labelf.background \
		-text "Background level:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-valuewidth) \
		-labelwidth $itk_option(-labelwidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add fwhm {
	    LabelValue $labelf.fwhm \
		-text "FWHM X:Y:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-valuewidth) \
		-labelwidth $itk_option(-labelwidth) \
		-relief groove \
		-anchor e
	} 
	itk_component add angle {
	    LabelValue $labelf.angle \
		-text "Angle of X axis:" \
		-labelfont $itk_option(-labelfont) \
		-valuefont $itk_option(-valuefont) \
		-valuewidth $itk_option(-valuewidth) \
		-labelwidth $itk_option(-labelwidth) \
		-relief groove \
		-anchor e
	} 


	pack $itk_component(y)  $itk_component(x) -side right -padx 0.5m \
             -pady 0.2m
	pack $itk_component(dec)  $itk_component(ra) -side right -padx 0.5m \
             -pady 0.2m

	pack $itk_component(equinox) $itk_component(object) \
	     $itk_component(background) $itk_component(fwhm) \
	     $itk_component(angle) \
	    -side top -padx 0.5m -pady 0.2m

   add_short_help $itk_component(x)   {X image (pixel) coordinate}
   add_short_help $itk_component(y)   {Y Image (pixel) coordinate}
   add_short_help $itk_component(ra)  {World Coordinates RA value}
   add_short_help $itk_component(dec) {World Coordinates DEC value}
   add_short_help $itk_component(background) {Mean background level}

   add_short_help $itk_component(equinox) \
                  {World Coordinates equinox (default: J2000)}
   add_short_help $itk_component(object) \
                  {Object: peak value of object above background}

   add_short_help $itk_component(fwhm) \
                  {FWHM: full width half maximum in X and Y}
   add_short_help $itk_component(angle) \
                  {Angle of major axis, degrees, along X = 0}
}
   
#------------------------------------------------------
# METHOD : make_rect
#------------------------------------------------------

body guidecamImagePick::make_rect {w} {
   global ::pickMode

   # Top frame for the image

   set top  [frame $w.rectf -bd 2 -relief groove]
   pack     $top -side left -fill both -expand 0

   label $top.label -text "Image Area"
   pack $top.label -side top -padx 1m -pady 1m

   # Frame where to place the image
   set rf   [frame $top.rf -bd 2 -relief groove \
             -width $itk_option(-maxsize)  -height $itk_option(-maxsize)]

   pack $rf  -fill none -expand 1 -anchor c

   # Frame for radioButtons

   set pickFrame [frame $top.pickFrame -bd 2 -relief groove]
   pack $pickFrame -fill x 

   label $pickFrame.lblPick -text "Select 'pick' mode"

   radiobutton $pickFrame.pickObj  -text "Pick Object" \
               -variable pickMode -value pickObject

   radiobutton $pickFrame.pickCur  -text "Pick Cursor" \
               -variable pickMode -value pickCursor

   pack $pickFrame.lblPick -anchor w
   pack $pickFrame.pickObj -anchor w
   pack $pickFrame.pickCur -anchor w

   set pickMode "pickObject" ;# Set default value

   # display the section of the image that will be used
   itk_component add image {
       RtdImage $rf.image -name "PickImage" \
                -verbose $itk_option(-verbose) \
                -scrollbars 0 -drag_scroll 0 \
                -show_object_menu 0 -graphics 0 \
                -canvaswidth $itk_option(-maxsize) \
                -canvasheight $itk_option(-maxsize) \
                -debug $itk_option(-debug) \
                -usexshm $itk_option(-usexshm) \
                -shelp {PickImage: displays section of image being examined}
   } {
   }

   pack $itk_component(image) -anchor c -fill none -expand 0

   set image_ [$itk_component(image) get_image]
   set canvas_ [$itk_component(image) get_canvas]

   $itk_component(image) config \
	    -canvaswidth $itk_option(-samplesize) \
	    -canvasheight $itk_option(-samplesize)

   catch {$target_image_ view remove $image_}
   $target_image_ view add $image_ 1
}


#------------------------------------------------------
# METHOD : add_buttons
#------------------------------------------------------
 
body guidecamImagePick::add_buttons {} {
   global gvar    

   # Add frames

   itk_component add buttons {
       frame $w_.buttons -borderwidth 2 -relief groove
   }

   itk_component add destFrame {
       frame $w_.destFrame
   }

   pack $itk_component(destFrame) $itk_component(buttons) -side top -fill x\
        -padx 1m -pady 0.5

   # Add Labels

#   itk_component add lblEnv {
#       label $w_.lblEnv -text "Env. Name :"
#   }

   itk_component add lblProc {
       label $w_.lblProc -text "Process :"
   }

   # Add Entry widgets

#   itk_component add envName {
#       entry $w_.envName -textvariable tcsEnvName -width 8
#   }

   itk_component add procName {
       entry $w_.procName -textvariable agProcName -width 24
   }

   pack  $itk_component(lblProc) $itk_component(procName) \
        -in $itk_component(destFrame) -side left -padx 2m -pady 0.5m

#   pack $itk_component(lblEnv)  $itk_component(envName) \
#        $itk_component(lblProc) $itk_component(procName) \
#        -in $itk_component(destFrame) -side left -padx 5 -pady 5

   # add dialog buttons

   itk_component add pick {
       button $w_.pick -text "Pick Object" -command [code $this pick_object]
   }

   itk_component add cancel {
       button $w_.cancel -text "Cancel Operation" \
              -command [code $this cancel]
   }

   if { !$gvar(lite) } {
       itk_component add readSetupFile {
	   button $w_.readSetupFile -text "Read Setup File" \
		   -command [code $this readSetupFile]
       }
   }

#   itk_component add writeSetupFile {
#       button $w_.writeSetupFile -text "Generate Setup File" \
#              -command "makeAgSetupFile"
#   }

   itk_component add close {
       button $w_.close -text "Close" -command [code $this close]
   }

   if { $gvar(lite) } {
       pack $itk_component(pick) $itk_component(cancel) \
	       $itk_component(close) \
	       -side left -expand 1 -padx 2m \
	       -pady 0.5m -in $itk_component(buttons)
   } else {
       pack $itk_component(pick) $itk_component(cancel) \
	       $itk_component(readSetupFile) $itk_component(close) \
	       -side left -expand 1 -padx 2m \
	       -pady 0.5m -in $itk_component(buttons)
   }

   add_short_help $itk_component(pick) \
   {Pick Object: {bitmap b1} = select object in image and display center and other statistics}

   add_short_help $itk_component(cancel) {Cancel: cancel the current pick operation}
   add_short_help $itk_component(close) {Close: close the pick window}
}



# ___oOo___
