# E.S.O. - VLT project/ESO Archive
# $Id 
#
# symbols.tcl - utility procs to draw plot symbols on a canvas
#
# Each of the procs defined here has the same interface. 
# draw_$symbol draws the symbol at the given coordinates on the canvas.
# For convenience, the center point (x, y) and the bounding box
# (x0, y0, x1, y1) are given, as well as the fg and bg colors, line width 
# and tag list.
#
# who         when       what
# --------   ---------   ----------------------------------------------
# A.Brighton 22 Jan 96   created


proc draw_square {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create rect $x0 $y0 $x1 $y1 \
	-outline $bg \
	-fill $bg \
	-width 2 \
	-stipple pat7 \
	-tags $tags
    $canvas create rect $x0 $y0 $x1 $y1 \
	-outline $fg \
	-fill $bg \
	-width 1 \
	-stipple pat7 \
	-tags $tags
}


proc draw_circle {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create oval [expr $x0-1] [expr $y0-1] [expr $x1+1] [expr $y1+1] \
	-outline $bg \
	-fill $bg \
	-width 1 \
	-stipple pat7 \
	-tags $tags
    $canvas create oval $x0 $y0 $x1 $y1 \
	-outline $fg \
	-fill $bg \
	-width 1 \
	-stipple pat7 \
	-tags $tags
}


proc draw_plus {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create line $x $y0 $x $y1 \
	-fill $bg \
	-width 2 \
	-tags $tags
    $canvas create line $x0 $y $x1 $y \
	-fill $bg \
	-width 2 \
	-tags $tags
    $canvas create line $x $y0 $x $y1 \
	-fill $fg \
	-width 1 \
	-tags $tags
    $canvas create line $x0 $y $x1 $y \
	-fill $fg \
	-width 1 \
	-tags $tags
}


proc draw_cross {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create line $x0 $y0 $x1 $y1 \
	-fill $bg \
	-width 2 \
	-tags $tags
    $canvas create line $x1 $y0 $x0 $y1 \
	-fill $bg \
	-width 2 \
	-tags $tags
    $canvas create line $x0 $y0 $x1 $y1 \
	-fill $fg \
	-width 1 \
	-tags $tags
    $canvas create line $x1 $y0 $x0 $y1 \
	-fill $fg \
	-width 1 \
	-tags $tags
}


proc draw_diamond {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create polygon $x0 $y $x $y0 $x1 $y $x $y1 \
	-outline $bg \
	-fill $bg \
	-width 2 \
	-stipple pat7 \
	-tags $tags
    $canvas create polygon $x0 $y $x $y0 $x1 $y $x $y1 \
	-outline $fg \
	-fill $bg \
	-width 1 \
	-stipple pat7 \
	-tags $tags
}

proc draw_triangle {canvas x y x0 y0 x1 y1 bg fg tags} {
    $canvas create polygon $x0 $y1 $x $y0 $x1 $y1 \
	-outline $bg \
	-fill $bg \
	-width 2 \
	-stipple pat7 \
	-tags $tags
    $canvas create polygon $x0 $y1 $x $y0 $x1 $y1 \
	-outline $fg \
	-fill $bg \
	-width 1 \
	-stipple pat7 \
	-tags $tags
}

