#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: commonProc.tcl,v 5.19 2003/02/14 14:23:32 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Store it into a separate file .. with comments
#                     Contains only variables needed by both panels
#                     and panel editor
# M.Comin   27/07/94  Add names for frames in the working window
# M.COMIN   07/03/95  add optional par list to panCallPanel 
# M.COMIN   12/04/95  Add panUsrExit
# M.COMIN   13/04/95  Add confirmation dialog to panClosePanel
# M.COMIN   05/05/95  Add X application defaults
# M.COMIN   08/09/95  Refined version of panNewWdgName
# M.COMIN   03/12/95  Add new procedure panNewVarName
# ------------------
# mcomin    08/01/96  Fixed bug in panNewVarName
# mcomin    27/04/96  Fixed another bug in panNewVarName
#           03/05/96  Fixed another bug in regexp of panNewVarName
#
#
#************************************************************************
#   NAME
#
#   commonProc - Set of general routines 
# 
#   SYNOPSIS
#
#   commonProc
# 
#   DESCRIPTION
#
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   This is also included in the panel files.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#----------------------------------------------------------------------------
# PROCEDURE : panReadOption - Read the color palette definition
#----------------------------------------------------------------------------

proc panReadOption { { userColorDef  "" } } {
    global par colordef EDITING

    if {! [cequal $userColorDef ""]} { set colordef $userColorDef }

    option  clear
    option  readfile [findVLTFile app-defaults/Xpanel.common]
    option  readfile [findVLTFile app-defaults/Xpanel.$colordef]

    if {[set filePath [findVLTFile app-defaults/XseqWish no]] != ""} {
       option  readfile $filePath
    }

    if {$EDITING} {
       option  readfile [findVLTFile app-defaults/Xpanel.private] interactive
    }

    # Load Application Resources

    if { [set tmp [findVLTFile "app-defaults/X$par(MY_NAME)" no ]] != "" } {
       option readfile $tmp
    }

    #
    # Set common resources
    #
    
    set par(OUT_FONT)       "-Adobe-courier-medium-r-normal--14*-iso8859-1"
    set par(FREETEXT_FONT)  "-Adobe-helvetica-medium-r-normal--14*-iso8859-1"
       
    set par(FREETEXT_FG)       black
    set par(BITMAP_ACTIVE_BG)  red
 
    switch $colordef {
      bw      { set par(BARCOLOR) Gray90 ;   set par(BARCHART_BG) Gray75 }
      spring  { set par(BARCOLOR) SteelBlue; set par(BARCHART_BG) Gray90 }
      winter  { set par(BARCOLOR) Blue;   set par(BARCHART_BG) Gray90  }
      summer  { set par(BARCOLOR) Blue;   set par(BARCHART_BG) #9f9bbf }
      autumm  { set par(BARCOLOR) Cyan;   set par(BARCHART_BG) SandyBrown }
    } ; #end of switch
}


#----------------------------------------------------------------------------
# PROCEDURE : setWidget - Create a new element of the array
#----------------------------------------------------------------------------

proc setWidget { name args } {
   global panWidget
   set panWidget($name) {}
   foreach item $args {
       lappend panWidget($name) $item
   }
}


#----------------------------------------------------------------------------
# PROCEDURE : appendWidget - Append new configuration parameters
#----------------------------------------------------------------------------

proc appendWidget { name args } {
   global panWidget 
   foreach item $args {
       lappend panWidget($name) $item
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : getWidget - Get Widget configuration parameter
#----------------------------------------------------------------------------

proc getWidget { name i } {
   global panWidget 
   return  [lindex $panWidget($name)  $i]
}


#----------------------------------------------------------------------------
# PROCEDURE : saveWidget - Save Widget configuration Parameter
#----------------------------------------------------------------------------

proc saveWidget { name i value } {
   global panWidget
   set list              $panWidget($name)  
   set panWidget($name)  [lreplace  $list $i $i $value ] 
}

#----------------------------------------------------------------------------
# PROCEDURE : panSetWidgetLimits -  Sets the widget limits according to
#                                   the DB values 
#----------------------------------------------------------------------------

proc panSetWidgetLimits {cv} {
   global i_type panWidget panDbData 
   global db_var db_onval db_offval db_min db_max

   foreach name [array names panWidget] {

      if { ! [info exists panDbData($name)] } { continue }

      set type [getWidget $name $i_type]

      switch $type {
         UifBarChart -
         UifEntry -
         UifScale {
           panSetLimit $cv $name minVal $db_min
           panSetLimit $cv $name maxVal $db_max
         }

         UifCheckButton {
           panSetLimit $cv $name offVal $db_offval
           panSetLimit $cv $name onVal  $db_onval
         }

         UifRadioButton { panSetLimit $cv $name onVal  $db_onval }
      } ;# End of switch
   } ;#end foreach ...
}

proc panSetLimit { cv name what idx } {
   if {[set tmp [getDbConfig $name $idx ]] == "0" } { return }

   set limit [panGetWdgAttribute $name $tmp]

   switch $what {
      minVal { $cv.$name configure -minVal $limit }
      maxVal { $cv.$name configure -maxVal $limit }
      onVal  { $cv.$name configure -onVal  $limit }
      offVal { $cv.$name configure -offVal $limit }
   }
}

