#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: createWidgets.tcl,v 5.19 2003/02/14 14:23:36 vltsccm Exp $" 
#
# who       when        what
# --------  ----------  ----------------------------------------------
# M.Bernini 24/05/94    created 
# M.Comin   05/07/94    Store it into a separate file .. with comments
# M.COMIN   21/03/95    Add UifListBox
# M.COMIN   23/03/95    Add UifOutputString
# M.COMIN   06/05/95    Add UifText
# M.COMIN   21/05/95    Take away parameter cv from newWidget
# M.COMIN   30/10/95    Trap error creating a class : the class is unloaded
#                       and the widget destroyed
# M.COMIN   08/11/95    Set auto_path before the class is declared.
# M.COMIN   21/11/95    Add dummy widget variable at widget creation
# --------------------
# mcomin    16/09/96    Add automatic search path for bitmaps
#           06/03/97    Porting to TK 4.2
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#
#************************************************************************
#   NAME
#
#   createWidgets - Creates the first instance of the widget in the panel
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   Each routine to create a specific widget perform the following actions :
#
#   - Give the widget a standard name
#   - Configure <B2> to move the widget
#   - Configure <B3> to call the widget configuration panel
#   - Initialize the data structures of the widget.
#   - increment the total widget counter.
#
#
#   Each time a widget is created, the canvas returns an index to the 
#   new widget.
#
#   One can give 'tags' to the new widget which can be used to configure it.
#   Several tags can be assigned to a given widget and can be used to alter a 
#   specific widget configuration or the configuration of family of widgets.
#
#   Each widget will get the following standard tags :
#
#   - family name          (example pushButton, output, $type, etc ... )
#   - widget identifier    (example widget1, widget23, etc ..)
#   - widget name          (example button1, scale10,  etc ... )
#
#   FILES
#    
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   old tag setup :   -tags " element $name window"
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

proc panImportList { {name ::} } {
   set result $name
   foreach n [namespace children $name] {
      append result " " [panImportList $n]
  }
  return $result
}

#----------------------------------------------------------------------------
# PROCEDURE : newWidget - Creates an instance of a widget in the panel
#                         It returns the current widget name.
#----------------------------------------------------------------------------

proc newWidget { type } {
   global  par panMaxWdg  i_clib classMatch INTROOT VLTROOT namespaceList
   global  panDEBUG

   panGetCvNames $par(ROOT_PANEL) cv bcv

#   puts "DBG > createWidgets panMaxWdg = $panMaxWdg"

   set  name   [string trimleft "$par(ROOT_PANEL)-wdg${panMaxWdg}" "."]
   set  ret    $name
   incr panMaxWdg

   switch $type {
     
      UifBitmap {

         set tmp "."

	 if {[file exists "../bitmaps"]} { set tmp "../bitmaps" }
	 if {[set file [panSelectFile $tmp "*.xbm"]] == ""} { return }

         set myBitmap  [exec basename $file]; 
         set file      [findVLTFile bitmaps/$myBitmap]
         set ret       [createWidget  $type $name $cv nw $file]
         initWidget    $type $name $cv $myBitmap
      }

      UifClass {

         set fsbReturn [panSelectFile . "*_uifClass.tcl" -subDir lib \
                  -top .classBrowser -title "Class Browser" -iconify ]

         if { $fsbReturn == "" } { return } 

#         puts "DBG> fsbReturn = $fsbReturn"

         # Determine type of class

         set className [panGetClassName $fsbReturn]

         if {[regexp "(\[a-zA-Z0-9\]+)_(\[a-zA-Z\]+)" $className all class ] == 0 } {
            set msg "This is not a UifCLass widget\nUse 'Import Mega Widget ...'"
            tk_dialog .dialog "Dialog Box" $msg info 0 OK
            return
         }

         set libPath [panReturnVLTPath  [file dirname $fsbReturn] lib]

         if {[cequal $panDEBUG dbgDEV]} {
            puts "DEV. DEBUG >  source file $fsbReturn"
            puts "DEV. DEBUG >  class name = $className   library = $libPath"
         }

         sourceMegaWidget $type $fsbReturn $libPath $className

         if {[createWidget $type $name $cv nw $className] == "FAILURE"} {
            return 
         }

         set myLib [file rootname [file tail $libPath]]
         $cv.$name configure -library $myLib

         return $name
      }


      PubClass {
         set fsbReturn [panSelectFile . "*" -subDir lib \
                  -top .classBrowser -title "Class Browser" -iconify ]

         if { $fsbReturn == "" } { return } 

         set className [panGetClassName $fsbReturn]
         set libPath [panReturnVLTPath  [file dirname $fsbReturn] lib]

         if {[cequal $panDEBUG dbgDEV]} {
            puts "DEV. DEBUG >  source file $fsbReturn"
            puts "DEV. DEBUG >  class name = $className   library = $libPath"
         }

         sourceMegaWidget $type $fsbReturn $libPath $className

         if {[createWidget $type $name $cv nw $className] == "FAILURE"} {
            return 
         }

         saveWidget $name $i_clib $libPath
         return $name
      }

      freeText { 
         createFreeText $name $cv 
         panSetWidgetBind  $cv  $name $type ; # Set Edit Bindings
      }

      UifActionButton { 
         set ret [createWidget $type $name $bcv nw]
         initWidget $type $name $bcv
      }

      default { 
         set ret [createWidget  $type $name $cv nw ]
         initWidget $type $name $cv
      }
   } 

   incr par(SAVE_FLAG)
   
   if {$ret != "FAILURE"} {
      return $name
   } else {
      return $ret
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : panGetClassName
#----------------------------------------------------------------------------

proc panGetClassName { filePath } {

   foreach member [split [eval exec grep class $filePath ] "\{\}\n"] {
      set key [namespace tail [lindex $member 0] ]
      if {[cequal $key class]} {
         return [lindex $member 1]
      }
   }
   return "UNKNOWN CLASS NAME"
}

#----------------------------------------------------------------------------
# PROCEDURE : sourceMegaWidget
#----------------------------------------------------------------------------

proc sourceMegaWidget { type filePath libPath className } {
   global par tcl_version

   incr par(SAVE_FLAG)
   
   # Check if the class has been already loaded and hence
   # known to the panel : if not source the class definition file

   set importList [panImportList]
   if {[cequal $type PubClass]} {

       if {$tcl_version >= 8.0} {
	   set nameSpace [namespace qualifiers $className]
       } else {
	   set nameSpace [info namespace qualifiers $className]
       }
      if {[set nameSpace] != ""} {
         # Check if namespace exists ...
         
         set found 0
         foreach item $importList {
	    if {[cequal [lindex $item 0] $nameSpace]} {
               set found 1 ; break
            }
         }

         if {$found == 0} { panAddNamespace $nameSpace }
      }
   }

#   if {$tcl_version >= 8.0} {
#       set classes [itcl::find classes]
#   } else {
#       set classes [info classes]
#   }

   if {[lsearch [panImportList] "*$className"] == -1 } {
#     set tmpPath [panReturnVLTPath $libPath]
#     puts "tmpPath = $tmpPath"
      # Link panel to class library
      panUpdateAutoPath $libPath
      source $filePath
   }
}


#----------------------------------------------------------------------------
# PROCEDURE : panReturnVLTPath - Return a filepath under the parent
#             directory of VLTROOT -e.g.- /vlt, if VLTROOT is /vlt/NOV97
#----------------------------------------------------------------------------

proc panReturnVLTPath { filePath subDir } {
   global par INTROOT VLTROOT MODULE TCLTK_ROOT

   if {[string match "${MODULE}/${subDir}*" $filePath]} {
      set len [string length ${MODULE}/${subDir}]
      return [string range $filePath [incr len] end]
   }

   if {[string match "${TCLTK_ROOT}/${subDir}*" $filePath]} {
      set len [string length ${TCLTK_ROOT}/${subDir}]
      return [string range $filePath [incr len] end]
   }

   if {[string match "${INTROOT}/${subDir}*" $filePath]} {
      set len [string length ${INTROOT}/${subDir}]
      return [string range $filePath [incr len] end]
   }

   if {[string match "${VLTROOT}/${subDir}*" $filePath]} {
      set len [string length ${VLTROOT}/${subDir}]
      return [string range $filePath [incr len] end]
   }

   # Check if this is a directory under VLTROOT parent
   if {[string match "*$par(VLT)*" $filePath] } {
      set idx [string last $par(VLT) $filePath]
      return [string range $filePath $idx end]
   }

   return $filePath
}



#----------------------------------------------------------------------------
# PROCEDURE : createWidget - 
#----------------------------------------------------------------------------

proc createWidget { type name cv anchor {optParam {}} } {
   global par id  ckActBut gvar tcl_version
   global out_fg panSelectedFont ;   # Foreground color and font for output

   if { $anchor == "" } { 
      set panAnchor "nw"
   } else {
      set panAnchor $anchor
   }

   if {[winfo exists $cv.$name]} {
      set msg "widget  $name\nALREADY EXISTS !! "
      tk_dialog .dialog "Dialog Box" $msg info 0 OK
      return "FAILURE"
   }

#   Set default variable name

   switch $type {  
       
       UifClass   -
       PubClass   {

           set className $optParam 
           set tmp  [namespace tail $optParam]

	   # Trick to cope with lower/upper case letters
	   
	   set key "*[string range $optParam 1 end]"
	   
#	   if {$tcl_version >= 8.0} {
#	       set className [itcl::find classes $key]
#	   } else {
#	       set className [info classes $key]
#	   }

           set classExists [lsearch [panImportList] "*$key"]

	   if { $classExists  == -1 } {
	       set msg "Class '$optParam' not DEFINED"
	       tk_dialog .dialog "Dialog Box" $msg info 0 OK
	       return FAILURE
	   }
	   
	   return [createClass $type $name $cv $panAnchor $className] 
       }
       
      UifOptionButton  { UifOptionButton $cv.$name -length 5 }
      UifArrowsButton  { UifArrowsButton $cv.$name }   
      UifBitmap        { UifBitmap       $cv.$name -bitmap @${optParam} }
      UifCheckButton   { UifCheckButton  $cv.$name -anchor w } 
      UifRadioButton   { UifRadioButton  $cv.$name -anchor w }
      UifListBox       { UifListBox      $cv.$name -geometry 5x5 -font $par(COUR_R14)}
      UifText          { UifText         $cv.$name -width 20 -height 3 \
                                         -font $par(COUR_R14) }
      UifOutputString  { UifOutputString $cv.$name -font $par(COUR_R14) -labelFont $par(HELV_R14)}

      UifOutput        -
      UifOutputList    { eval $type    $cv.$name -font $par(COUR_R14) -labelFont $par(HELV_R14)}

      UifOutputArray   { UifOutputArray  $cv.$name -size 1 -width 5 \
                         -labelFont $par(HELV_R14) -font $par(COUR_R14)
                       }
      UifBarChart      { UifBarChart     $cv.$name -barColor $par(BARCOLOR)\
                                  -background $par(BARCHART_BG) -maxVal 100 \
                                  -length 100 -minVal 0  -label ""            
                       }
      UifEntry         {UifEntry  $cv.$name -width 5 -labelFont $par(HELV_R14)}

      UifOutputLed     { UifOutputLed  $cv.$name -labelFont $par(HELV_R14) }
      UifButton        -
      UifEntry         -
      UifScale         -
      UifFrame         -
      UifText          { eval $type  $cv.$name }

      UifActionButton  { UifActionButton $cv.$name

                       set tmp [$cv cget -height]

                       if { $ckActBut == 0 && $tmp < 40 } { 
                          panSetActionAreaSize 40 
                       }

                       $cv.$name configure -canvasAnchor $panAnchor
                       $cv create window 50 10 -window $cv.$name   \
                                  -anchor $panAnchor -tags "$type $name widget"
                       incr ckActBut
		       }

      default          { 
                       set msg "Widget Type: $type\nNOT VALID !!"
                       tk_dialog .dialog "Dialog Box" $msg info 0 OK
                       return "FAILURE"
                       }                          
   }; # end of switch .. 


   if {$type != "UifActionButton" } {

#      puts "DBG > create widget $name $type $par(X) $par(Y)"
     
      $cv.$name configure -canvasAnchor $panAnchor  
      $cv create window $par(X) $par(Y) -window $cv.$name \
                 -anchor $panAnchor -tags "$type $name widget"
   }

}


#----------------------------------------------------------------------------
# PROCEDURE : createClass - Creates a 'class' widget
#----------------------------------------------------------------------------

proc createClass { type name cv where className } {
   global par auto_path  i_clAnchor

#   puts "createClass  $type $name $cv $where $className"
   #  Create an instance of the class

   if { [catch {eval "$className $cv.$name"} ret] } {
       set    msg  "ERROR CREATING CLASS  $className !!\n\n$ret\n\n"
       append msg "OPEARTION ABORTED !!"

       panShowError .dialog  $msg info 0 OK
       
       if {[winfo exists $cv.$name]} {
          delete class $class ;  # unload class definition and
          destroy $cv.$name   ;  # destroy current instance   
       }
       return FAILURE
   }

   if { $type == "UifClass" } {
      eval "$cv.$name  display" ;   #  initialize and display
      $cv.$name configure -relief groove -canvasAnchor $where
      $cv.$name setRecName $name ;  # Set data record's name
   }
   
   $cv create window $par(X) $par(Y) -window $cv.$name \
              -anchor $where -tags "$type $name widget"    

   setWidget         $name $type $par(X) $par(Y) 1 0 0 0 0 "" 0 ;
   saveWidget        $name $i_clAnchor $where
   panSetWidgetBind  $cv $name $type
   return 1
}


#----------------------------------------------------------------------------
# PROCEDURE : createFreeText - Creates a 'freeText' widget
#----------------------------------------------------------------------------

proc createFreeText { name cv } {
   global par panSelectedFont freeTextFonts
   
   if {[info exists panSelectedFont] == 0} {
      set panSelectedFont $par(FREETEXT_FONT) 
   }

   if {[lsearch $freeTextFonts $panSelectedFont] == -1} {
      set panSelectedFont $par(FREETEXT_FONT)
   }

   $cv dtag activetext activetext;    # Delete tag 'activetext' from
                                      # previous selected freeText     
           
   set id [$cv create text $par(X) $par(Y) -anchor nw -fill $par(FREETEXT_FG)\
	   -tags "text $name activetext item "  -font $panSelectedFont]
                                     
   $cv   icursor $id 0;  # set cursor inside freeText
   $cv   focus   $id ;   # Set keyboard focus to current freeText
   focus $cv;            # Direct keyboard events to the canvas
   return             
}
   
#
#            Initialize Widget Configuration
# 
# Configuration :               COMMON    FIELDS  
#
#                   type  x  y  status  dbVar  default  dbDef  class
#
# Configuration :  NEW  (index starts from 10)
#
# bitmap       10   file 
# UifText       9    
# freeText     12   text        0     font
# action       10   bitmap      
# pushButton   10   bitmap   
# checkButton  14     0          1      0    dbOff   dbOn
# radioButton  14     0          0      0    dbOff   dbOn
# output        9  
# outputString  9     
# entry        16   dataType   min    max   dbMin   dbMax   Type   Range
# scale        14              min    max   dbMin   dbMax  
# barChart     14              min    max   dbMin   dbMax  
# UifClass      9        
# UifFrame      9   ReSize
# UifListBox    9   list

#------------------------------------------------------------------------------
# PROCEDURE : initWidget - Create and initialize the data structures describing
#                          the widget
#------------------------------------------------------------------------------

proc initWidget { type  name cv {optParam { }} } {
   global  par myBitmap id i_default i_var

   #
   # Set Widget Data Structures. Each widget has a common part including
   # the following parameters :  type,    posX,   posY,  status, dbvar
   #                             default, dbDef            
   #

   set pos [$cv coords $name]   
   set posx [expr int([lindex $pos 0]) ]
   set posy [expr int([lindex $pos 1]) ]       

   set optList [panGetOptionList $type]

   # Set Common Fields
 
   setWidget $name $type $posx $posy 1 0 0 0 0 "" 0 ;
   
   switch $type {
     UifFrame         { appendWidget $name 0 }
     UifListBox       { appendWidget $name {} }
     UifOptionButton  { appendWidget $name {} }
     UifButton        { appendWidget $name {} }
     UifActionButton  { appendWidget $name {} }
     UifBitmap        { appendWidget $name $optParam }
     UifEntry         { appendWidget $name $par(NUMBER) \
                                     $par(RANGE_MIN) $par(RANGE_MAX) 0 0 0 0  
                        saveWidget   $name $i_default {}      
                      }
     UifCheckButton   { appendWidget $name 0 1 0 0 0 } 
     UifRadioButton   { appendWidget $name 0 0 0 0 0 }
     UifOutput        { appendWidget $name {} {} }
     UifOutputList    { appendWidget $name {} {} }
     UifOutputString  { appendWidget $name {} }
     UifScale         { appendWidget $name 0 $par(RANGE_MIN) \
                                $par(RANGE_MAX) 0 0 }

     UifBarChart      { appendWidget $name 0 $par(RANGE_MIN) \
                                $par(RANGE_MAX) 0 0 }
     default          { set do nothing }                 
   } ; # End of switch

   panSetWidgetBind  $cv $name $type ; # Set Edit Bindings 

   if {[lsearch $optList "noVar"] == -1 } {
      panSaveConfigVar  $cv $name $type "${name}Var" ; # Set default variable
   } 
   
##   displayWidget $name "initWidgets  $type"
}


