#**************************************************************************
# E.S.O. - VLT project
#    
# "@(#) $Id: dbInterface.tcl,v 5.19 2003/02/14 14:23:38 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   27/09/94  created 
# M.COMIN   15/02/95  Add formatting of output strings
# M.COMIN   07/03/95  Fixed bug in building symbolic address: now accepts
#                     view specifier and relative path notation
# M.COMIN   08/03/95  Reduce logging for polled DB variables
# M.COMIN   15/03/95  If dbread fails it is not repeated ...
# M.COMIN   24/03/95  Improved internal DB interface
# M.COMIN   08/05/95  DB is read with proper precision for reals
# M.COMIN   11/05/95  Scale widget (cursor) is set to current value
# M.COMIN   05/09/95  CWP does not have ":" at the end
# M.COMIN   06/09/95  add catch to set scale values ... PATCH SOLUTION.
#                     improved dbRead to read rel. addresses to current cwpEnv
# M.COMIN   12/09/95  Add check on exists of panDbdata in dbInputRefresh
# M.COMIN   18/10/95  Add check during polling if variable is updated with
#                     event
# M.COMIN   31/10/95  panDbGetFullAddress takes into account <alias>
# ------------------
# mcomin    02/09/96  Add panLogDbOperation to log DB operations
#
#
#************************************************************************
#   NAME
#
#   dbInterface - Set of routines to interface to CCS On-Line Database
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panLogDbOperation - 
#------------------------------------------------------------------------------

proc panLogDbOperation { what addr text {value ""} } {
   global panDEBUG

   set tmp "USER DEBUG> "
   switch $what {
      Read      { append tmp "DB READ  FAIL for $addr" }
      Write     { append tmp "DB WRITE FAIL for $addr > value = $value" }
      evtAttach { append tmp "DB EVENT ATTACH FAIL for $addr" }
      evtDetach { append tmp "DB EVENT DETACH FAIL for event $value" }
      default   { append tmp "??" }
   }

   puts "${tmp}\n Reason : $text"
   seq_logData $text ;  # Log Error
}

#------------------------------------------------------------------------------
# PROCEDURE : getDbConfig - 
#------------------------------------------------------------------------------

proc getDbConfig { name i } {
   global panDbData 
   return [lindex $panDbData($name)  $i]
}


#------------------------------------------------------------------------------
# PROCEDURE : setDbConfig - 
#------------------------------------------------------------------------------

proc setDbConfig { name i value } {
   global panDbData 
   set list $panDbData($name)
   set panDbData($name) [lreplace  $list $i $i $value ] 
}

#
# This procedure has been added for the cases when no CCS is present
#

proc panHeartBeat { shArea { pollRate {3000}} } {
   global EDITING ckDbPoll

   if {$pollRate < 1000 } { set pollRate [expr $pollRate * 1000]}

   if {[info exists EDITING] == 0}   { set EDITING 0 }
   if { $EDITING && $ckDbPoll == 0 } { return }

   if {[catch {winfo exists $shArea} ret]} { set ret 0 }

   if {$ret} { 
      $shArea heartBeat
      after $pollRate panHeartBeat $shArea $pollRate 
   }
}


#------------------------------------------------------------------------------
# PROCEDURE : dbInputRefresh - Update input field : one-shot read
#------------------------------------------------------------------------------

proc dbInputRefresh {args} {
   global par panWidget gvar panDbSymbAddr panDbData cv
   global db_var db_type i_dbVar i_type

   if {$par(CCS) == 0 } { return }

   if { [lindex $args 0] == "all" } { 
        set widgetList [panGetInputList $cv]
   } else {
        set widgetList $args
   }
          
   foreach name $widgetList {
      set newVal [dbRead $panDbSymbAddr($name)] ;  # Read DB value
      set myAddr [lindex $panDbData($name) $db_var]   
      set gvar($myAddr) $newVal
#      puts "DBG > dbInputRefresh > gvar($myAddr) $newVal"

      if {[cequal [lindex $panWidget($name) $i_type] UifScale]} {
         set wdgPath [panGetWdgPath $cv $name] ; # get the full widget name
         $wdgPath setValue $newVal
      }
      update idletasks
   }; #end foreach
}


#-----------------------------------------------------------------------------
# PROCEDURE : panGetWdgDbAddr - Return the Db attribute full name
#-----------------------------------------------------------------------------

proc panGetWdgAttribute { name myAddr } {
   global db_env panDbData panWdgEnvName cwpEnv panActiveEnv panEnvCwp

   set envName [lindex $panDbData($name) $db_env]  ;# Get complete symbolic

   if {[cequal [string index $envName 0] "\$"] } {
      set envName [uplevel #0 set [string range $envName 1 end]]
   }

   set tmpEnv [ctoken envName ":<"]

   switch  $tmpEnv {
      local   { set tmpEnv @${cwpEnv} }
      ""      { set tmpEnv @${cwpEnv} }
      default { set tmpEnv @${tmpEnv} }
   }

   set panWdgEnvName($name) $tmpEnv
   set panActiveEnv($tmpEnv) 1
   # Note: string range .... is used to take away @ from env name
   # specifically becausse here, seq_dbGetCwp fails on @lt1m1m3 (a DEC95 LCU)
   # but succeeds with lt1m1m3 ... and all the others we tried.
   # 
   if { [ catch {set cwp [seq_dbGetCwp [string range $tmpEnv 1 end] ]} ] } {
       set panEnvCwp($tmpEnv) :

   } else {
       set panEnvCwp($tmpEnv) $cwp
   }
   #  puts "DBG> panGetWdgAttribute $name    $envName  $panWdgEnvName($name)" 
   return [panDbGetFullAddress $panWdgEnvName($name) $myAddr]
}

#-----------------------------------------------------------------------------
# PROCEDURE : panDbGetFullAddress - Return the Db attribute full name
#
# The symbolic address includes the env. name
# Extract the attribute name and append that name to the new environment name.
# The results below assume the local RTAPENV is set to wte67
#  Test cases :
#
#   envName  |  myAddr       | output string
#   ----------------------------------
#    ""        :a:b.c          @wte67:a:b.c
#    ""        <alias>b.c      @wte67<alias>b.c
#    ""        b.c             @wte67<relative>b.c
#   wte49      :a:b.c          @wte49:a:b.c
#   wte49:     :a:b.c          @wte49:a:b.c
#   wte49      b.c             @wte49<relative>b.c
#   wte49:a    b.c             @wte49<relative>b.c         
#   wte49:a    <alias>b.c      @wte49<alias>b.c          
#   etc ...      
#   
#   What comes after the environment name.
#
#-----------------------------------------------------------------------------

proc panDbGetFullAddress { remoteCwp  myAddr } {
   global cwpEnv env panActiveEnv

   if {[cequal $myAddr 0] || [cequal $myAddr ""] } {return ""}

#   puts "DBG> panDbGetFullAddress $remoteCwp $myAddr"

   set char  [string index $myAddr 0]
   if {[cequal $char "@"]} { return $myAddr }

   if { [cequal $char ":"] == 0  && [cequal $char "<"] == 0 } {
      set tmp ${remoteCwp}<relative>${myAddr}
   } else {
      set tmp ${remoteCwp}${myAddr}
   }

#   puts "DBG> return symbAddr = $symbAddr    tmp = $tmp"
   return $tmp
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSetMainCwp - Set the CWP for the panel
#-----------------------------------------------------------------------------

proc panSetMainCwp { CWP {startDbUpdate {1}} } {
   global EDITING cv par cwpEnv myName env ckCCS localEnv initDone panDEBUG
   global panEnvCwp

   if {$par(CCS) == 0 || $ckCCS == 0 } { return }
   if {[set cwp [string trim $CWP " "]] == ""} { set CWP ":"}

   if {![cequal $panDEBUG dbgNONE]} { 
      puts "USER DEBUG > Set Cwp for local env. to \"$cwp\"" 
   }

   if {[string index $CWP 0] == "@" } {
      set tmp    [split  $CWP ":"]
      set cwpEnv [lindex $tmp 0]
      set cwpEnv [string trim $cwpEnv "@ "]
   } else {
      set cwpEnv $localEnv
   }

   set myCwp [string trimright $CWP ": "]
   set panEnvCwp(@$cwpEnv) $myCwp

   if {[cequal [panDbSetCwp $myCwp] "FAILURE"]} { return FAILURE }
   
#   puts "DBG > locEnv = $localEnv cwp = $myCwp  cwpEnv = $cwpEnv"

   panDbSetSymbAddr $cv ;  # Re-set symbolic names according to new CWP

   if {$startDbUpdate && $EDITING == 0 } {
      panEvtDbOperation $cv RESTART all ;# Re-start events
      panDbPoll $cv 0 1
   }

   return ""
}


#-----------------------------------------------------------------------------
# PROCEDURE : panDbSetSymbAddr - Set the symbolic address for each widget
#-----------------------------------------------------------------------------

proc panDbSetSymbAddr { cv } {
   global panDbSymbAddr panDbData db_var

   foreach name [panCvWidgetNames $cv uifType] {
      if {[info exists panDbData($name)] == 0} { continue }
      set symbAddr [panGetWdgAttribute $name [lindex $panDbData($name) $db_var]]

      if {![cequal $symbAddr ""]} { set panDbSymbAddr($name) $symbAddr }
   }  ;# end foreach ...
}
