#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: load.tcl,v 5.19 2003/02/14 14:23:32 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Add comments to the code
# M.COMIN   24/11/94  Modification to support 'parent' specification
# M.COMIN   08/03/95  Add diagnostic if loaded fileName does not have .pan
# M.COMIN   21/03/95  Add UifListBox
# M.COMIN   07/04/95  Fixed bug loading freeText : font
# M.COMIN   03/05/95  Add widget default value
# M.COMIN   04/09/95  Improved def. value for scale widget
#                     Fixed bug for anchoring point in create act. buttons
#                     and classes.
# mcomin    24/10/95  Add dialog to show errors during sourcing
# mcomin    25/10/95  Root panel for classes now takes the name of 
#                     the instance.
# mcomin    01/12/95  New implementation of loadShortHelp()
#
# mcomin    27/02/96  Porting to TK4.0
#           25/06/96  Fixed bug in determining the initial widget config.
#           07/08/96  Fixed bug in loadCanvasItems when a class is edited.
#           22/08/96  Fixed bug to get anchor from pubClass
#           26/08/96  Add check if widget exist when loading short help.
#                     Introduced to work with classContainer
#           31/03/97  Fixed buf while setting defaults ; variable is now read
#                     directly from the -variable option
#           23/04/97  Fixed bug with 'propagate' option for uifFrame
#           05/03/99  Fixed bug in panInitEnvNames
#
#************************************************************************
#
#   This files contains a set of routine called to load a panel
#


#---------------------------------------------------------------------------
# PROCEDURE :  loadPanel - Replace the current edited panel with one selected
#                          by the user
#---------------------------------------------------------------------------

proc loadPanel { {what {pan}}  {ask {y}}  } {
   global par cv bcv env EDITING mainHelp fileName filePath maxid mainGrid
   global i_type  db_env
   global ckShHelp ckMenuBar ckCCS ckCmdReply classMatch wdgConfig editButton
   global menu menuItem rootMenu sepCounter panMaxWdg auto_path
   global panClassLib panWidgetHlp panDbData panWidget panUsrLib
   global panXPos panYPos initDone namespaceList wdgConfig 
       
   set myState  normal

   if {[cequal $ask y]} {
      if { $par(SAVE_FLAG) } {
         set    msg "This Panel HAS BEEN MODIFIED \n "
         append msg "Do you want to LOAD another one ?"
         if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { 
            return 
         }
      }

      if { [cequal $what pan] } { 
         set tmp [panSelectFile . "*.pan" -subDir src]
      } else {
         set tmp [panSelectFile . "*_uifClass.tcl" -subDir src]
      }
      
      if { $tmp == "" } { update idletasks ; return }

      if {$par(CCS)} {
          panEvtDbOperation $cv DETACH all ;# Disable all events
      }

      if {[winfo exists $par(ROOT_PANEL)]} {
         bind $par(ROOT_PANEL) <Enter> ""
      }

      if {[winfo exists .panWdgInfoList]} {
         destroy .panWdgInfoList
      }

      panClassUnload ;    # Unload current class definitions

      set fileName        $tmp
      set par(EDIT_WHAT)  [file extension $tmp]

      if {$what == "pan"} {
         if { $par(EDIT_WHAT) != ".pan"} {
            $mainHelp displayText "Invalid fileName"
            return 
         } 
         
         set par(MYCLASS) "";

      } else {

         set tmp [file tail $fileName]
         if {[eval regexp {$classMatch} $tmp all class key ] == 0 } {
            puts "DBG > Invalid class Name .. $fileName" ; return
         }

         if { ![cequal $key $par(CLASS)] } { 
            puts "DBG > This is not a uifClass .." ; return 
         }
         set par(MYCLASS) $class
         set myState      disabled
      }       
   }

   eval $editButton                  configure -state $myState
   eval $wdgConfig(setDEFAULT_POS)   configure -state $myState 
   eval $wdgConfig(clearDEFAULT_POS) configure -state $myState 

   $mainHelp displayText "LOADING ... "
   catch { destroy $par(ROOT_PANEL) }  ;# Delete all panel widgets  
   panCleanData;                        # Clean all data structures 

   if { [catch {uplevel #0 source $fileName} ret] } {
       set    msg "ERROR SOURCING THE FILE !!\n\n$ret\n\n"
       append msg "PANEL COULD BE CORRUPTED !!"

       if {[panShowError .top $msg question 1 Continue Cancel]} {
          newPanel ; return
       }
   }
 
   if { $par(EDIT_WHAT) == ".pan" } {
      panDrawGrid             ;# Resume current grid setup
      panSetProcList "panUsr" ;# Force compatibility with prev. releases
      panInitProcEditor       ;# Update user's defined list of procs
      panInitUsrLibList       ;# Update user's defined list of libraries
      loadCanvasItem  $cv

      $par(mbFile) entryconfig 5 -state normal
      $par(mbFile) entryconfig 4 -state normal

   } else {
      eval panInitEnvNames $par(ENV_LIST)
      panLoadClass $class

      $par(mbFile) entryconfig 5 -state disabled
      $par(mbFile) entryconfig 4 -state disabled
   } 

   set allWdg    [panGetWdgNames $par(ROOT_PANEL) cv bcv ] 
   set panMaxWdg $maxid ;# Here for backward compatibility

   if {$maxid <= 1} {
      set panMaxWdg [llength $allWdg];   # Set max number of widget
   }
   incr panMaxWdg

   foreach name $allWdg {
      #
      # Get list of variables holding the environment name
      # 
      if {[info exists panDbData($name)]} {
          panSetEnvNames [getDbConfig $name $db_env]
      }
      #
      # Set bindings
      #
      # puts "DBG> load widget $name"
      set type [lindex $panWidget($name) $i_type]  
      switch $type {      
         bitmap           { continue } ; # Do nothing 
         UifActionButton  { panSetWidgetBind  $bcv  $name $type }    
         default          { panSetWidgetBind  $cv   $name $type }  
      }   
   }; # end foreach...

   if { $EDITING } { 
      panSetClassBind $cv ;# Set bindings for text
      panDrawGrid         ;# Show current grid
   }

   bind $cv <1>  "panSetCursorCoord %X %Y %x %y"

   # Display current default position ... if specified

   if {[info exists par(PANELX)]} { set panXPos $par(PANELX) }
   if {[info exists par(PANELY)]} { set panYPos $par(PANELY) }

   # Set binding to update panel geometry

   update idletasks
   panInstallGeoHandle $par(ROOT_PANEL)

   if { $initDone == 1} {
      dbInputRefresh all
   } else {
      set tmp  "WARNING : Initialization FAILED \n"
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
   }

   $mainHelp displayText "Panel LOADED"

   # Enable button to refresh the menubar configuration panel.

   if {[winfo exists .menuConfig]} {
      $wdgConfig(butREFRESH) configure -state normal
   }


   set par(SAVE_FLAG) 0;
}

#--------------------------------------------------------------------------
# PROCEDURE :  panLoadClass - Set working window to load the class
#--------------------------------------------------------------------------

proc panLoadClass { class } {
   global par shArea i_subClass db_env
   global panClassLib panWidgetHlp panDbData panWidget  
   #
   # Create the empty panel
   #
   set par(ROOT_PANEL) .${class}

   toplevel    $par(ROOT_PANEL)
   set top     $par(ROOT_PANEL).top;    
   set bot     $par(ROOT_PANEL).bot;
   set shArea  $bot.sh

   frame $top ;  frame $bot ;  frame $top.wdg ; frame $bot.actBut;

   # The short help is created but not packed in order to avoid the automatic
   # re-size of the class during editing

   panCreateShArea $shArea ; # Define ShortHelp Area

   eval "${class}_$par(CLASS) $top.wdg.${class}" ; # Create instance of class
   eval "$top.wdg.${class}  display " ;

   pack $top               -fill both  -expand 1
   pack $top.wdg           -fill both  -expand 1
   pack $top.wdg.${class}  -fill both  -expand 1 ;   # Pack the whole class

   #
   # Set proper data structures
   #
   set allWdg [panGetWdgNames $par(ROOT_PANEL) cv bcv ] 

   foreach myWdg $allWdg {

      set   name              ${class}.${myWdg}        
      set   panWidget($myWdg) $panWidget($name)
      unset panWidget($name)

      # Convert first layer class widgets into 'normal' widgets

      saveWidget $myWdg $i_subClass 0 ;

      if {[info exists panDbData($name)]} {
          set panDbData($myWdg) $panDbData($name)
          unset panDbData($name)
          panSetEnvNames [getDbConfig $myWdg $db_env]
      }

      if {[info exists panWidgetHlp($name)]} {
          set panWidgetHlp($myWdg) $panWidgetHlp($name)
          unset panWidgetHlp($name)
      }        
   }; # end foreach ...

   panDbSetSymbAddr $cv        ;# Complete DB addresses 

   loadCanvasItem  $cv $class 
   loadShortHelp   $shArea $cv $allWdg ;# Load Help Text

   wm resizable $par(ROOT_PANEL) 1 1;     # Allows th canvas to be resized
   wm title     $par(ROOT_PANEL) "Edit Class : $class"
}

#
# Get list of variables holding the environment name
# 

proc panSetEnvNames { envName } {
   global par

   if {[cequal [string index $envName 0] "\$"]} {
      set tmp [string range $envName 1 end]
       
      if {[lsearch $par(ENV_LIST) $tmp] == -1} {
         lappend par(ENV_LIST) $tmp  ;# Add new env. name to the list
      }
   }
}


#---------------------------------------------------------------------------
# PROCEDURE :  panCleanData - Clears the data structutes describing a widget
#---------------------------------------------------------------------------

proc panCleanData { } {
   global cv bcv par maxid gvar par namespaceList panWdgEnvName panEnvCwp
   global panWidgetHlp panWidget panDbData panDbSymbAddr panClassPath
   global menu menuItem menuHelp rootMenu fileName ckCCS 
   global panUsrProcs  panUsrLib panXPos panYPos oldWidth oldHeight
   global tcl_version

   #
   #  Clean previous data structures
   #  
   
   if {[info exists namespaceList]} {
      foreach item $namespaceList {
         if {$tcl_version >= 8.0} {
             catch {namespace delete $item}
         } else {
             catch {delete namespace $item}
         }
      }
      set namespaceList {}
   }

   #
   #  Clean previous data structures
   #

   set maxid 1
   set panUsrProcs   {}  ;# List of panel's procedures  
   set panUsrLib     {}  ;# List of user's libraries
   set par(ENV_LIST) {}  ;# List of env. variables

   if {[info exists  panEnvCwp]}      { unset panEnvCwp}
   if {[info exists  panWdgEnvName]}  { unset panWdgEnvName}
   if {[info exists  gvar]}           { unset gvar }
   if {[info exists  panWidgetHlp]}   { unset panWidgetHlp }
   if {[info exists  panWidget]}      { unset panWidget } 

   if {[info exists  menu]}           { unset menu }
   if {[info exists  menuItem]}       { unset menuItem }
   if {[info exists  menuHelp]}       { unset menuHelp }
   if {[info exists  rootMenu]}       { unset rootMenu }

   if {[info exists  panDbData]}      { unset panDbData  }
   if {[info exists  panDbSymbAddr]}  { unset panDbSymbAddr }
   if {[info exists  panClassPath]}   { unset panClassPath }

   set oldFileName $fileName
   set oldCkCCS    $ckCCS

   setPrivateVar; setPublicVar;  

   set ckCCS     $oldCkCCS
   set fileName  $oldFileName
   set panXPos   ""
   set panYPos   ""

   set par(PANEL_PROC_NAME) ""
   catch { unset oldWidth oldHeight }
}

#---------------------------------------------------------------------------
# PROCEDURE : panSetWdgStatus - Set Widget's initial status
#---------------------------------------------------------------------------

proc panSetWdgStatus { cv name type } {
   global i_status gvar panWidget

   if {! [cequal $type UifButton] && ! [cequal $type UifActionButton]} { 
      return 
   }

   # Check if 'name' comes from a class

   if {[cequal [set tmp [file extension $name]] ""]} {
       set  w ${cv}.${name}
   } else {
       set  w ${cv}${tmp}
   }

   # Get variable name

   set var [$w cget -stateVar]

   if {[cequal $var ""]} {
      set val [lindex $panWidget($name) $i_status]  
   } else {
      if {[uplevel #0 info exists $var]} {
         set val [uplevel #0 set $var]
      } else {
         set val [lindex $panWidget($name) $i_status]
         uplevel #0 set $var $val
      }
   }

   if {$val != 0} {
      $w enable
   } else {
      $w disable
   }
}

#---------------------------------------------------------------------------
# PROCEDURE : panLoadWidget -  Creates the Widgets in the positions 
#                              specified in array 'panWidget'
#---------------------------------------------------------------------------

proc panLoadWidget { className  shArea parent cv  { bcv {}} } {
   global par gvar maxid EDITING panWidget panDEBUG panDbData cwpEnv
   
   global i_type      i_posX     i_posY    i_status  i_dbVar \
          i_dbOff     i_ckType   i_ckRange i_text    i_var     i_clAnchor \
          i_dataType  i_dbOn     i_min     i_max     i_orient  i_bitmap \
          i_stateVar  i_subClass i_list    i_font    i_reSize  i_default \
          db_event
   
   if {[info exists EDITING ] == 0 } { set EDITING 0 }

#    puts "     --- panLoadWidget ---   \n"
#    puts "DBG > className=$className, parent=$parent, cv=$cv, bcv=$bcv"
#   puts "New : cv = $cv, bcv = $bcv"
#   puts "panWidget array : [array names panWidget]"

   set bgVal [lindex [$cv configure -background] 4]

   if {[cequal $bcv ""]} {
      set allWdg [panGetWdgList $cv ]
   } else {
      set allWdg [concat [panGetWdgList $cv] [panGetWdgList $bcv]]
   }

   foreach myWdg $allWdg {
     if {$className == 0 } {
         set name $myWdg
     } else {
	 set name ${className}.${myWdg}
     }
      
     set type [lindex $panWidget($name) $i_type]

     switch $type {
        UifClass  {   
                 # Create the canvas containing the sub-panel      
                 $cv create window [lindex $panWidget($name) $i_posX] \
                     [lindex $panWidget($name) $i_posY] -window $cv.$myWdg \
                     -anchor [$cv.$myWdg cget -canvasAnchor] \
                      -tags "$type $myWdg class"

                  $cv.$myWdg configure -helparea $shArea
                  $cv.$myWdg setCwpEnv @$cwpEnv
                  $cv.$myWdg display
                  continue
        }
        UifActionButton  {
                 $bcv create window [lindex $panWidget($name) $i_posX] \
                      [lindex $panWidget($name) $i_posY] -window $bcv.$myWdg \
                      -anchor [$bcv.$myWdg cget -canvasAnchor] \
                      -tags "$type $myWdg widget" 

                 $bcv addtag [winfo class $bcv.$myWdg] withtag $myWdg
                 loadBitmap $bcv $myWdg $name
                 panSetWdgStatus $bcv $name $type 
                 continue
        }
        PubClass  {
           if {[catch {set myAnchor [$cv.$myWdg cget -canvasAnchor]}]} {
              set myAnchor nw
           }
        }
        default {
           set myAnchor [$cv.$myWdg cget -canvasAnchor]
           $cv.$myWdg setRecName $name
        }
     }; # End of switch 
       
     ###########################################
     #       Processing of 'Normal' widgets
     ###########################################            

     $cv create window [lindex $panWidget($name) $i_posX] \
                       [lindex $panWidget($name) $i_posY] \
                       -window $parent.$myWdg -anchor $myAnchor \
                       -tags "$type $myWdg  widget"
     switch $type {
        UifButton   -
        UifBitmap   { loadBitmap $parent $myWdg $name }
        UifBarChart { $parent.$myWdg configure -barColor $par(BARCOLOR) }
        UifFrame    {
                    pack propagate [$parent.$myWdg component uifFrame] \
                                   [lindex $panWidget($name) $i_reSize]
                    }
        default     { set do nothing }
     }; # end switch

     panSetWdgStatus $cv $name $type ; # Set initial widget status

     #
     # Set widget variable to default value for those not connected to
     # Database values
     #

     if {[lindex $panWidget($name) $i_dbVar ] != 0}          { continue }
     if { [catch {set myVar [$cv.$myWdg cget -variable]} ] } { continue }
     if {[cequal $myVar ""]}                                 { continue }

     set defValue [lindex $panWidget($name) $i_default]

     switch $type {

        UifCheckButton - UifRadioButton - UifEntry {
	   set tmp [uplevel #0 set [list ${myVar}]]
	   if {![cequal $tmp ""]} { set defValue $tmp }
           uplevel #0 set [list ${myVar}] [list ${defValue}]
        }

        UifScale - UifOptionButton - UifListBox {
	    if {[eval info exists [list ${myVar}]]} {
		set defValue [uplevel #0 set [list ${myVar}]]
           }

           uplevel #0 set [list $myVar] [list ${defValue}]

          # For some widgets explicitly set current value.
    
          if {[cequal $type UifScale] || [cequal $type UifOptionButton]} {
              $parent.$myWdg setValue $defValue 
          }
        }
        default { continue }
     } ;# End of switch

#     puts "DBG> $type i_default = [lindex $panWidget($name) $i_default]"
#     puts "DBG> myVar = $myVar defValue = $defValue \n"

  }; #end foreach ...
}


#-----------------------------------------------------------------------------
# PROCEDURE : loadCanvasItems - Load canvas item such as free text, etc ..
#-----------------------------------------------------------------------------

proc loadCanvasItem { cv {class {}}  } {
   foreach item [$cv find withtag text] {        
      set name  [lindex [lindex [$cv itemconfigure $item -tags ] 4]  1]
      panSetWidgetBind  $cv $name freeText
   }
}


#-----------------------------------------------------------------------------
# PROCEDURE : loadBitmap - The name of the bitmap-file name is re-calculated 
#                          to ensure portability
#-----------------------------------------------------------------------------

proc loadBitmap { cv wdg name } {
   global i_bitmap panWidget
   set myName [lindex $panWidget($name) $i_bitmap]

   if {[cequal $myName ""] || $myName == 0 } { return }

   if { [set file [findVLTFile bitmaps/$myName]] == "" } { 
      puts "loadBitmap ($wdg) : File bitmap/$myName NOT FOUND"
      return 
   }
   $cv.$wdg  configure -bitmap "@$file"
}


#-----------------------------------------------------------------------------
# PROCEDURE : loadShortHelp - Sets the short help as described by array
#                             'panWidgetHlp'
#-----------------------------------------------------------------------------

proc loadShortHelp { shArea root wdgList } {
  global i_type panWidgetHlp par panDEBUG panWidget

  panGetCvNames $par(ROOT_PANEL) cv bcv

  # There are VLT mega-widgets which do not start with 'uif', therefore
  # we need to select 'all' widgets.

#  puts "DBG > loadShortHelp .. $shArea $root  wdgList=$wdgList"

  foreach name $wdgList {
     if {[info exists panWidgetHlp($name)] == 0 } { continue }

     if {[cequal [lindex $panWidget($name) $i_type] UifActionButton] } {
        set wdgPath $bcv.$name
     } else {
        set wdgPath [panGetWdgPath $root $name]
     }

     if {[winfo exists $wdgPath]} {
#       puts "DBG > panLoadShortHelp $wdgPath - $panWidgetHlp($name)"
       $wdgPath configure -helparea $shArea -helptext $panWidgetHlp($name)
       continue
     }
  }; # end foreach ..
}  


#-----------------------------------------------------------------------------
# PROCEDURE : panGetWdgNames - Returns a list of widget name under a given
#                              root and the corresponding canvas names
#-----------------------------------------------------------------------------

proc panGetWdgNames { root cv bcv} {
  upvar $cv   cv1
  upvar $bcv  bcv1

  set myList {}
  panGetCvNames $root cv1 bcv1 ;# Get canvas names according to the root

  #
  # Get list of widget names from top canvas
  #
  set rootLen [string length $cv1] ; incr rootLen
  foreach item [winfo children $cv1 ] {
     set wdgName [string range $item $rootLen end]
     lappend myList $wdgName
  }

  #
  # Get list of widget names from bottom canvas
  #
  set rootLen [string length $bcv1] ; incr rootLen
  foreach item [winfo children $bcv1 ] {
     lappend myList [string range $item $rootLen end]
  }
  return $myList
}

#-----------------------------------------------------------------------------
# PROCEDURE : panGetWdgList - Returns a list of widget name under a given
#                             root and the corresponding canvas names
#-----------------------------------------------------------------------------

proc panGetWdgList { root } {
  set rootLen [string length $root] ; incr rootLen
  set myList {}
  foreach item [winfo children $root] {
     lappend myList [string range $item $rootLen end]
  }
  return $myList
}

#-----------------------------------------------------------------------------
# PROCEDURE : panInitEnvNames - Initialize values for env. names
#
#  These values can be overwritten from command line or specified in 
#  the <xxx>GlobalData procedure.
#
#  The variable specifying the env. name has the format of a current
#  working point -e.g.- wsTat =  @wte641:PARAMS:SCALARS
#                                or wte492
#
#-----------------------------------------------------------------------------

proc panInitEnvNames { args } {
   global par cwpEnv panDEBUG panEnvCwp

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panInitEnvNames  $args"
   }

   if {$par(CCS) == 0 } { return }

   set currCwp [seq_dbGetCwp]
   set tmpEnvList ""
#   puts "panInitEnvNames     cwpEnv $cwpEnv   currCwp $currCwp"

   foreach name $args {

      lappend tmpEnvList $name
#      puts "name = $name"
      if {[uplevel #0 info exists $name]} {

#         puts "                exist definition ..."
         set envDef [uplevel #0 set $name]

	 if {[cequal $envDef ""] || [cequal $envDef local] } {

            # Set environment name to the local one.
            # The definition uses the local current working point.

	     uplevel #0 set $name ${cwpEnv}${currCwp} ;# Set to local env. name
       
         } else {

            # Separate env. name from cwp definition
            # wte134:PARAMS  becomes   wte134  :PARAMS

            set envName [ctoken envDef ":<"] ;# Extract env. Name
            set newCwp $envDef               ;# the rest is the cwp

            # If path not defined, set it to DB root point
	    if {[cequal $newCwp ""]} { set newCwp $currCwp }

            if { ![cequal [string index $envName 0] @] } {
	       set envName @$envName
            }

            if {[cequal $panDEBUG dbgDEV]} {
              puts "DEV. DEBUG > Env. ${envName} set CWP to ${newCwp}"
            }

            panDbSetCwp ${envName}${newCwp}
            set panEnvCwp($envName) ${newCwp}
#            puts "panEnvCwp($envName) $panEnvCwp($envName)"
	 }
         
      } else {
         # No definition exists, therefore set environment name to the 
         # local one and use the local current working point.

         uplevel #0 set $name ${cwpEnv}${currCwp}   ;# Set to local env. name
      }

#      puts "\n New definition name = [uplevel #0 set $name]\n\n"
   }


   set par(ENV_LIST) [lrmdups $tmpEnvList]

}
