#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: loadMenuBar.tcl,v 5.19 2003/02/14 14:23:38 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   05/07/94  created 
#           17/02/97  re-written
#
#


#----------------------------------------------------------------------------
# PROCEDURE : panLoadMenuBar - Creates the MenuBar
#----------------------------------------------------------------------------

proc panLoadMenuBar { {shArea {}} } {
   global ckMenuBar menuItem mbArea menuHelp menuBar menu 
   global m_name m_help
   
   if { $ckMenuBar == 0} {return} 
  
   if {[info exists menuItem]} {
      panLoadOldMenuBar
   }

   panSetMenuBind $mbArea

   if {[winfo exists $shArea] == 0 } { return }

   # Load Short-Help for MenuButtons

   foreach i [array names menu] {
      if {[set tmp [lindex $menu($i) $m_help]] != "" } {
         set name [lindex $menu($i) $m_name]
         bindMenubuttonHelp $menuBar.$name $tmp $shArea
      }
   }

   # Load Short-Help for cascade menus : definitions of other panels
   # are skipped

   foreach w [array names menuHelp] {
      if {[string match ${menuBar}* $w] } { 
         bindMenuHelp $w $shArea
      }
   }
}

# These procedures are used to display a 'short-help' text when the cursor
# moves into a Menubutton or enters an element of the cascade menu

proc bindMenubuttonHelp { w text shArea } {
   if {$text == ""} {
      bind $w <Any-Enter> ""
      bind $w <Any-Leave> ""
   } else {
      bind $w <Any-Enter> "$shArea displayText \"$text\""
      bind $w <Any-Leave> "[code $shArea hide]"
   }
}

proc bindMenuHelp { w shArea } {
   bind $w <Motion>    "panShowMenuHelp %W %y  $shArea" 
   bind $w <Any-Leave> "[code $shArea hide]"
}

# For cascade menus the help text is stored into the auxiliary data 
# structure 'menuHelp'. This is an array, each element containing the list
# of all help texts for the associated menu entries.

proc panShowMenuHelp { w  ycoord shArea } {
   global menuHelp

   if {[info exists menuHelp($w)]} {
      set idx [$w index @$ycoord]  ;# Translate cursor position to entry index

      # If idx = none the current cursor position is no longer
      # on the menu ...

      if {![cequal $idx none]} {
         $shArea displayText [lindex $menuHelp($w) $idx]
      }
   }
}


proc panSetMenuBind { mbArea } {
   global par EDITING

   if { $EDITING == 0 } { return }

   bind $mbArea.menu <3>  "createMenuPanel"   
      
   # Disable Quit menu button during editing
        
   set tmp $par(ROOT_PANEL).topMenu.menu.file.menu
      
   if {[set idx [tkMenuFindName $tmp Quit]] != "" } {
         $tmp entryconfigure $idx -state disabled
    }
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#         Here only for backwards compatibility
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


proc panLoadOldMenuBar { } {
   global par menu rootMenu menuBar menuItem menuHelp EDITING
 
   #
   # Create Menu Bar Structure
   #

   foreach name $rootMenu {
      set menuList ""
      set level 0
      configMenu "create"  $menuBar  $name  
      set menuList [getSubMenu $name]                  
      makeSubMenu  $name  $name  $menuList  $level ;# Create sub-Menu 

   }; # end foreach (rootMenu)

   # Convert old structured into new ones

   set i 0;
   unset menu menuItem
   
   if {[info exist menuHelp]} { unset menuHelp }

   foreach name $rootMenu {
      # Just in case that the name starts with capital letters
      set newName [string tolower $name]

      set w $menuBar.$newName
      set menu($i) [list [$w cget -text] [$w cget -underline] $newName "" ]
      incr i
   }
}  

#------------------------------------------------------------------------------
# PROCEDURE : makeSubMenu - Creates sub-menu recursively
#------------------------------------------------------------------------------

proc makeSubMenu { parentName path  myList  level } {
   global menu m_type m_label m_uline m_parent m_sub m_cmd m_var m_val

   if { [llength $myList] == 0 } { return }
    
   #
   # For each element
   # - delete 'parentMenu' string from the name
   # - make direct sub-menu and ignore sub-menu elements from the list
   # - Make the sub-menu for all immediate menus
 
   set oldPath $path ; incr level  
 
#   puts "DBG> myList = $myList"
      
   foreach name $myList { 
      set path  $oldPath  
      regsub "$parentName"  $name "u " newName ;     # Strip 'parentName' off
      set newName [string trim [lindex $newName 1] " ."]
      #
      # Check if the menu does not belong to this level or the name is empty
      # 
      if {[string match "*.*" $newName] == 1} { continue };  # Get next     
      if { $newName == "" }                   { continue };  # Get next

      if { [info exists menu($name)] } {
           #
           # Create new sub-menu and its sub-menus
           #       
           configSubMenu "create" $path $name      
                  
           set menuList  [getSubMenu $name]                  
           set newPath   [string trim "$path.$newName"  " ."] 
           makeSubMenu   $newName  $newPath  $menuList  $level
      } else {      
           configMenuItem "create" $name
      }
        
  } ; # end foreach ...        
}   
   

##############################################################
#
#          Set of routines to create/edit menus
#
##############################################################

#------------------------------------------------------------------------------
# PROCEDURE : getSubMenu - Get List of submenu
#------------------------------------------------------------------------------

proc getSubMenu { name } {
    global menu  m_sub

    set subList    ""
    set listNames  [lindex $menu($name) $m_sub]

    foreach item $listNames {
       lappend subList "$name.$item"
    }  
    return $subList
}


#------------------------------------------------------------------------------
# PROCEDURE : configMenuItem - Configure a Menu
#------------------------------------------------------------------------------

proc configMenuItem { mode name } {
    global menu  menuItem
    global m_type  m_label  m_uline  m_parent  m_sub  m_cmd  m_var  m_val

    set list {}
    set list $menuItem($name)
    #
    # Extract Parent Name
    #
    
    set parent   [getParentMenu $name]
    set type     [lindex $list $m_type]
    set menuPath [lindex $menu($parent)  $m_parent]

    switch $mode {

      "create" {
          
               set label [lindex $list $m_label] 
               set uline [lindex $list $m_uline] ; incr uline -1
               set cmd   [lindex $list $m_cmd] 

##               puts "DBG > configMenuItem $name $type"
##               puts "DBG > $label $uline $cmd"
              
               switch $type {
               
                  "cmd" { addMenuCmd $menuPath $label $uline $cmd }
                  "sep" { $menuPath  add  separator }
                  
                  "ck"  { 
                        set myVar  [lindex $list $m_var]
                        addMenuCk  $menuPath $label $uline $cmd  $myVar     
                        }
                        
                  "rb"  { 
                        set myVar  [lindex $list $m_var]
                        set myVal  [lindex $list $m_val]
                        addMenuRb  $menuPath $label $uline $cmd $myVar $myVal
                        }                  
                  
                  }; # end switch type              
              }    
     }
    return $menuPath
}

#------------------------------------------------------------------------------
# PROCEDURE : configMenu - Configure a Menu
#------------------------------------------------------------------------------

proc configMenu { mode parent name } {
    global m_label m_uline m_parent menu

    switch $mode {
    
      "create"  {   
           
            set label [lindex $menu($name) $m_label]     
            set uline [lindex $menu($name) $m_uline]

            if { $label == "Help" } {
                set menuPath  [addMenu $parent $label $uline "right"]   
	    } else {
                set menuPath  [addMenu $parent $label $uline "left"]  
	    }
            
            saveMenuInfo  $name $m_parent $menuPath
            
            } 
                       
    } ; # end switch            
            
    return $menuPath
}


#------------------------------------------------------------------------------
# PROCEDURE : configSubMenu - Configure a sub-Menu
#------------------------------------------------------------------------------

proc configSubMenu { mode parent name } {
    global m_label m_uline m_parent menu

    switch $mode {
      "create"  {
         
          set label  [lindex $menu($name) $m_label]     
          set uline  [lindex $menu($name) $m_uline]
          set myPath [lindex $menu($parent) $m_parent]
                                                    
          set menuPath [ addSubMenu $myPath $label $uline]

          set list         $menu($name)  
          set menu($name)  [lreplace  $list $m_parent $m_parent $menuPath ] 
            
          } 
                       
    } ; # end switch            
            
    return $menuPath
}

#------------------------------------------------------------------------------
# PROCEDURE : saveMenuInfo - Save Menu configuration Parameter
#------------------------------------------------------------------------------

proc saveMenuInfo { name i value } {
   global menu menuButton
      
   set nameMenu [ string trim $name " ."]

   if { [info exist menu($nameMenu) ] } {

      set list         $menu($name)  
      set menu($name)  [lreplace  $list $i $i $value ] 

   } elseif { [info exist menuButton($nameMenu)] } {

      set list               $menuButton($name)  
      set menuButton($name)  [lreplace  $list $i $i $value ] 
   }   
      
   return 
}


#------------------------------------------------------------------------------
# PROCEDURE : getParentMenu - Returns the name of the parent Menu
#------------------------------------------------------------------------------

proc getParentMenu { name } {
   global menu
     
   regsub -all {\.} $name { } bufChar ;         # Convert '.' in spaces

   set len     [ expr [llength $bufChar] - 1];  # Get index to last token 
   set bufChar [lreplace $bufChar $len $len] ;  # delete last token

   regsub -all { } $bufChar "." parentMenu ;    # Convert spaces in '.'
    
   return $parentMenu
}


#-----------------------------------------------------------------------------
# PROCEDURE : getMenuInfo - Get Menu Configuration Parameters
#-----------------------------------------------------------------------------

proc getMenuInfo { name index } {
   global menu
   return [lindex $menu($name) $index]
}
