#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panCallPanel.tcl,v 5.19 2003/02/14 14:23:44 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    11/05/95  created
#           21/04/98  Add dubugging information
#

#************************************************************************
#   NAME
#   
#   panCallPanel - Call a panel from an independent interpreter.
#
#   SYNOPSIS
#   
#   panCallPanel  <panelName> ?parameter_list?
#
#   DESCRIPTION
#   
#   This procedure allows the user to call a panel and pass some parameters
#   in the command line.
#
#     <panelName>        Name of the panel. The name is the same of the file 
#                        produced by the panel editor, without the ".pan" 
#                        extension
#                        The panel to be invoked must be present in the "bin" 
#                        sub-directory of the following root directories :
#                        current module , INTROOT or VLTROOT.
#
#     ?parameter_list?   List of parameters to be used in the command line.
#                        An item of this list has the following format :
#                                      <var>=<value>
#                        Where :
#                        <var>    Specifies a global variable
#                        <value>  The new value
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   None
#
#   CAUTIONS
#
#   EXAMPLES
#
#   Simple example :
#
#     panCallPanel myPanel "cwp=@wte13:PARAMS" "gvar(ccdName)=ccd"
#
#   Sometimes the list of parameters can be long or it is more convenient
#   to build it up in a list variable :
#
#     set myParList {}
#     lappend myParList "cwp=@wte13:PARAMS"
#     lappend myParList "gvar(ccdName)=ccd"
#     lappend myParList "pollRate=2000"
#   
#     panCallPanel myPanel $myParList 
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panCallPanel { fileName args } {
   global panDEBUG

   if {[string compare $fileName @] == 0} { 
      set tmp "Error: Substitute symbol \"@\" with the filename"  
      tk_dialog .dialog "Dialog Box" $tmp error 0 OK
      return
   }

   if { [set filePath [findVLTFile bin/$fileName]] == ""} { return }

   set myExec "exec $filePath "

   set lenArg [llength $args]

   if {$lenArg == 1} {
      set myParList [split [string trim $args " { }"]]
      foreach item $myParList {
         append myExec " \"$item\" "
      }
   } else {
   
      foreach item $args {
         append myExec " \"$item\" "
      }
   }

   if {[info exists panDEBUG] && ![cequal $panDEBUG dbgNONE] } {
      puts "USER DEBUG >  $myExec"
   }

   eval $myExec &
}


#
# ___oOo___
