#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panClosePanel.tcl,v 5.19 2003/02/14 14:23:37 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  04/06/95    created
# mcomin  20/10/95    Disable event mechanism before quitting 
# ------------------
# mcomin  07/08/96    Fixed bug when 'destroy' is called
#         24/04/97    Toplevel root is automatically recognized
#                     Execute seq_ccsInit only if exiting the main panel
#         26/04/98    Cancel polling with 'after cancel $id' 
#

#************************************************************************
#   NAME
#     panClosePanel, panIconifyPanel  - Close and Iconify a panel
#
#   SYNOPSIS
#
#     panClosePanel    <askFlag>  ?root?
#
#     panIconifyPanel  <askFlag>  ?root?
#
#   DESCRIPTION
#
#   All these routines take the same parameters
#
#     <askFlag>    If this is set to 'confirm', the user must aknowledge
#                  before the panel is 'closed'. 
#
#     ?root?       Name of the root window : by default it is set to "."
#                  Panels generated with the panel editor have the name of
#                  the file (without extension) as root window.
#                  Example : xyz.pan have ".xyz" as root window.
#
#   panClosePanel
#      This procedure close and destroy a panel and exits the interpreter.  
#      It also executes the exit procedure possibly associated with the panel.
#
#   panIconifyPanel 
#      Close and iconify a panel. 
#      (It is internally used to process panels called with panImportPanel)
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#    # Close panel with root window = ".myRoot" and ask user confirmation
#    # before proceeding.
#
#    panClosePanel confirm ".myRoot"
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#----------------------------------------------------------------------------
# PROCEDURE : panClosePanel -  Quits the panel
#----------------------------------------------------------------------------

proc panClosePanel { {ask {continue}} {root {.}} } {
   global cv EDITING seq_ccsStatus par panDEBUG

   # Get immediately the current value for cv before the cursor moves away
   # from the current panel.

   if {[info exists cv]} { set tmpCv $cv }

   if {$EDITING } {
     set msg " You Cannot 'QUIT' the panel\n during editing !!"
     tk_dialog .dialog "Dialog Box" $msg info 0 OK
     return
   }

   if {[cequal $ask "confirm"] } {
      set msg "Do you want to QUIT the panel?"
      if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { return }
   }

   # Stop Polling

   if {[info exists par(pollId)]} {
      after cancel $par(pollId)
   }

   if {[info exists tmpCv] && [winfo exists $tmpCv] } {

      if {$root == "."} { 
	 catch {[winfo toplevel $tmpCv] root }
      } else {
         panGetCvNames $root tmpCv tmpBcv
      }

      if {[winfo exists $root] == 0 } {
         set msg "Panel or root window\n '$root'\n DOES NOT EXIST"
         tk_dialog .dialog "Dialog Box" $msg info 0 OK
         return
      }

      if {[cequal $panDEBUG dbgDEV]} {
          puts "DEV. DEBUG > close panel $tmpCv"
      }

      panEvtDbOperation $tmpCv DETACH all ;# Disable main panel events

      foreach child  [winfo children $root] {
         set  myRoot  [string trim $child "."]
         set  childCV "${child}.top.wdg.can"

         panEvtDbOperation $childCV DETACH all ;# Disable child events
         update

         if { [info procs ${myRoot}Exit] != "" } {
           eval ${myRoot}Exit;        # Exec 'Exit' procedure
         }
         destroy $child
      }
   }

   # Stop Polling

   # Exit the seqWish only if this is the 'main' panel

   if { $root == "." && [info exists seq_ccsStatus]} {
      seq_ccsExit
   }

   if {[info exists tmpCv] && [winfo exists $tmpCv]} { update }
   destroy $root
}

#----------------------------------------------------------------------------
# PROCEDURE : panIconifyPanel - Iconifies a panel
#----------------------------------------------------------------------------

proc panIconifyPanel { {ask {no}} {root {.}} } {
   global EDITING seq_ccsStatus par cv

   if { $ask == "confirm" } {
      set msg "Do you want to QUIT the panel?"
      if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { return }
   } 

##   puts "DBG > panCloseImportPanel  $root , [winfo children .]"
 
   if {$root == "."} {
      set root [winfo toplevel $cv] 
   }

   wm iconify $root
}

#
# ___oOo___
