#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panCommonProcs.tcl,v 5.19 2003/02/14 14:23:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    11/01/96  created by merging different small files
#
# mcomin    27/02/96  Porting to TK4.0
#           08/10/96  Add update idletasks to force widget updating without
#                     delay in all procedures
#
#************************************************************************
#   NAME
#
#     panResetPanel, panSetDefaults, panUpdateCanvas - Panel Operations
#
#   SYNOPSIS
#
#     panResetPanel   <canvas> 
#
#     panSetDefaults  <canvas>
#
#     panUpdateCanvas <canvas>
#
#   DESCRIPTION
#
#     All these routines take as parameter the name of the panel canvas
#
#     canvas - <IN>  Name of the panel canvas : by default this is '$cv'.
#
#
#     panResetPanel :
#
#     This procedure clears all entry fields according to the
#     type of data specified in the widget definition :
#
#     - NUMBER , the entry is set to '0'
#     - ASCII  , the entry is set to "" 
#
#
#     panSetDefaults :
#
#     This procedure sets all input widgets to their default values.
#
#
#     panUpdateCanvas :
#
#     This procedure updates all panel's input widgets to the current 
#     associated database value.
#
#     
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#     none
#
#   CAUTIONS
#
#   EXAMPLES
#
#     global cv 
#
#     panResetPanel   $cv
#     panSetDefaults  $cv
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#----------------------------------------------------------------------------
# PROCEDURE : panResetPanel -  Clear all entry widgets
#----------------------------------------------------------------------------

proc panResetPanel { cv } {
   global panWidget i_type i_dataType
        
   foreach name [panWdgReferenceList $cv all uifType]  {
      if { [getWidget $name $i_type] == "UifEntry" } {
         set wdgPath  [panGetWdgPath $cv $name]
         $wdgPath erase
         
         if {[getWidget $name $i_dataType] == "N" } {
            $wdgPath insert 0 0
         }
      }      
   } ; # end foreach 
 
   update idletasks
}

#----------------------------------------------------------------------------
# PROCEDURE : panSetDefaults -  Set Default values for all input widgets
#----------------------------------------------------------------------------

proc panSetDefaults { cv } {
   global  panWidget gvar
   global  i_type  i_default
   
   foreach name [panWdgReferenceList $cv all uifType] {
           
      set type     [getWidget $name $i_type]
      set defValue [getWidget $name $i_default]
      set wdgPath  [panGetWdgPath $cv $name]
    
#      puts "DBG > panSetDefaults $name $type $wdgPath"

      switch $type {
         UifRadioButton -
         UifCheckButton {
             set var        [panParseVar [$wdgPath cget -variable]]
             set gvar($var) $defValue
         }
         
         UifOptionButton  { $wdgPath setValue $defValue }
         UifEntry         { $wdgPath  erase ; $wdgPath insert 0 $defValue }
         UifScale         { $wdgPath setValue $defValue }
      } ;# end of switch

   } ; # end foreach  

   update  ;# Force the window manager to update
            # the widgets without delay
}


#----------------------------------------------------------------------------
# PROCEDURE : panUpdateCanvas -  
#----------------------------------------------------------------------------

proc panUpdateCanvas { canvas } {
   
   global panDbData gvar cv
   global i_dbVar i_type
         
   # Update widget associated with a DB attribute
   
   dbInputRefresh all         
         
   # Update widget NOT associated with a DB attribute
       
   foreach name [panWdgReferenceList $cv all uifType] {

      if { [info exists panDbData($name)]}  { continue }

      set wdgPath  [panGetWdgPath $cv $name]   
      set type     [getWidget $name $i_type]
   
#      puts "DBG > name = $name  path = $wdgPath  type = $type"

      if {$type == "Scale" || $type == "UifScale"} {     
         set var [$wdgPath cget -variable]
#         puts "DBG > var = $var .... update to  [eval set $var]"      
         $wdgPath setValue [eval set $var]       
      } else {
         set do nothing               
      }  
   }; # end foreach ...

   update ;# Force the window manager to update
           # the widgets without delay
}

#
# ___oOo___
