
#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panCreateFrame.tcl,v 5.19 2003/02/14 14:23:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.COMIN   21/03/95  Take away binding for freeText
# M.COMIN   12/05/95  Binding are set only for EDITING
# M.COMIN   01/06/95  Moved to separate file from 'createEmptyPanel'
# mcomin    22/01/96  Declare cv as global otherwise events do not work
#                     when panel is called by another one.
#
# mcomin    27/02/96  Porting to TK4.0
# mcomin    22/04/97  panUpdateCv now computes shArea as well
#           28/08/98  Add new entries to standard menu
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   Creates an empty frame for the control panel.
#   This frame is used to build the control panel during editing.
#
#   Window hierarchy :
#
#   mbArea     $par(ROOT_PANEL).topMenu;
#   wdgArea    $par(ROOT_PANEL).top.wdg
#   cv         $wdgArea.can
#   shArea     $par(ROOT_PANEL).bot.sh
#   actButArea $par(ROOT_PANEL).bot.actBut
#   bcv        $actButArea.can
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO 
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panCreateFrame - Creates an empty panel frame
#------------------------------------------------------------------------------

proc panCreateFrame { {root 1} {xpos ""} {ypos ""} } {
   global par ckShHelp ckMenuBar EDITING panImportFlag ckCmdReply env
   global mbArea  actButArea wdgArea cwp cv bcv panDEBUG menuBar shArea
   global panClock

   if {![info exists env(PANEL_TIME)]} {
      set env(PANEL_TIME) ""
   }

   set panClock(1) [clock seconds]

   if {[info exists panImportFlag] == 0 } { set panImportFlag 0  }

   panSetAutoPath  [list libuifCommon libuif libuifClass] {}

   if {[string index $root 0] != "."} {
      set panRoot .${root}
   } else {
      set panRoot ${root}
   }

   set par(ROOT_PANEL) $panRoot
   set par(EDIT_WHAT)  ".pan"

   #
   #  Definition of names for frames in the working window
   #

   if {$root == "1" || $root == "0"} {
   
      # Backward Compatibility
      set par(ROOT_PANEL) ".panel" 
      set panRoot $par(ROOT_PANEL)
      
      if { [info procs panUsrGlobalData] != "" } {
         panUsrGlobalData ;   # Set defaults specified by the user
      }

      puts "WARNING : Very old panel definition format"
      puts "          Use Panel Editor to load and save it again"
   }

   #
   # Define names for main panel frames
   #
   panGetCvNames $panRoot cv bcv

   if {$EDITING || $panImportFlag } {
      toplevel     $panRoot
      set w  $panRoot
   } else {
      frame  $panRoot ; pack $panRoot -fill both -expand 1
      set w .
   }

   # These two statements are here for backward compatibility

   if {[info exists par(PANELX)] && ![cequal par(PANELX) ""]} {
      set xpos $par(PANELX)
   }

   if {[info exists par(PANELY)] && ![cequal par(PANELY) ""]} {
      set ypos $par(PANELY)
   }

   if {![cequal $xpos ""] && ![cequal $ypos ""]} {
      wm geometry  $w  +$xpos+$ypos
   }

   #
   # Define the main frames
   #

   set top        [frame $panRoot.top];
   set bot        [frame $panRoot.bot];
   set mbArea     [frame $panRoot.topMenu] ;# Name of the menuBar Area
   set wdgArea    [frame $top.wdg] 
   set actButArea [frame $bot.actBut]      ;# Area with action buttons

   # Name of Command reply  Area 
   set panReplyArea  [frame $panRoot.replyArea -bd 2 -relief raised]

   # Create canvases for widgets and action buttons

   canvas $cv  -width 200 -height 60 -highlightthickness 0
   canvas $bcv -width 200 -height 5  -highlightthickness 0

   bind $panRoot <Enter> "panUpdateCv %W"; # Update current Canvas Name

   #
   # Create the empty panel
   #

   pack $mbArea              -side top  -fill x 
   pack $top                 -side top  -fill both -expand 1
   pack $panReplyArea  $bot  -side top  -fill x

   pack $wdgArea $cv -fill both -expand yes
   pack $actButArea  -side bottom -fill both -expand yes     
   pack $bcv         -fill both -expand yes
   
   if {$EDITING} { 
       wm resizable $panRoot 1 1 ; # Allows th canvas to be resized
       wm title     $panRoot $par(TITLE)
       update idletasks
       panSetClassBind $cv
       bind $cv <1>  "panSetCursorCoord %X %Y %x %y"
   } else {

       if {$panImportFlag} {
	   wm resizable $panRoot 0 0 ; # Disable canvas resizing  
       } else {
	   wm resizable . 0 0 ;        # Disable canvas resizing  
       }
   }

   uplevel #0 set shArea [panGetShArea $panRoot] ;# Name of short help  Area 

   if { [info exists ckShHelp] && $ckShHelp } {
      panCreateShArea $shArea
   }

   # Define anyhow the menuBar area
   set menuBar [makeMenuFrame $mbArea]

   # Set defaults specified by the user

   set glData [string trimleft ${panRoot}GlobalData "."]

   if { [info procs $glData] != "" } { eval $glData }
}

proc panGetShArea { root } {
   if { [string index $root 0] != "." } {
      set root ".$root"
   }

   return $root.bot.sh
}


proc panUpdateCv { win } {
   global par cv bcv shArea menuBar

   # This event triggers for toplevel window and also for all 
   # its children, therefore I need to take only the first name

   set root  [lindex [split $win "."] 1]
   set par(ROOT_PANEL) .${root}

   # Patch for backward compatibility
   if {[info exists par(EDIT_WHAT)] == 0 } { set par(EDIT_WHAT) ".pan" }
   
   set cv      $par(ROOT_PANEL).top.wdg.can     ;# Canvas for widgets
   set bcv     $par(ROOT_PANEL).bot.actBut.can  ;# Canvas for act. buttons
   uplevel #0 set shArea  [panGetShArea $par(ROOT_PANEL)]  ;# Name of short help Area
   set menuBar $par(ROOT_PANEL).topMenu.menu

   if {![cequal $par(MYCLASS) ""]} {
      if {[winfo exists  $cv.$par(MYCLASS).cv]} {
         append cv ".$par(MYCLASS).cv"
      }  
   }
}


proc panCreateNewMenuBar { menuBar } {
   global menu

   # Each item specifies : label,  underline char and widget name.

   set menu(0) [list File  1 file]
   set menu(1) [list "Std. Options" 0 stdoptions]
   set menu(2) [list Help  1 help]

   #  Menu : Help

   set mb [addMenuButton $menuBar help " Help " -1 right]
   addMenuCmd $mb "Extended Help"  0  {puts "Not Implemented"}

   #  Menu : File

   set mb [addMenuButton $menuBar file " File " -1 left]
   addMenuCmd $mb Quit 0  panClosePanel

   #  Menu : Options

   set mb [addMenuButton $menuBar stdoptions " Std. Options " -1 left]

   addMenuCmd $mb "Refresh Database values"   0  {global cv; panUpdateOutput $cv}
   addMenuCmd $mb "Re-Start DataBase Events"  0  \
                  {global cv ; panEvtDbOperation $cv RESTART all }
}

proc panCreateStdOption {menuBar} {
   global menu par

   # Check if menuButton exists
   if {![cequal [panGetMbPath " Std. Options "] ""]} { return }

   incr par(SAVE_FLAG)

   set max [array size menu]

   for {set i [expr {$max - 1}]} { $i >= 1 } { incr i -1} {
       set menu([expr {$i + 1}]) $menu($i)
   }

   set menu(1) [list " Std. Options " 0 stdoptions {}]
   set mb [addMenuButton $menuBar stdoptions " Std. Options " -1 left]

   set w $menuBar.[lindex $menu(0) 2]

   addMenuCmd $mb "Refresh Database values"  0  {global cv; panUpdateOutput $cv}
   addMenuCmd $mb "Re-Start DataBase Events"  0  \
                  {global cv ; panEvtDbOperation $cv RESTART all }

   pack $menuBar.stdoptions -after $w -side left -padx 5
}

#
# ___oOo___
