#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panDbEvents.tcl,v 5.19 2003/02/14 14:23:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  11/12/96  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#-----------------------------------------------------------------------
# PROCEDURE : panEvtDbAttach -
#-----------------------------------------------------------------------

proc panEvtDbAttach { wdgPath wdgName } {
   global panDbData panWidget db_event  panDEBUG i_type db_var
   
   if {[info exists panDbData($wdgName)] == 0 } { return }
   if {[winfo exists $wdgPath] == 0}            { return }

   set evtHandle [lindex $panDbData($wdgName) $db_event]

   if {![cequal $evtHandle ""] && ![cequal $evtHandle free]} { return }

   set attr [panGetWdgAttribute $wdgName [lindex $panDbData($wdgName) $db_var]]
   
   if {[set last [string first "\(" $attr]] != -1 } {
      set attr [string range $attr 0 [expr $last -1]]
   } 

   if {[catch {eval seq_evtAttach $attr [$wdgPath cget -evtFilter] \
               "\"[$wdgPath cget -evtScript]\""} ret] } {
      panLogDbOperation evtAttach $attr $ret
      puts "          Invalid DB attribute or filter : VECTORS and TABLES use 'w' filter only"
      return
   }

   if {$panDEBUG != "dbgNONE" } {
      puts "USER DEBUG > [getWidget $wdgName $i_type] $wdgName script = {[$wdgPath cget -evtScript]}"
      puts "USER DEBUG > ATTACH EVENT ($ret) to attribute >> $attr"
   }

   setDbConfig  $wdgName $db_event $ret
   # Read current DB value and update local variable

   set newValue [panPollDbRead $wdgName [lindex $panWidget($wdgName) $i_type]]

   if {! [cequal $newValue panFAILURE]} {
       panUpdateLocalVar $wdgName [lindex $panWidget($wdgName) $i_type] $newValue
   }
}


proc panEvtDbOperation { cv mode where } {
   global panWidget panDbData i_type db_event db_poll panDEBUG
  
   if {[winfo exists $cv] == 0 } { return }

   if {! [cequal $panDEBUG dbgNONE] } {
      puts "\nUSER DEBUG > Event Operation  $mode   args = $where $cv"
   }

   foreach name [panWdgReferenceList $cv $where uifType] {
      set type [lindex $panWidget($name) $i_type]

      # Check if the widget is enabled to get DB events

      set wdgPath [panGetWdgPath $cv $name]
      if {[catch {$wdgPath cget -dbMode}] } {
         continue
      }

      if { [info exists panDbData($name)] == 0 }             { continue }
      if {[cequal [lindex $panDbData($name) $db_poll] 1]}    { continue }
      if {[cequal [lindex $panDbData($name) $db_poll] free]} { continue }

      switch $mode {
         "DETACH"  { panEvtDbDetach $name }
         "ATTACH"  { panEvtDbAttach $wdgPath $name }
         "RESTART" { panEvtDbDetach $name;
                     panEvtDbAttach $wdgPath $name }
      }
   }; # end of foreach ...
}


#----------------------------------------------------------------------------
# PROCEDURE : panEvtDbDetach -
#----------------------------------------------------------------------------

proc panEvtDbDetach { wdgName } {
   global i_type panDbData db_event db_var panDEBUG
   
   if {[info exists panDbData($wdgName)] == 0 } { return }
   set evtHandle [lindex $panDbData($wdgName) $db_event]

   if {! [cequal $panDEBUG dbgNONE] } {
      set type   [getWidget   $wdgName $i_type]
      puts "\nUSER DEBUG > $type   $wdgName "
      puts "USER DEBUG > DETACH EVENT ($evtHandle) to attribute >> [getDbConfig $wdgName $db_var]"
   }

   if { [cequal $evtHandle ""] || [cequal $evtHandle free] } { return }

   setDbConfig $wdgName $db_event "" 

   if {[catch {eval seq_evtDetach $evtHandle } ret] } {
     panLogDbOperation evtDetach [lindex $panDbData($wdgName) $db_var] $ret $evtHandle
     return
   } 
}


proc panEvtDbUpdate { wdgName attr attrType data { quality "" } } {
   global panDbSymbAddr panWidget i_type panDEBUG

   if {[info exists panWidget($wdgName)] == 0} { return }

   if {[cequal $attrType SCALAR]} {
      panUpdateLocalVar $wdgName [lindex $panWidget($wdgName) $i_type] [lindex $data 2]

      if {[cequal $panDEBUG dbgDEV]} {
          puts "DEV. DEBUG > Event Update : $attrType  $attr   Old([lindex $data 1])  New([lindex $data 2]) Values"
      }

      if { [cequal $quality ""] } { return }

      if { ![cequal [lindex $quality 1] "OK"] } {
          panSetQualityFlag $wdgName 1
      } else {
          panSetQualityFlag $wdgName 0
      }

   } elseif { [cequal $attrType VECTOR] } {

      # extract attribute specification in the format :
      #   vector(r1:r2)  r1,r2 element range

      if {[regexp {\(([0-9a-zA-Z:,]+)\)} $panDbSymbAddr($wdgName) all rec]} {
	 if { ![panCheckOnRange $rec [lindex $data 0] [lindex $data 1] ]} {
            return 
         }
      }
      set newValue [panPollDbRead $wdgName [lindex $panWidget($wdgName) $i_type]]
      if {[cequal $panDEBUG dbgDEV]} {
          puts "DEV. DEBUG > Event Update : $attrType  $attr   element = $data  New Value = $newValue"
      }

      if {[cequal $newValue panFAILURE]} { return }

      panUpdateLocalVar $wdgName [lindex $panWidget($wdgName) $i_type] $newValue

   } elseif { [cequal $attrType TABLE] } {  

      # extract attribute specification in the format :
      #  table(r1:r2,f1:f2)  r1,r2 record range  f1,f2 field range

      if {[regexp -nocase {\(([\:0-9]*)([,]*)([\:a-z0-9]*)\)} $panDbSymbAddr($wdgName) all rec tmp fields] == 0 } {
         puts "DBG> Not valid syntax for table $panDbSymbAddr($wdgName)"
         return
      }

      # Check Record ranges 

      lassign $data r0 r1 f0 f1 ;# Get indexes of current element

      # Check if record/fields are in range 

      set recOk   [panCheckOnRange $rec $r0 $r1]
      set fieldOk [panCheckOnRange $fields $f0 $f1]

      if { $recOk == 0 && $fieldOk == "0" } { return }

      set newValue [panPollDbRead $wdgName [lindex $panWidget($wdgName) $i_type]]

      if {[cequal $panDEBUG dbgDEV]} {
          puts "DEV. DEBUG > Event Update : $attrType   $attr   element = $data  New Value = $newValue"
      }

      if {[cequal $newValue panFAILURE]} { return }

      panUpdateLocalVar $wdgName [lindex $panWidget($wdgName) $i_type] $newValue
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : panCheckOnRange -
#
#   index    Index specification associated to a DB attribute
#            It can have the following formats : (2:7) or (5).
#   min,max  Range of values changed in the database as returned by the
#            event
#   The procedure returns three values :
#
#   0 - index not in range
#   1 - index in range
#   s - index is a string
#
#----------------------------------------------------------------------------

proc panCheckOnRange { index  min max } {

   if { [regexp {([0-9]+)([:]*)([0-9]*)} $index all r1 tmp r2]  == 0} {
      return s ;# Index is a string
   }

   if {[cequal $r1 ""]} { return 0 }

   if {[cequal $r2 ""]} {
      # Index has a format such (5)
      if { ($r1 >= $min)  && ($r1 <= $max) } {
          return 1
      } else {
          return 0
      }
   } else {
          
      # Index has a format such (2:7)
      if {$r1 > $max || ($r2 < $min) }  { 
          return 0 
       } else {
          return 1 
       }
   }
}


#
# ___oOo___
