#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panDbPoll.tcl,v 5.19 2003/02/14 14:23:53 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin    02/07/97    created
#           19/04/98    Polling disabled when window is iconified.
#           02/05/98    Catch format if conversion cannot be applied
# bgilli    12/10/98    Removed overwriting shArea with empty string
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panUpdateLocalVar
#------------------------------------------------------------------------------

proc panUpdateLocalVar { wdgName type newVal } {
   global gvar db_var i_outFormat panWidget panDbData

   # The new value can be optionally supplied in the call

   set myAddr [lindex $panDbData($wdgName) $db_var]

#   puts "DBG> panUpdateLocalVar db_var($myAddr) newVal($newVal)"

   if {[cequal $type UifOutput] || [cequal $type UifOutputList] } {
      if { [set myFormat [lindex $panWidget($wdgName) $i_outFormat]] != ""} {
         
         # Catch if format conversion cannot be done

         if {[catch {set value [format $myFormat $newVal]} ]} {
            set gvar($myAddr) $newVal
         } else {
            set gvar($myAddr) $value
         }

      } else {
         set gvar($myAddr) $newVal
      }
   } else {
      set gvar($myAddr) $newVal
   }
   update idletasks
} 


#-----------------------------------------------------------------------------
# PROCEDURE : panDbPoll - 
#-----------------------------------------------------------------------------

proc panDbPoll { cv pollRate {init {0}} } {
   global panWidget par EDITING panPollList ckDbPoll i_type panActiveEnv \
          panWdgEnvName shArea panEnvCwp tcl_version

   if {![info exists par(NOT_ACTIVE_ENV)]} { set par(NOT_ACTIVE_ENV) 0 }
   if {$pollRate < 1000 } { set pollRate [expr $pollRate * 1000]}

   if {[winfo exists $cv] == 0 } { return }
   if {$init || $EDITING} { set panPollList(ROOT) [panSetPollList $cv] }

   set root [winfo toplevel $cv]
   if {[cequal [wm state $root] iconic]} {
      if {$ckDbPoll} { 
         set par(pollId) [after $pollRate panDbPoll $cv $pollRate]
      }
      return
   }

   # Check if all envs. are active

   set notActEnvs {}
   if {[info exists panActiveEnv]} {
      foreach e [array names panActiveEnv] {
          set tmpCwp $panEnvCwp($e)

          if {([string first ":" $tmpCwp] < 0) && ([string first "<" $tmpCwp] < 0)} {
             append tmpCwp ":"
          }

	  if {[catch {seq_dbSetCwp ${e}$tmpCwp }]} {
              lappend notActEnvs [string range $e 1 end]
              set par(NOT_ACTIVE_ENV) 1
              set panActiveEnv($e) 0
          } else {
              set panActiveEnv($e) 1
          }
      }
   }

   if {[winfo exists  $shArea] && $par(NOT_ACTIVE_ENV)} {
      if {[lempty $notActEnvs]} {
         if {$par(NOT_ACTIVE_ENV)} {
            $shArea displayText ""
            set par(NOT_ACTIVE_ENV) 0
         }
      } else {
         $shArea displayText "WARNING Environments NOT ACTIVE : $notActEnvs"
      } 
   }

   # Process polling over main panel

   foreach name $panPollList(ROOT) {

       set tmp $panWdgEnvName($name)
       if {$panActiveEnv($tmp) == 0} {continue }

       set type     [lindex $panWidget($name) $i_type ]
       set newValue [panPollDbRead $name $type]

       if {! [cequal $newValue panFAILURE]} {
          panUpdateLocalVar $name $type $newValue
       }
   }

   #
   # Get Widgets that need 'polling' in the imported XXX_uifClass objects
   #
   if {$tcl_version >= 8.0} {
       set classes [itcl::find classes "*_uifClass"]
   } else {
       set classes [info classes "*_uifClass"]
   }
   foreach class $classes {
       if {$tcl_version >= 8.0} {
	   set objects [itcl::find objects -class $class]
       } else {
	   set objects [info objects -class $class]
       }
      foreach member $objects {

        # This checks the correct syntax of the window name
        # to prevent 'crash' when the definition is saved
          
	  if {$tcl_version >= 8.0} {
	      set member [namespace tail $member]
	  } else {
	      set member [info namespace tail $member]
	  }

        # Check if the window belongs is unmapped or one of its
        # ancestors is unmapped.

        if {[winfo ismapped $member] == 0} { continue }
        if {[winfo viewable $member] == 0} { continue }

        foreach name [$member getPollList] {

           set tmp $panWdgEnvName($name)
           if {$panActiveEnv($tmp) == 0} {continue }

           set type     [lindex $panWidget($name) $i_type ]
           set newValue [panPollDbRead $name $type]

           if {! [cequal $newValue panFAILURE]} {
              panUpdateLocalVar $name $type $newValue
           }
        }
      }; # end foreach member
   }; # end foreach classes

   if {$ckDbPoll} { 
         set par(pollId) [after $pollRate panDbPoll $cv $pollRate]
   }

   update idletasks
   return
}

#-----------------------------------------------------------------------------
# PROCEDURE : panUpdateOutput - 
#-----------------------------------------------------------------------------

proc panUpdateOutput { cv } {
   global panWidget par EDITING  ckDbPoll i_type tcl_version

   if {[winfo exists $cv] == 0 } { return }

   # Process polling over main panel

   foreach name [panSetPollList $cv all ] {
       set type     [lindex $panWidget($name) $i_type ]
       set newValue [panPollDbRead $name $type]

       if {! [cequal $newValue panFAILURE]} {
          panUpdateLocalVar $name $type $newValue
       }
   }

   #
   # Get Widgets that need 'polling' in the imported XXX_uifClass objects
   #

   if {$tcl_version >= 8.0} {
       set classes [itcl::find classes "*_uifClass"]
   } else {
       set classes [info classes "*_uifClass"]
   }
   foreach class $classes {
       if {$tcl_version >= 8.0} {
	   set objects [itcl::find objects -class $class]
       } else {
	   set objects [info objects -class $class]
       }
      foreach member $objects {

        # This checks the correct syntax of the window name
        # to prevent 'crash' when the definition is saved
          
	  if {$tcl_version >= 8.0} {
	      set member [namespace tail $member]
	  } else {
	      set member [info namespace tail $member]
	  }

        # Check if the window belongs is unmapped or one of its
        # ancestors is unmapped.

        if {[winfo ismapped $member] == 0} { continue }
        if {[winfo viewable $member] == 0} { continue }

        foreach name [concat [$member getPollList] [$member getEventList]] {
           set type     [lindex $panWidget($name) $i_type ]
           set newValue [panPollDbRead $name $type]

           if {! [cequal $newValue panFAILURE]} {
              panUpdateLocalVar $name $type $newValue
           }
        }
      }; # end foreach member
   }; # end foreach classes

   update idletasks
   return
}

#----------------------------------------------------------------------------
# PROCEDURE : panPollDbRead 
#----------------------------------------------------------------------------

proc panPollDbRead { name type } {
   global gvar panDbSymbAddr panDataToWidget  panWidget panDbData\
          i_outFormat i_precision  db_log tcl_precision orig_tcl_precision

   if {[cequal $type UifOutput] } {
      if { [set prec [lindex $panWidget($name) $i_precision]] != "" } {
         set tcl_precision $prec
      }
   }

#  puts "DBG> $name $panDbSymbAddr($name)"

   if { [catch {seq_dbReadSymbolic $panDbSymbAddr($name) -noBadQLog scanFlag } newVal ] } {
        if {[lindex $panDbData($name) $db_log] == 0 } {
           panLogDbOperation Read $panDbSymbAddr($name) $newVal
           setDbConfig $name $db_log 1 ;           # Disable logging
        }
        set tcl_precision $orig_tcl_precision;  # Set to default value
        return panFAILURE
   } else {
        setDbConfig $name $db_log 0          
   }

   panSetQualityFlag $name $scanFlag

#   puts "DBG> NEW VALUE $symbAddr $newVal scanFlag = $scanFlag"
     
   set tcl_precision $orig_tcl_precision;  # Set to default value
   return $newVal
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSetPollList - 
#-----------------------------------------------------------------------------

proc panSetPollList { cv {allFlag "" } } {
   global par panDbData EDITING db_type db_poll panDbSymbAddr

   set pList {}
   if {$par(CCS) == 0 || [winfo exists $cv] == 0 } { return }

   foreach name [panCvWidgetNames $cv uifType] {
      # Skip : input widgets and those generating a read error
      if { [info exists panDbData($name)] == 0 }   { continue }
      if { [cequal [lindex $panDbData($name) $db_type ] in ]} { continue }

      if {[cequal $allFlag all]} {
          lappend pList $name
      } else {
          set dbPoll [lindex $panDbData($name) $db_poll ]
          if { [cequal $dbPoll free] == 0 && $dbPoll != 1} { continue }
      }

      lappend pList $name
   }
   return $pList
}

#----------------------------------------------------------------------------
# PROCEDURE : panStartDbPoll - 
#----------------------------------------------------------------------------

proc panStartDbPoll { cv pollRate } {
   global panDbData db_log shArea shArea ckDbPoll

   # Enable logging for each widget

   if { [info exists panDbData] } {
      foreach name [array names panDbData] {
          setDbConfig $name $db_log 0 
      }; #end foreach
   }

   if { $ckDbPoll != 0} {
      after $pollRate panHeartBeat $shArea $pollRate
      after $pollRate panDbPoll $cv $pollRate 1
   }
}


#
# ___oOo___
