#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panDbRead.tcl,v 5.19 2003/02/14 14:23:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  03/03/98  created
#

#************************************************************************
#   NAME
#     panDbRead  - Reads a database element
#
#   SYNOPSIS
#
#     panDbRead  <symbAddress> <retValue>
#
#   DESCRIPTION
#     This procedure reads a database element taking care of the error 
#     handling.  In case of errors a dialog box appears allowing the user
#     to see the error stack.
#
#     <symbAddress>   Symbolic name of the database attribute.
#                     If the name does not contain the environment name, the
#                     procedure will read the data from the local environment
#                     -i.e.- the one defined by RTAPENV environment variable.
#
#     <retValue>      Returned current value
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#     SUCCESS The read value is stored in the <retValue> parameter
#     FAILURE .
#
#   CAUTIONS
#     This makes procedure dbRead obsolete.
#   EXAMPLES
#
#   If a want to read the database element :PARAMS:SCALARS.scalar_int8,
#   there are the following possibilities :
#  
#   - Using the absolute address :
#     panDbRead @wte16:PARAMS:SCALARS.scalar_int8 value
#   
#   - Using a relative address and cwp :
#     seq_dbSetCwp @wte16:PARAMS
#     panDbRead @wte16<relative>SCALARS.scalar_int8 value
#   
#   - Using the alias address :
#     panDbRead @wte16<alias>SCALARS.scalar_int8 value
#
#   To handle possible errors. use do as follows :
#
#   if {[panDbRead ":PARAMS:SCALARS.scalar_int32" value] == "FAILURE"} {
#      process error condition
#   }
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panDbRead { symbAddr value } {
   global par
   upvar $value newVal

   if {[info exists par(CCS)] == 0 || $par(CCS) == 0 } { return FAILURE }

   if {[string index $symbAddr 0] != "@"} {
      set symbAddr [panDbGetFullAddress "" $symbAddr]
   }

   set scanFlag 0 ; set newVal 0
   if { [catch {seq_dbReadSymbolic $symbAddr -noBadQLog scanFlag } newVal ] } {
      panLogDbOperation Read $symbAddr $newVal
      seq_errDisplay
      return FAILURE
   }
   return SUCCESS
}

#------------------------------------------------------------------------------
# PROCEDURE : dbRead -   OBSOLETE
#
#                        Kept here for backward compatibility
#------------------------------------------------------------------------------

proc dbRead { symbAddr } {
   global par

#  puts "DBG dbRead $symbAddr "
   if {[info exists par(CCS)] == 0 || $par(CCS) == 0 } { return 0 }

   if {[string index $symbAddr 0] != "@"} {
      set symbAddr [panDbGetFullAddress "" $symbAddr]
   }

   set scanFlag 0
   if { [catch {seq_dbReadSymbolic $symbAddr -noBadQLog scanFlag } newVal ] } {
      panLogDbOperation Read $symbAddr $newVal
      set newVal 0
   }

#   puts "DBG dbRead $symbAddr $val" 
   return $newVal
}

#
# ___oOo___
