#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panExecTerm.tcl,v 5.19 2003/02/14 14:23:49 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    15/05/96  created
#           06/03/97  Porting to TK 4.2
#
#
#************************************************************************
#   NAME
#     panExecTerm - Procedure to run UNIX exec commands and
#                   display result on a Xterm-like panel.
#
#   SYNOPSIS
#     panExecTerm <cmd> ?params?
#
#   DESCRIPTION
#     
#     This procedure executes a UNIX command.
#     If <cmd> is executed in background the result is printed on the screen
#     otherwise the result is displayed on a Xterm like window.
#
#     <cmd>   UNIX command
#     ?args?  Optional list of parameters for the command.
#             This list can also contain the symbol '&' to execute the
#             command in background
#
#     The Xterm like window contains a menu bar containing a 'File' menu,
#     which includes the following entries :
#
#     - Print On Screen   Force result to be dumped on the screen.
#                         This is used for automatic test procedures.
#
#     - Quit              Exits the window and resume execution of the 
#                         Tcl/Tk script calling panExecTerm.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#     none
#
#   CAUTIONS
#
#   EXAMPLES
#
#     panExecTerm ls -lt
#     panExecTerm myScript par1 par2 &
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panExecTerm { cmd  args } {

   global panTermReturn

   set last [expr [llength $args] - 1]
   
   if { [lindex $args $last] == "&"} {
      eval exec $cmd $args
   } else {
      set retExec [eval exec -keepnewline $cmd $args]
      panTerm "panTerm" $retExec

      vwait panTermReturn ; # Suspend until 'panTermReturn' change
   }
}

#
# ___oOo___
