#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panImportPanel.tcl,v 5.19 2003/02/14 14:23:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    22/05/95  created
# M.COMIN   18/07/95  Add flag to control panel creation
# M.COMIN   05/09/95  Append current cwp spec to argv
#           21/10/98  Modified to work with 'normal' tcl scripts
#

#************************************************************************
#   NAME  
#     panImportPanel - Call a panel from the same interpreter.
#
#   SYNOPSIS
#     panImportPanel  <panelName> ?-iconify? ?parameter_list?
#
#   DESCRIPTION
#   
#   This procedure allows the user to call a panel and pass some parameters
#   in the command line.
#
#     <panelName>        Name of the panel. The name is the same of the file 
#                        produced by the panel editor, without the ".pan" 
#                        extension
#                        The panel to be invoked must be present in the "bin" 
#                        sub-directory of the following root directories :
#                        current module , INTROOT or VLTROOT.
#
#     ?-iconify?         If this is defined the imported panel is iconified
#                        when closed.
#
#     ?parameter_list?   List of parameters to be used in the command line.
#                        An item of this list has the following format :
#                                      <var>=<value>
#                        Where :
#                        <var>    Specifies a global variable
#                        <value>  The new value
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   None
#
#   CAUTIONS
#
#   EXAMPLES
#
#   Simple example :
#
#     panImportPanel myPanel "cwp=@wte13:PARAMS" "gvar(ccdName)=ccd"
#
#   Sometimes the list of parameters can be long or it is more convenient
#   to build it up in a list variable :
#
#     set myParList {}
#     lappend myParList "cwp=@wte13:PARAMS"
#     lappend myParList "gvar(ccdName)=ccd"
#     lappend myParList "pollRate=2000"
#   
#     eval panImportPanel myPanel -iconify $myParList 
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panImportPanel { file args } {
   global par ckMenuBar EDITING shArea panImportFlag cwp initDone 
   global menuBar panDEBUG

   if {[set filePath [findVLTFile bin/$file]] == ""} { return }

   if {[winfo exists ".${file}" ] } {
      wm deiconify ".${file}" ; # If the panel already exist just deiconify it
      return
   }

   # Create a new instance of the panel

   set oldRoot         $par(ROOT_PANEL)
   set oldShArea       $shArea
   set panImportFlag   1
   set iconifyFlag     0
   set par(ROOT_PANEL) .${file}

   if {[info exists panDEBUG] && ![cequal $panDEBUG dbgNONE] } {
      puts "USER DEBUG >  panImportPanel $file $args"
   }

   if {[set idx [lsearch $args "-iconify"]] != -1} {
      set iconifyFlag 1
      set args [lreplace $args $idx $idx] ;# Remove option -iconify from args
   }

   # set CWP to current value by default

   set argv [eval list $args]
   set argc [llength $argv]

   if {[info exists cwp] && $cwp != ""} { lappend argv "cwp=$cwp" }
   panCmdLine [llength $argv] $argv
   source $filePath

   # Check if what we are running is a panel or a normal tcl script

   if {[winfo exists .${file}]} {

      #
      # Change the 'Quit' command to iconify the panel.
      #
       set wroot [winfo toplevel .${file}]
       wm resizable $wroot 0 0

       if { $ckMenuBar && ![cequal [set w [panGetMbPath File]]  ""] } {

	   set menuPath [$w cget -menu]
	   set idx      [$menuPath index last]
	   set tmp      [wm protocol $wroot WM_DELETE_WINDOW]

	   if { $iconifyFlag } {
	       $menuPath entryconfigure $idx -command "panIconifyPanel"
	       wm protocol $wroot  WM_DELETE_WINDOW "$tmp ; panIconifyPanel"
	   } else {
	       $menuPath entryconfigure $idx \
		    -command "panClosePanel {} $par(ROOT_PANEL)"
	       
	       wm protocol $wroot  WM_DELETE_WINDOW "$tmp ; panClosePanel {} $par(ROOT_PANEL)"
	    }
	}

   }

   #
   # Restore previous parameters
   #
   if { $EDITING } { 
      set par(ROOT_PANEL) $oldRoot
      set shArea          $oldShArea
   }

   set panImportFlag 0
}


#
# ___oOo___
