#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panInit.tcl,v 5.19 2003/02/14 14:23:42 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# MCOMIN    05/09/95  created
# MCOMIN    14/09/95  add routine to process command line parameters
#           12/12/97  add code to re-position the panel by giving new
#                     coordinates from command line.
#           27/03/98  panInitCCS initializes the proc. name also with a 
#                     fixed name specified by the user
#           30/04/98  Add new standard procedure xxxSetInitData
#           27/08/98  Add debug infos.
#           13/12/99  Comment statement panLoadPackage
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panInit { argc argv {root ""} } {
   global env par myName argv0 EDITING panImportFlag panDEBUG panClock \
          ckCCS seq_ccsEnvName panArgv cwp cwpEnv localEnv cv env

   set retStatus 1

   if {![cequal $env(PANEL_TIME) ""]} {
       set panClock(2) [clock seconds] 
   }

   set myName $par(MY_NAME)_[pid]

   if {[info exists seq_ccsEnvName]} {
      set cwpEnv   $seq_ccsEnvName
      set localEnv $seq_ccsEnvName
   } elseif {[info exists env(RTAPENV)]} {
      set cwpEnv   $env(RTAPENV)
      set localEnv $env(RTAPENV)    
   } else {
      set cwpEnv   ""
      set localEnv ""
   }

   if {![info exists cwp] } { set cwp "" }
   if {![info exists panDEBUG] } { set panDEBUG dbgNONE }

   panCmdLine $argc $argv  ;# Process Command Line Arguments

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > process cmd line : $argv"
      puts "DEV. DEBUG > Create Panel Frame      [clock format $panClock(1) -format %T]"
      puts "DEV. DEBUG > panInit START           [clock format [clock seconds] -format %T]"
   }

#   panLoadPackage Rtd

   if {$EDITING} {
      set tmpArgv [eval list $panArgv]
      panCmdLine [llength $tmpArgv] $tmpArgv 
   }
   
   ######################################
   #           CCS Initialization
   ######################################

    if {[cequal $panDEBUG dbgDEV]} { puts "DEV. DEBUG > init CCS" }

    if {[panInitCCS] == 0} {
	if {$EDITING} { 
           return 0 
        } else {
           exit
        }
    }

   # Set Current Working Point
   if {[panSetMainCwp $cwp 0] == "FAILURE" } { set retStatus 0 }

   if {[cequal $panDEBUG dbgDEV] && $par(CCS)} { 
         puts "DEV. DEBUG > panInit   CWP Current Value = [seq_dbGetCwp]"
   }

   if {[winfo exists .$root] && ![cequal [info procs ${root}InitData] ""]} { 
      eval ${root}InitData 
   }

   if {[cequal $panDEBUG dbgDEV]} {puts "DEV. DEBUG > panInit END ..."}
   update idletasks

   if {![cequal $env(PANEL_TIME) ""]} {
       set panClock(3) [clock seconds] 
   }

   return $retStatus
}

#############################################
#     Procedure to process the command line
#############################################

proc panCmdLine { argc argv } {
   global argv0 panDEBUG pollRate cmdCwp cwp gvar

   ###################################################
   #  Override defaults using the run time parameters
   ###################################################

   if { $argc == 0 } { return }

   # Get current window coordinates
   set xcoord [winfo rootx .]
   set ycoord [winfo rooty .]
   set geoFlag 0

   set idx 0
   while { $idx < $argc } {

      set opt  [lindex $argv $idx]

      if {[set i [string first "=" $opt ]] == -1 } { incr idx; continue }

      set var    [string range $opt 0 [expr $i-1]]
      set value  [string range $opt [expr $i+1] end]
            
#      puts "DBG > var = $var value = $value"
        
      # Skip if value is not defined
      if {[cequal $value ""]} { incr idx; continue }

      switch $var {
         xcoord   { set xcoord $value ; incr geoFlag }
         ycoord   { set ycoord $value ; incr geoFlag }          
         pollRate { set pollRate [expr $value * 1000] }
         cwp      { set cwp "$value" }
         default  { uplevel #0 set $var \"${value}\" }
      }

      incr idx
   }; # end of while ..

   if {$geoFlag} { wm geometry . +${xcoord}+${ycoord} }
}

#----------------------------------------------------------------------------
# PROCEDURE : panInitCCS -  Starts the panel
#----------------------------------------------------------------------------

proc panInitCCS { } {
   global ckCCS myName par initDone EDITING panDEBUG seq_ccsType panImportFlag

   set par(CCS) $ckCCS

   if {![info exists initDone]} { set initDone 0 }

   # Check if seq_ccsInit has been already called and failed
   if {$initDone == -1} { set par(CCS) 0 }

   if { $ckCCS == 0 || $initDone != 0 || $panImportFlag} { return 1 }
   set home [pwd]

   if {  [info exists par(PANEL_PROC_NAME)] &&
        ![cequal $par(PANEL_PROC_NAME) ""] } {
     
      set procName $par(PANEL_PROC_NAME)
   } else {
      set procName $myName
   }

   # Check length of proc. name 

   set tmp [string range $procName 0 18]

   if {([string length $procName] > 19) && $EDITING } {
      set tmp "Process Name \"$procName\" Too Long\n(maximun 19 characters)"
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK

      puts "Process Name \"$procName\" Too Long : maximun 19 characters"
      puts "The name is automatically shortened to \"$tmp\""
   }

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panInit $procName initDone = $initDone "
   }

   set initDone 1

   if {[cequal $seq_ccsType light]} {
      set flag [catch {seq_ccsInit -sendObituaries $tmp} ret]
   } else {
      set flag [catch {seq_ccsInit $tmp} ret]
   }

   if { $flag } {
      set initDone -1
      cd $home;
      set tmp "$procName Initialization FAILURE : $ret\n"
      seq_logData $tmp ;  # Log Error

      if {[cequal $panDEBUG dbgDEV]} {puts "DEV. DEBUG > $tmp"}

      set par(CCS) 0

      if {$EDITING} {
         if {[panShowError .top $tmp question 1 Continue Cancel]} {
            return 0
         }
      } else {
         puts $tmp 
      }
   }

   return 1
}




#
# ___oOo___
