#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panMainStart.tcl,v 5.19 2003/02/14 14:23:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.COMIN   27/03/95  Add possibility to pass array elements on command line.
# M.COMIN   12/04/95  Add panUsrInit
# M.COMIN   01/06/95  Moved to separate file from 'createEmptyPanel'
# M.COMIN   20/07/95  Add host to title.
# M.COMIN   05/09/95  CCS initialization moved to separate file
# M.COMIN   14/09/95  Command line parameters are re-processed
# M.COMIN   02/10/95  Add Cmd. Reply Area
# M.COMIN   20/10/95  Add processing for DB events
# M.COMIN   01/11/95  Add check after setting CWP
# M.COMIN   14/12/95  Changed proc. to create cmd.reply widget
# gfilippi  15/10/98  commented two "puts" in the temporary fix (GIF)

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panMainStart - Creates the whole panel
#             
#------------------------------------------------------------------------------

proc panMainStart { root {argc {0}}  {argv { }} } {
   global env par  argv0 EDITING panImportFlag gvar panDEBUG ckHeartBeat\
          ckShHelp ckActBut ckMenuBar ckCCS ckCmdReply ckDbPoll shArea \
          actAreaHeight panUsrProcs panDbData db_log initDone auto_path \
          seq_ccsEnvName cv geoCmdReply  ckBackCompatible pollRate cwp cwpEnv \
	  panActiveEnv panEnvCwp panClock

   if {![cequal $env(PANEL_TIME) ""]} {
       set panClock(4) [clock seconds] 
   }

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panMainStart START [clock format [clock seconds] -format %T]"
   }

   set auto_path [lrmdups $auto_path]

   if {$ckCCS} {
       if {[info exists env(RTAPENV)]} { set tmpTitle $env(RTAPENV) }
   } else {
       if {[info exists env(HOST)]} { set tmpTitle $env(HOST) } 
   }

   if {[info exists tmpTitle] } {
      if {$EDITING || $panImportFlag } {
         wm title .${root} "[wm title .${root}] - @${tmpTitle}"
      } else {
         wm title  "." "[wm title .] - @${tmpTitle}" 
      }
   }

   # Enable logging for each widget with DB connection

   if {[info exists par(ENV_LIST)]} {
      lrmdups $par(ENV_LIST) ;# remove duplicates of env. names
   }

#   puts "panMainStart reset panDbData ... [clock format [clock seconds] -format %T]"
   if { [info exists panDbData] } {
      foreach name [array names panDbData] {
          setDbConfig $name $db_log 0 
      } ;#end foreach
   }

   # For backward compatibility

   if {![info exists ckDbPoll]}   { set ckDbPoll 1 }
   if {![info exists ckHeartBeat]} { set ckHeartBeat 1 }
   if {![info exists cwp]}        { set cwp "" }   
   if {![info exists ckCmdReply]} { set ckCmdReply 0 }
   if {![info exists initDone]}   { set initDone 0 } 
   
   # Initialization was done before ... but failed
      
   if { $panImportFlag } { panInit $argc $argv }

   panGetCvNames $root cv bcv ;     # Get canvas names

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panMainStart   root window $root cwp = $cwp"
   }

#   puts "panMainStart load widgets [clock format [clock seconds] -format %T]"
#   puts "DBG > panMainStart : root = $root , cv = $cv , bcv = $bcv "

   set shArea [panGetShArea .${root}]    ;# Name of Short Help Area

   panLoadWidget 0  $shArea $cv $cv $bcv ;# Load widgets

   if { $ckActBut } { $bcv configure -height $actAreaHeight }

#   puts "panMainStart load short help  [clock format [clock seconds] -format %T]"
   if { $ckShHelp } {
      loadShortHelp $shArea $cv  [panCvWidgetNames $cv all]
      loadShortHelp $shArea $bcv [panCvWidgetNames $bcv all]
   }

#   puts "panMainStart load MenuBar  [clock format [clock seconds] -format %T]"

   if { $ckMenuBar  } { panLoadMenuBar $shArea  }
   if { $ckCmdReply } { createCmdReply .${root} }

   panDbSetSymbAddr $cv ;# Process symbolic names

#   puts "panMainStart dbInputRefresh, panDbPoll, etc ...  [clock format [clock seconds] -format %T]"

   if {$par(CCS)} {
      dbInputRefresh  all ;# Update all input field

      if {$pollRate > 0 && $ckDbPoll} {
	  # fill up panEnvCwp from panActiEnv
	  foreach i  [array names panActiveEnv] {
	      if {[info exist panEnvCwp($i)]} continue;
	      #GIF15-10 puts "Fixing panEnvCwp for $i"
	      # Note: string range .... is used to take away @ from env name
	      # specifically becausse here, seq_dbGetCwp fails on @lt1m1m3 (a DEC95 LCU)
	      # but succeeds with lt1m1m3 ... and all the others we tried.
	      # 
	      if { [ catch {set cwp [seq_dbGetCwp [string range $i 1 end] ]} ] } {
		  set panEnvCwp($i) :
	      } else {
		  set panEnvCwp($i) $cwp
	      }
	      #GIF15-10 puts "panEnvCwp for $i set to $panEnvCwp($i)"
	  }
         set par(NOT_ACTIVE_ENV) 0
         panDbPoll $cv $pollRate 1; # Start polling of output fields
      }
      panEvtDbOperation $cv ATTACH canvas

   } 

   if { $EDITING == 0 && $ckHeartBeat} { panHeartBeat $shArea $pollRate }

   panSetWidgetLimits $cv

   ######################################
   #       USER's  Initialization
   ######################################

   if {![cequal $env(PANEL_TIME) ""]} {
       set panClock(5) [clock seconds] 
   }

   if { ![cequal [info procs panUsrInit] ""] }   { panUsrInit }
   if { ![cequal [info procs ${root}Init ] ""] } { eval ${root}Init }
   update idletasks

   # Add window manager exit handler to close panel.

   if {$panImportFlag} {
       set procName panIconifyPanel
       set wmRoot .${root} 
   } else {
       set procName panClosePanel
       set wmRoot   .
   }

   # Query current winndow manager exit handler.
   set tmp [wm protocol $wmRoot WM_DELETE_WINDOW]


   if {[cequal $tmp ""]} {
       wm protocol $wmRoot WM_DELETE_WINDOW $procName
   } else {
       wm protocol $wmRoot WM_DELETE_WINDOW "$tmp ; $procName"
   }

   if {![cequal $env(PANEL_TIME) ""]} {
       set panClock(6) [clock seconds]

       for {set i 1} {$i < 7} { incr i} {
           puts "panClock($i) $panClock($i) "
       }

       puts "\nPanel $argv0"
       puts "Frame Creation and Global Data procedure    [expr $panClock(2) - $panClock(1)]"
       puts "CCS and InitData procedure                  [expr $panClock(3) - $panClock(2)]"
       puts "Creation of basic widgets and menus         [expr $panClock(4) - $panClock(3)]"
       puts "Loading classes, DB events                  [expr $panClock(5) - $panClock(4)]"
       puts "User Init procedure                         [expr $panClock(6) - $panClock(5)]"
       puts "                                            Total >  [expr $panClock(6) - $panClock(1)]"

   }

   if {[cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panMainStart END        [clock format [clock seconds] -format %T]"
   }

   return
}

#
# ___oOo___
