#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panMisc.tcl,v 5.19 2003/02/14 14:23:42 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin    12/12/95    created
# mcomin    01/02/96    Add check in panWdgReferenceList for not existing 
#                       'member'
#           30/08/96    Check if class is mapped
#           12/09/96    Add panSetQualityFlag
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panLoadPackage { args } {
   if {[lempty $args]} { return }

   foreach item $args {
      catch {package require $item}
   }
}

proc panAddNamespace { args } {
   global namespaceList tcl_version
   foreach item $args {
      lappend namespaceList $item
       if {$tcl_version >= 8.0} {
	   namespace eval $item {namespace export *}
	   # catch = temp fix to avoid problems with old-style tclIndex files
	   catch {namespace import -force ${item}::* }
       } else {
	   # next command creates the namespace if it doesn't exist yet
	   namespace $item;
	   import add $item
       }
   }
}

proc panCreateShArea { shArea } {
   global pollRate ckHeartBeat EDITING

   if {![info exists ckHeartBeat]} {
      set ckHeartBeat 1
   }

   if {[winfo exists $shArea] == 0 } {
      UifShortHelp $shArea -showHeartBeat $ckHeartBeat \
           -dbBitmap uifDbRefresh.xbm \
           -command  {global cv ; panEvtDbOperation $cv RESTART all; panUpdateOutput $cv}

      set w [$shArea component dbRefresh]

      if {$ckHeartBeat == 0 || $EDITING} {
         # Usually this is the case of small panels, therefore smaller
         # help text is required.
         set tmp "Recover access to DB"
      } else {
         set tmp "Re-Start DB events and update output widgets"
      }

      UifShortHelp::add $shArea $w $tmp
 
      pack $shArea -side top -fill x
   }

#   after $pollRate panCheckHeartBeat $shArea
}

#-----------------------------------------------------------------------------
# PROCEDURE : panWdgReferenceList - Get list of widget names in the canvas
#                                   of the main panel.
#
# Parameter "where" can be :
#
# - "canvas" all widgets in the main canvas are listed
# - "all"    widgets of the whole panel are listed
#
# Parameter "what" can be :
#
# - "all"    all widgets in the canvas are listed
# - "uifType" only base uifWidgets are listed 
#
#-----------------------------------------------------------------------------

proc panWdgReferenceList { rootWdg where what } {
   global panClassPath tcl_version

   if {[winfo exists $rootWdg] == 0 }  { return {} }
   if {[info exists panClassPath] }    { unset panClassPath }

   # Get Widgets for current top widget

#   puts "panWdgReferenceList> $rootWdg $where $what"
   set wdgList [panCvWidgetNames $rootWdg $what]
#   puts "panWdgReferenceList> wdgList = $wdgList"

   if {[cequal $where canvas] } { return $wdgList }

   #
   # Get Widgets in the imported classes
   #

   if {$tcl_version >= 8.0} {
       set classes [itcl::find classes "*_uifClass"]
   } else {
       set classes [info classes "*_uifClass"]
   }
   foreach class $classes {
       if {$tcl_version >= 8.0} {
	   set objects [itcl::find objects -class $class]
       } else {
	   set objects [info objects -class $class]
       }
      foreach member $objects {

          # This checks the correct syntax of the window name
          # to prevent 'crash' when the definition is saved
          
	  if {$tcl_version >= 8.0} {
	      set member [namespace tail $member]
	  } else {
	      set member [info namespace tail $member]
	  }

          # Check if it belongs to a unmapped window
          # In that case do not add to the list

	  if {[winfo viewable $member] == 0}             { continue }
          if {[string match "${rootWdg}*" $member] != 1} { continue }
          set wdgList [concat $wdgList [panClassWidgetNames $member uifType]]

      }; # end loop on class members
   }; # end loop on classes
    
#   puts "DBG > Elem = [llength $wdgList] , total widget list = $wdgList"

#   puts "panWdgReferenceList> RETURN wdgList = $wdgList"
   return $wdgList
}  

#
#
#-----------------------------------------------------------------------------
# PROCEDURE : panCvWidgetNames - Returns list of widget names in the canvas
#                                of the main panel.
# Parameter Flag can be :
# - "all"    all widgets in the canvas are listed
# - "uifType" only base uifWidgets are listed 
#-----------------------------------------------------------------------------
#
proc panCvWidgetNames { cv  what } {
   global i_type panWidget
   
   set myList   {}
                                      
   foreach child [winfo children $cv ] {
      set wdgRef [winfo name $child]
      if {[info exists panWidget($wdgRef)] == 0 } { continue }

      if {[cequal $what "all"]} {
          lappend myList $wdgRef
      } else {
          set type [lindex $panWidget($wdgRef) $i_type]
          if {![cequal $type UifClass] && ![cequal $type PubClass]} {
                lappend myList $wdgRef
          }
      }
   }
   return $myList
}

#-----------------------------------------------------------------------------
# PROCEDURE : panClassWidgetNames - Returns list of widget names in the canvas
#                                   within a uifClass.
# Parameter Flag can be :
# - "all"     all widgets in the canvas are listed
# - "uifType" only base uifWidgets are listed 
#-----------------------------------------------------------------------------

proc panClassWidgetNames { rootWdg what } {
   global i_type panWidget panClassPath EDITING par
   
   set myList   {}

   set className [winfo name $rootWdg]
   append rootWdg ".cv"
   set panClassPath($className) $rootWdg
   set classFlag 0

   if { $EDITING == 0 || [cequal $par(EDIT_WHAT) .pan] ||
        ! [cequal $className $par(MYCLASS)] } {
       set classFlag 1
   }
 
#   puts "panCvClassNames > classFlag($classFlag) rootWdg($rootWdg) what($what)"

   foreach child [winfo children $rootWdg ] {

      if {$classFlag} {
         set wdgRef ${className}.[winfo name $child]
      } else {
         set wdgRef [winfo name $child]
      }
      
      if {[info exists panWidget($wdgRef)] == 0 } { continue }

      if {[cequal $what all]} {
         lappend myList $wdgRef
      } else {
         set type [lindex $panWidget($wdgRef) $i_type]
         if {![cequal $type UifClass] && ![cequal $type PubClass]} {
                lappend myList $wdgRef
          }
      }
   }
   return $myList
}


proc panSetQualityFlag { name quality } {
   global panDataToWidget panDbData db_var db_log panWidget
   
   set attr "gvar([getDbConfig $name $db_var])"

   if {[info exists panDataToWidget($attr)] == 0 }       { return }
   if { $quality == [lindex $panDataToWidget($attr) 0] } { return }

   # Log that a change on quality has occurred

   if {$quality} {
      seq_logData "Data Quality for $attr : BAD"
   } else {
      seq_logData "Data Quality for $attr : OK"
   }

   if {[info exists panDbData($name)]} {
      setDbConfig $name $db_log $quality
   }

   set panDataToWidget($attr) [lreplace $panDataToWidget($attr) 0 0 $quality]

   foreach w $panDataToWidget($attr) {
       if {[winfo exist $w]} { $w displayScanStatus $quality }
   }
   update idletasks
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSetActionAreaSize - Sets the size of the action area
#-----------------------------------------------------------------------------

proc panSetActionAreaSize { newSize } {
   global par cv bcv panCvYSize

   set currSize  [lindex [$bcv configure -height] 4]

   if { $currSize != $newSize && $newSize != "" } {

      set diff [expr $newSize - $currSize]

      #
      # Trick to adjust keep the total geometry the same, by reducing the
      # widget canvas size : this avoids problems with the automatic 
      # re-sizing of the panel.
      # The user can finally adjust the size manually
      # 
      set panCvYSize [expr [lindex [$cv configure -height] 4] - $diff]
      
      $cv  configure -height $panCvYSize ; # Trick
      $bcv configure -height $newSize
      update idletasks
   }
}


#----------------------------------------------------------------------------
# PROCEDURE : panGetWdgPath - Returns the widget whole path
#----------------------------------------------------------------------------

proc panGetWdgPath { cv wdgName } {
   global i_subClass panClassPath

   if {[getWidget $wdgName $i_subClass] == 0} { return $cv.$wdgName }

   set tmp [split $wdgName "."]
   return $panClassPath([lindex $tmp 0]).[lindex $tmp 1]
}

#-----------------------------------------------------------------------------
# PROCEDURE : panParseArray - Returns the array and element names
#-----------------------------------------------------------------------------

proc panParseArray { var array elem } {
   upvar $array a
   upvar $elem  e

   set ret [regexp -nocase {([a-z0-9]+)\((.*)\)} $var all a e]
   if {$ret == 0 } { set a "" ; set e  $var }
#   puts "DBG > panParseArray $var  --> $a ($e) "
   return $ret
}

#------------------------------------------------------------------------------
# PROCEDURE : panClassUnload - Unload all current class definitions
#------------------------------------------------------------------------------

proc panClassUnload { } {
    global par i_type panDEBUG EDITING tcl_version

    panGetCvNames $par(ROOT_PANEL) cv bcv
    set delClassList {}

    if {$panDEBUG == "dbgDEV"} {puts "DEV. DEBUG > panClassUnload START ..."}

    # Get the list of all classes to be unloaded

    if {$tcl_version >= 8.0} {
	set classes [itcl::find classes "*_uifClass"]
    } else {
	set classes [info classes "*_uifClass"]
    }
    foreach member $classes {
       if {[string match "itcl*" $member]} { continue }
       if {[string match "Uif*"  $member]} { continue }
       if {[string match "*itk*" $member]} { continue }

       if {$panDEBUG == "dbgDEV"} {puts "DEV. DEBUG > add $member to delete"}
       lappend delClassList $member
    }

    # Unload classes : this unloads, as well, the class under editing
    # The catch is used to prevent a crash in case the class does no longer
    # exist.

    foreach class $delClassList {
	catch { delete class $class } ; # Delete class definition
        update
    }

    if {$panDEBUG == "dbgDEV"} {puts "DEV. DEBUG > panClassUnload END ..."}
}

#----------------------------------------------------------------------------
# PROCEDURE : panParseVar - Parse panel editor var. definition
#----------------------------------------------------------------------------

proc panParseVar { varString } {
   global par nameMatch

   # First check if variable format is gvar(xxx)
   # Then  check if this represent an array -i.e.-  item(elem)
   # The regular expression recognizes Rtap symbolic addresses.

   if { [panParseArray $varString array elem] } {
      set myVar $elem
   } else {
      set myVar $varString
   }

   #
   # uifClass widgets : strip out class CWP from variable name
   #

   if { $par(EDIT_WHAT) != ".pan" } {
      set tmp        [string trimleft $par(ROOT_PANEL) "."]
      set clInstance "$par(ROOT_PANEL).top.wdg.$tmp"
      set myCwp      [$clInstance cget -cwp ]

      if {[string match "$myCwp*" $myVar ] } {
         set cwpLen     [string length $myCwp]
         set myVar      [string range $myVar $cwpLen end]
      }
   }

   return [string trim $myVar " "]
}

#-----------------------------------------------------------------------------
# PROCEDURE : panUpdateAutoPath - This routine updates the auto_path
#-----------------------------------------------------------------------------

proc panUpdateAutoPath { libPath } {
   global auto_path panUsrLib env
 
   # Check if library name has a complete path
   # puts "DBG> panUpdateAutoPath libPath = $libPath"

   set myLib [file tail [file rootname $libPath]]
   
   if {[lsearch $panUsrLib $myLib] != -1 }   { return "" }
   if {[lsearch $panUsrLib $libPath] != -1 } { return "" }

   # Search for the library name as it is.
   set vltPath [findVLTFile lib/${myLib} no]

   # Search for the library name with VLT extension.
   if {[cequal $vltPath ""]} {
       set vltPath [findVLTFile lib/${myLib}.tcl no]
   }

#   puts "DBG> libPath = $libPath   vltPath = $vltPath  myLib = $myLib"

   if {[cequal $vltPath ""]} {

      # This is not a library under VLT lib directory, check if 
      # the path leads to another directory

      #
      # Does the file exist in the TCLTK root area
      #

      if {![info exists env(TCLTK_ROOT)]} {
         set env(TCLTK_ROOT) undefined
      }

      if {[file exists $env(TCLTK_ROOT)/lib/$libPath] || 
          [file exists $libPath] } {

         lappend panUsrLib $libPath
         lappend auto_path $libPath        

      } else {
         set msg "Library  : $myLib not FOUND"
         tk_dialog .dialog "Dialog Box" $msg info 0 OK
         return ""
      }

   } else {
      lappend panUsrLib $myLib
      lappend auto_path $vltPath 
   }

   return $libPath
}

#
# ___oOo___
