#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: seqDisplayTerm.tcl,v 2.77 2003/03/18 19:44:18 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    19/09/96  created
#
#************************************************************************
#   NAME
#
#   seq_displayTerm  - General purpose utility to display text on a window
#
#   SYNOPSIS
#
#   seq_displayTerm <top> <title> <fileName> <text>
#
#   DESCRIPTION
#
#   The procedure creates a window to show text ASCII files.
#
#   top      - toplevel window
#   title    - Window title
#   fileName - Name of the file
#   text     - Buffer containing the text to be displayed  
#
#   The window is formed by :
#   - MenuBar containing 
#   - The name of the file to be displayed
#   - A text widget with vertical and horizontal scroll bars containing the
#     text to be displayed
#
#   If the toplevel window already exists, the new <text> replaces the one
#   displayed in the current instance of the toplevel window.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc seq_displayTerm { top title fileName buffer} {

   set c      $top.vr;  # Define 'Virtual Root'
   set f1     $c.f1 ;   # Contains the fileName
   set f2     $c.f2 ;   # Contains the text widget
   set txtWdg $f2.txt ; # Name of the text widget

   # If the window already exist , just update the contents

   if {[winfo exists $top]} { 
      $txtWdg  delete 0.0 end
      $txtWdg  insert 0.0 $buffer      
      $c.f1.fileName configure -text $fileName
      wm title $top $title
      return
   }

   toplevel  $top ;     # Define the outer most part of the panel
   frame $c ; pack $c

   #################################      
   #     Creation of Menu Bar   
   #################################

   frame  $c.menu -border 1 -relief raised 
   ###pack append  $c $c.menu {padx 10 pady 10 fillx}
   pack $c.menu -padx 10 -pady 10 -fill x
   set menuBar $c.menu

   #  Create Menus : File 

   set tmp $menuBar.file
   menubutton $tmp -menu $tmp.menu -text File
   ###pack append $menuBar $tmp { left padx 5 } 
   pack $tmp -side left -padx 5 

   set mbFile [menu $tmp.menu]

   #  Add component to menu File

   $mbFile add command -label Quit -command "destroy $top"

   #################################      
   #     Creation of panel
   #################################

   frame $f1 ; frame $f2;

   pack $f1 -pady 12 -fill x
   pack $f2 -fill both

   label $f1.lbl1     -text "Name of the File : "
   label $f1.fileName -text $fileName

   pack $f1.lbl1  $f1.fileName -side left -padx 5 

   # Create text widget with scroll bars

   text $txtWdg -width 80 -height 24 -setgrid false -wrap none -borderwidth 1 \
               -font -Adobe-courier-medium-r-normal--12* -relief sunken
   
   scrollbar $f2.scrollv -orient v -borderwidth 2 -width 10\
             -command "$txtWdg yview" -relief sunken

   scrollbar $c.scrollh -orient h -borderwidth 2 -width 10\
                -command "$txtWdg xview" -relief sunken

   $txtWdg configure -yscrollcommand "$f2.scrollv set" \
	             -xscrollcommand "$c.scrollh set"

   pack $txtWdg       -side left -fill both -expand 1  
   pack $f2.scrollv   -side left -fill y
   pack $c.scrollh    -fill x

   $txtWdg insert 0.0 $buffer 
   
   wm title     $top $title
   wm resizable $top 0 0
}
#
# ___oOo___
