#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: seqErrProcedures.tcl,v 2.77 2003/03/18 19:44:19 vltsccm Exp $" 
#
# (used to be errShowStack)
#
# who        when        what
# ---------  ----------  ----------------------------------------------
# M.COMIN    26/10/94    First release
# M.COMIN    26/01/95    Add date to select the stack
# -------------------
# mcomin     22/01/96    Stack was displayed in increasing order instead
#                       of decreasing.
#
# mcomin     15/03/96    This set of functions related to the error system
#                        has been moved to the seq module
#            27/09/96    Add errStackPrintOnScreen
#            30/10/97    Fixed bug in formatStackLine (backward compatibility)
# eallaert   1998-07-03  Replaced "cequal" by "switch" (which is a lot faster)
# mcomin     01/09/98    Provide possibility to search fro the last n-th lines
#                        of the logFile
# eallaert   1998-10-21  Made compatible with Tcl 8.0; various sections
#                        cleaned up
# mcomin     1999-08-05  Replace lindex with ctoken.
# mcomin     2000-01-24  Add wm group ..  and wm transient .. (SPR 960106)
#
#
#************************************************************************
#   NAME
#
#   SYNOPSIS
#
#   DESCRIPTION
#       
#
#   ENVIRONMENT VARIABLES
#
#   EXAMPLES
#
#------------------------------------------------------------------------
#
proc ::internal::errShowHelp { { helpFile ""} } {
    variable errCnt 
    variable errList 
    variable indexList 
    variable errStack
    variable mainShArea

    switch -- $helpFile {} {

	# Get the name of the helpFile from the error description
	switch -- [set idx [$errStack getIndex]] {} {return}
	
	set idx      $indexList([expr {$idx + 1}])
	set tmp      [lindex $errList($idx) 12]
	set errMemo  [string trim $tmp " :"]
	set helpFile ${errMemo}.hlp
    } default {
	set errMemo  [file rootname $helpFile]
    }
    
    set buf [errReadHelpFile $helpFile "r"]
    switch -- [string trim $buf " "] {} {
	set tmp "NO Help File for : $errMemo"

	if {[info exists mainShArea] && [info command $mainShArea] != ""} {
	    $mainShArea displayText $tmp
	    return
	} else {
	    tk_dialog .dialog "Dialog Box" $tmp info 0 OK
	    return
	}
    } 

    seq_displayTerm .txtDisplay "Help On Error" $helpFile  $buf
}


#-----------------------------------------------------------------------------
# PROCEDURE : errGetProcStack - Retrieves the process error stack.
#-----------------------------------------------------------------------------

proc ::internal::errGetProcStack {{option {}}} {
    variable errList
    variable indexList
    variable stackOverflow
    variable logStack
    variable stackId
    
    set stackSize [eval seq_errGetStackSize $option]

    if {[info exist errList]} {unset errList};	# effective way to clear it out

    for {set i 1 } {$i <= $stackSize} {incr i} {

	set tmp     [eval seq_errGetFromStack $option $i]
	set seqNumb [lindex $tmp 2]
	set stackId [lindex [lindex $tmp 1] 1]
	set    buf [lindex $tmp 8]
	append buf " [lindex $tmp 0 ] [lindex $tmp 4]"
	append buf " proc 0 62 [lindex $tmp 3] [lindex $tmp 5]"
	append buf " $stackId $seqNumb [lindex $tmp 6] [lindex $tmp 7]"

	set errList($seqNumb) $buf
    }

    if {$stackSize > 20 } {
	set stackOverflow "Stack Overflow"
	$logStack configure -state normal
    }

    updateStack $stackId
}    

#-----------------------------------------------------------------------------
# PROCEDURE : getStackOnLogFile - Reads the log file and retrieves the part 
#             of the stack which was already logged because a stack overflow 
#             occurred.
#             The optional parameter 'maxLen' is used to make a search on 
#             the last maxLen lines of the file
#
#-----------------------------------------------------------------------------

proc ::internal::getStackOnLogFile { stackId {file {}} {maxLen ""}} {
    variable errList
    variable indexList
    global ::env

    set stackFile  /tmp/stack_$env(USER)
    set logFile    $env(VLT_LOG_FILES)

    switch -- $file {} {
	append logFile "/logFile"
    } default {
	append logFile "/$file"
    }

    ###set dateFormat  "+ %y-%m-%d  %H:%M:%S"
    ###set today       [lindex [exec date $dateFormat] 0]
    # Remark that this time stamp is NOT according to VLT standards.
    # It should also be in UT! The correct command would be:
    #    clock format [clock seconds] -format " %Y-%m-%dT%H:%M:%S" -gmt 1
    set today [lindex [clock format [clock seconds] \
	    -format " %y-%m-%d  %H:%M:%S"] 0]

    if {[string length $maxLen] == 0} {
	exec errFilter.awk date=$today stackId=$stackId $logFile > $stackFile
    } else { 
	set tmpFile $env(HOME)/.errTailFile

	# This trick is needed otherwise a tail from the end of the file
	# is limited to a maximum 20k of data.

	set fromLine [lindex [exec wc -l $logFile] 0]
	if {[set fromLine [expr {$fromLine - $maxLen}]] < 1} {set fromLine 1}

	exec tail +$fromLine $logFile > $tmpFile
	exec errFilter.awk date=$today stackId=$stackId $tmpFile > $stackFile
	if {[file exists $tmpFile]} {exec rm -f $tmpFile}
    }

    set fh [open $stackFile "r" ];      
    
    while {[set line [gets $fh]] != "" } {
	set seqNumb [lindex $line 15]
	set errList($seqNumb) [lrange $line 5 end]
    }
    
    close $fh
    exec rm $stackFile
}    

#-----------------------------------------------------------------------------
# PROCEDURE :           
#-----------------------------------------------------------------------------

proc ::internal::errCompleteStack {} {
    variable stackId
    variable logStack
    getStackOnLogFile $stackId ;  updateStack
    $logStack configure -state disabled
}

#-----------------------------------------------------------------------------
# PROCEDURE : updateStack -        
#-----------------------------------------------------------------------------

proc ::internal::updateStack { {stackId {} } } {
    variable errStack
    variable errList
    variable indexList
    variable errCnt
    variable logStack

    set errCnt 1 
    
    foreach name [lsort -decreasing [array names errList]] { 
	set indexList($errCnt) $name ; incr errCnt
    }

    $errStack deleteItem 0 end

    if {[info exists indexList]} {
	$errStack updateSortList \
		"-decreasing" "-integer" errList ::internal::formatStackLine
	$errStack setCursor 1
	namespace eval ::internal {getErrInfo 1}
    } else {
	$errStack addItem 0 "Error Stack ($stackId) DOES NOT EXIST"
    }
}


#-----------------------------------------------------------------------------
# PROCEDURE : formatStackLine - Format the stack to be displayed           
#-----------------------------------------------------------------------------
#
# This procedure uses ctoken to select part of the log message because 
# the statement lindex can crash if in the log there are quotes,parenthesis ..
 
proc ::internal::formatStackLine { log } {

    # Split the log in order to access it as a list of values and
    # get rid of the additional {} element between day and time.
    # Kept for backward compatibility
    set text $log

    # Get rid of the first 7 elements
    for {set i 0} {$i < 7} {incr i} {
        ctoken text " "
    }

    set formatLine  "%-22s %-25s %-2s  %s"
    set location [ctoken text " "]

    ctoken text " " ; ctoken text " " ; ctoken text " "

    set severity [ctoken text " "]
    set mnemonic [ctoken text " "]
    set errText  [string trim $text " :"]

    if { $severity != "W" && $severity != "F" &&  $severity != "S" } {
	set severity "?"
    }    
    
    return [format $formatLine  $location $mnemonic $severity $errText ]
}         

#-----------------------------------------------------------------------------
# PROCEDURE :  getErrInfo - Updates the additional information for the error
#                           selected by the user                   
#-----------------------------------------------------------------------------

proc ::internal::getErrInfo { index } {
    variable errList
    variable infoFormat
    variable infoData
    variable indexList
    
    set index $indexList($index);   set text $errList($index)

    set date   [ctoken text " "]
    set time   [ctoken text " "]
    set env    [ctoken text " "]
    set module [ctoken text " "]
    set proc   [ctoken text " "]

   # Get rid of the next three tokens
    ctoken text " ";  ctoken text " "; ctoken text " "

    set numb   [ctoken text " "]

    set infoData  [format $infoFormat $date $time $env $module $proc $numb ]
    
    # Update help file, if the panel is already on the screen.
    
    if {[winfo exists .txtDisplay] } { errShowHelp }      
    
    return
}            

#-----------------------------------------------------------------------------
# PROCEDURE : errStackPanel - Creates the panel where to display the stack 
#-----------------------------------------------------------------------------

proc ::internal::errStackPanel { {rootWidget . } } {
    variable errList
    variable errCnt
    variable infoFormat
    variable infoData
    variable logStack
    variable errStack
    variable indexList
    variable mainShArea 
    variable stackOverflow

    switch -- $rootWidget "." { 
	set top  .errStackPanel
    } default {
	toplevel $rootWidget 
	set top  ${rootWidget}.errStackPanel
    }

    # If this panel is already displayed, simply return
    switch -- [info command $top] {} {} default {return}
    frame  $top ; pack $top
    
    set menuBar  [ makeMenuFrame $top]
    
    #######################################      
    #     Creation of main components
    #######################################

    set smallFont  "-Adobe-courier-medium-r-normal--12*"
    set infoFormat "%-8s  %-17s  %-9s  %-9s  %-16s  %-4s"
    
    set formatLine "%-22s %-25s %-2s  %s" ; # It is defined twice
    
    set f1       $top.f1 ;     # Contains the info names 
    set f2       $top.f2 ;     # Additional Info for error
    set f3       $top.f3 ;     # Stack overflow
    set errStack $top.stack;   # ListBox showing the stack
    set logStack $f3.logStack
    
    frame  $f1 ; frame $f3
    frame  $f2 -relief raised -bd 2
    
    UifListBox $errStack -geometry 110x8 -font $smallFont
    
    pack  $f1 $top.stack $f3 $f2 -fill x -padx 5 -pady 5
    
    #################################      
    #     Creation of Menu Bar   
    #################################

    #  File Menu
    set mbFile [addMenu $menuBar  File  1];
    addMenuCmd $mbFile "Print On Screen"  0 "::internal::errStackPrintOnScreen $errStack"
    addMenuCmd $mbFile "Quit"  0  "destroy $rootWidget "

    #  "Help" Menu
    set mbHelp [addMenu $menuBar  Help  1 right];      # Help Menu
    addMenuCmd $mbHelp "Extended Help"  0  {puts "Not Implemented"}  


    #######################################      
    #              Complete GUI
    #######################################

    set tmp [format $formatLine "Location" "Error Mnemonic and Severity" " " "Text"]
    label $f1.format -text $tmp
    
    pack $f1.format -side left
    
    # Fill frame f2
    label  $f3.infoTitle -text "Additional Information On Error "
    label  $f3.overflow  -textvariable ::internal::stackOverflow  
    button $logStack     -text "Show Complete Stack" \
	    -state disabled -command {::internal::errCompleteStack}
    
    pack $f3.infoTitle -fill x -side left -padx 30 -pady 5
    pack $f3.overflow  -side left
    pack $logStack     -side right -padx 20
    
    # Additional information on selected error
    
    frame $f2.r1 ; frame $f2.r2
    
    label  $f2.r2.infoData  -textvariable ::internal::infoData -font $smallFont

    set tmp [format $infoFormat "Date"    "Time" "EnvName" \
	    "Module" "Process" "err.Numb   "]

    label  $f2.r1.infoLabel -text $tmp 
    
    button $f2.r2.help  -text "Get Help On Error" -width 20 \
	    -command {::internal::errShowHelp}
    
    pack $f2.r1 $f2.r2 -fill x
    pack $f2.r1.infoLabel $f2.r2.infoData  -side left  -padx 5
    pack $f2.r2.help                       -side right -padx 10 -pady 10
    
    #
    # Set ShortHelp Area and Bindings
    #
    frame $top.help ; pack $top.help -fill x

    set mainShArea $top.help.sh ; UifShortHelp $mainShArea 
    pack $mainShArea -fill x -pady 5
    
    $errStack setBind  <B1-ButtonRelease>  \
	    "set ::internal::errCnt \[$errStack getIndex\] ;\
	    incr ::internal::errCnt ;\
	    namespace eval ::internal {getErrInfo \$errCnt}"
    
    UifShortHelp::add $mainShArea $f3.logStack \
	    "Retrieves the portion of the stack already logged in case of stack overflow"
    
    UifShortHelp::add $mainShArea $f2.r2.help \
	    "Shows the help associated to the selected error"
  
    wm group [winfo toplevel $top] .
    wm trans [winfo toplevel $top] .
    wm resizable $rootWidget 0 0
    wm title $rootWidget "VLT Tool - Error Display"
    return
}

#-----------------------------------------------------------------------------
# PROCEDURE : errStackPrintOnScreen - Dumps error stack on screen
#-----------------------------------------------------------------------------

proc ::internal::errStackPrintOnScreen { w } {
    foreach item [ $w getList ] {
	::puts $item
    }
    flush stdout
}


#-----------------------------------------------------------------------------
# PROCEDURE : errDialogPanel - Creates the panel for the errDialog 
#-----------------------------------------------------------------------------

proc ::internal::errDialogPanel { {rootWidget .} } {
    variable errDialogOK
    variable errDialogSTACK
    variable errDialogHELP
    variable myMemo

    switch -- $rootWidget "." {
	set top  .errDialogPanel
    } default {
	toplevel $rootWidget 
	set top  ${rootWidget}.errDialogPanel
    }

    frame $top

    set FONT    "-Adobe-helvetica-medium-r-normal--14*"

    set dialog  $top.f0;       frame $dialog -relief raised -bd 2
    set f1      $dialog.f1;    frame $f1 
    set f2      $dialog.f2;    frame $f2
    set f3      $dialog.f3;    frame $f3
    set but     $top.b0;       frame $but -relief raised -bd 2

    pack $top $dialog $f1 $f2 $f3 $but -fill x

    label $f1.bitmap   -bitmap "error" 
    label $f1.proc     -textvariable ::internal::myProc 
    label $f1.severity -textvariable ::internal::mySev  -font $FONT
    label $f1.lbl1     -text "ERROR in"      -font $FONT
    label $f2.msgTitle -text "Error Message" -font $FONT
    
    message $f3.errText -justify left -textvariable ::internal::myText \
	    -width 15c -bd 2 -relief groove 

    pack $f1.bitmap $f1.severity  -side left -fill x -padx 3  -pady 10
    pack $f1.lbl1   $f1.proc      -side left -fill x -padx 5  
    pack $f2.msgTitle             -side left -fill x -padx 10 
    pack $f3.errText   -side left -fill x -padx 10 -pady 3 -ipadx 3 -ipady 4

    set frameOK        $but.fOK
    set errDialogOK    $frameOK.ok
    set errDialogSTACK $but.stack
    set errDialogHELP  $but.help 

    frame   $frameOK        -relief sunken -bd 1
    button  $errDialogSTACK -text "Show Stack" -font $FONT
    button  $errDialogOK    -text "OK"  -font $FONT \
	    -command "catch {destroy ${rootWidget}}"
    button  $errDialogHELP  -text "Error Help" -font $FONT \
	    -command "::internal::errShowHelp ${myMemo}.hlp"

    pack $frameOK -side left -padx 5 -pady 5 -expand 1

    pack  $errDialogHELP $errDialogSTACK  -side left -padx 5 \
	    -pady 5 -expand 1 -ipadx 8 

    pack $errDialogOK -padx 3 -pady 3 -ipadx 8 

    wm group [winfo toplevel $top] .
    wm trans [winfo toplevel $top] .

    wm resizable ${rootWidget} 0 0 ; # Disable window resizing
    return $errDialogSTACK;		# return widget name of this button
}


#-----------------------------------------------------------------------------
# PROCEDURE : errReadHelpFile - Reads the help file and strips off the
#                               file header. It returns the file content.
#-----------------------------------------------------------------------------

proc ::internal::errReadHelpFile { helpFile mode } {
    switch -- $mode r {
	set helpPath [findVLTFile ERRORS/HELP/${helpFile} no ]
    } default {
	set helpPath [findModuleFile ERRORS/HELP/${helpFile}]   
    }
    
    switch -- $helpPath {} { return "" }
    
    set fileSize [file size $helpPath]   
    set fh       [open $helpPath "r"]

    # Skip header
    
    while { [gets $fh buf] != -1 } {
	set char [string index $buf 0 ]
	if {$char != "#" && $char != "\n" } { break }
    }

    append buf [read $fh $fileSize]  
    close  $fh
    return $buf
}
