#*************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqInit.tcl,v 2.77 2003/03/18 19:44:14 vltsccm Exp $" 
#
# who      when       what
# -------- --------   ------------------
# eallaert 22/03/95   created for cmm
# eallaert 23/03/95   if interactive, don't pop up wish-window (SPR 950132)
# eallaert 28/07/95   is also executed in full absence of CCS
# eallaert 19/09/95   put in variable tcl_rcFileName for Tk4.x
# eallaert 15/11/95   only put in wm-commands if Tk is present
# eallaert 19/11/95   removed use of global vars seq_exitCmd and seq_scriptCmdEval
# mcomin   24/09/96   Replaced info exist with file exist ..
# eallaert 25/09/96   Implemented other workaround for tearoff menu problem
# mcomin   26/09/96   Resource file is split into XseqWish.private and XseqWish
# eallaert 08/04/97   Adapted to load RTD dynamically
# eallaert 14/07/97   Adapted for tkcon
# eallaert 1998-02-16 Made the seqWish icon a coloured one
# eallaert 1998-06-04 Define seq_logData for NOCCS
# eallaert 1998-07-02 Setting for auto_path takes package loading into account
#                     (SPR 980338)
# eallaert 2000-06-08 removed DFLOW_ROOT refs (no longer part of VLT stdEnv)
#                     check for HOSTNAME instead of HOST
# eallaert 2002-05-12 include "package require Itcl/Itk"
# eallaert 2002-07-03 remove load of seqXpm package (Img is available everywhere)
# eallaert 2002-10-24 take MODROOT into account (SPR 20020574)
# eallaert 2002-10-30 don't use special inputmethods (cd tk manpage)
# eallaert 2002-11-12 moved code for seqWish icon to seqCon.tcl
# eallaert 2002-11-27 now also with tkcon only single pass through this file
# eallaert 2003-03-18 fixed problems with Tcl's package database
#
# seqInit.tcl --
#
# Execute a number of commands to initialize the Sequencer

#***************************************************************************
#   NAME 
#   seqInit.tcl - Execute a number of commands to initialize the Sequencer
#  
#   SYNOPSIS 
#   source seqInit.tcl
#
#   DESCRIPTION
#   This is executed automatically at startup by seqInit.c;  There is no need
#   to call it again.
#  
#   RETURN VALUES
#   as any Tcl procedure
#------------------------------------------------------------------------ 
# Sequencer startup file.

# Use only standard Tcl commands, otherwise there will be problems with
# pkg_mkIndex !!!!!
#-------------------------------------------------------------------------
# Strip the "--" off if it is the first argument.  This is due to an 
# inconsistent behaviour of different Tcl shells - see SPR 960174.
if {[info exists argv]} {
    if {[llength $argv] > 0 && ![string compare [lindex $argv 0] "--"]} {
	puts "argv = $argv"
	set argv [lreplace $argv 0 0]
    }
} else {
    # The global var "argv" does not exist in a new interpreter, like
    # when the pkg_mkIndex is run.
    # A trick for package handling! Say here that we'll get all Tcl
    # commands as soon as we "load" the Seq module
    set me [info script]
    foreach f [glob -nocomplain -- [file dirname $me]/seq*.tcl] {
	if {[string compare $f $me]} {source $f}
    }
    unset me
}

#--------------------------------------------------------------------------
# Before the first "package require" of an unknown package (which will 
# trigger Tcl's "package unknown" procedure tclPkgUnknown), make sure that
# auto_path includes our VLT standard directories. Otherwise, Tcl's package
# database may exclude some of our dirs (e.g. INTROOT), but still be
# indicating that a package is available (thereby not forcing a pass
# through tclPkgUnknown) when a later "package require" is given. This means
# that "older" versions would get loaded, while newer are available!
#
# The same net result would be obtained by a forced "tclPkgUnknown" at
# the end of this file, but that takes another 200-300 ms (versus the
# extra ~8 ms due to the fiddling with auto_path in the next few lines).
set orig_auto_path $auto_path
set auto_path {}
foreach dir {MODROOT INTROOT VLTROOT TCLTK_ROOT} {
    if {[info exists env($dir)] && [file isdirectory $env($dir)/lib]} {
	lappend auto_path $env($dir)/lib
    }
}
set auto_path "$auto_path $orig_auto_path"

# Make some packages available asap, i.e. before using any of their instructions.
# As its libs are at the same level as the Tcl ones, and as Tcl has been
# loaded already, auto_path is set properly for "package require" to find itcl.
package require Itcl 3.2
# For compatibility reasons with [incr Tcl] 2.2:
namespace import itcl::*

# set the auto_path back to what it was - it will be finalized later on. 
set auto_path $orig_auto_path
unset orig_auto_path
#-------------------------------------------------------------------------- 
# Some of the procs in seqLib.tcl are needed further down, and we want to
# avoid a call to "unknown" 
source [file join ${seq_library} seqLib.tcl]
#--------------------------------------------------------------------------
# Adjust the auto-load path.
# tkcon 2.3 seems to add an empty dir to auto_path at some point during its 
# init-phase, and pass it on to its slave.
while {[set idx [lsearch -exact $auto_path {}]] >= 0} {
    set auto_path [lreplace $auto_path $idx $idx]
}

# seqInit.c has made sure that $seq_library is an absolute path.
# Make sure it is the first path in the list.
while {[set idx [lsearch -exact $auto_path $seq_library]] >= 0} {
    set auto_path [lreplace $auto_path $idx $idx]
}
set auto_path "$seq_library $auto_path"

# Tcl 8.3 seems to add the directory where libseq.sl is found
###while {[set idx [lsearch -exact $auto_path [file dirname $seq_library]]] >= 0} {
###    set auto_path [lreplace $auto_path $idx $idx]
###}
#---------------------------------------------------------------------------
# Define 2 internal debugging procedures, which are called by all seq_*
# commands written in pure Tcl
# Remark that as a side-effect, the namespace "::internal" gets created,
# which is accessed by the seq_err* procedures.
namespace eval ::internal {
    proc seq_initDbg {name argList} {
	global seq_debug
	if {$seq_debug($name) & 4} {
	    puts -nonewline stderr "[seq_isoTime] "
	}
	if {$seq_debug($name) & 1} {
	    puts stderr "(#[expr {[info level] -2}]) $name $argList"
	}
    }
    
    proc seq_returnDbg {name stat val} {
	global seq_debug
	if {$seq_debug($name) & 2} {
	    puts stderr "     $stat >$val<"
	}
    }
}

#-------------------------------------------------------------------------
# Rename Tcl's original unknown proc, and put it in the list of
# unknown procs to call. Other unknown procs added later will get
# called before this one.
switch -- [info comm tcl_unknown] {} {rename unknown tcl_unknown}
set unknown_handlers(tcl) tcl_unknown
set unknown_handler_order tcl

proc unknown {args} {
    global unknown_handler_order unknown_handlers errorInfo errorCode

    #
    # Be careful to save error info now, and restore it later
    # for each handler.  Some handlers generate their own errors
    # and disrupt handling.
    #
    set savedErrorCode $errorCode
    set savedErrorInfo $errorInfo
    
    foreach handler $unknown_handler_order {
	set status [catch {uplevel $unknown_handlers($handler) $args} result]
	
	if {$status == 1} {
	    #
	    # Strip the last five lines off the error stack (they're
	    # from the "uplevel" command).
	    #
	    set new [split $errorInfo \n]
	    set new [join [lrange $new 0 [expr {[llength $new] - 6}]] \n]
	    return -code $status -errorcode $errorCode  -errorinfo $new $result
	    
	} elseif {$status != 4} {
	    return -code $status $result
	}
	
	set errorCode $savedErrorCode
	set errorInfo $savedErrorInfo
    }
    
    set name [lindex $args 0]
    return -code error "invalid command name \"$name\""
}

proc unknown_handler {name args} {
    
    global unknown_handler_order unknown_handlers
    
    set unknown_handlers($name) $args
    set unknown_handler_order [linsert $unknown_handler_order 0 $name]
}
    

# We will delete the "internal" command seq_errDisplay if Tk and CCS are in.
# That way auto_load will pick up the version contained in seqErrDisplay.tcl
# Append also the path of the uif-lib, as seq_errDisplay needs it.
# Makes only sense if Tk is used and CCS (light or full) is linked in.
if {[info exists tk_version]} {
    tk useinputmethods 0;	# we don't have special (Japanese) input devices
    package require Itk 3.2
    package require BLT 2.4
    namespace import itk::*
    package require Iwidgets
    namespace import iwidgets::*
    if {[catch {seq_findFile lib/libuif.tcl} uiflib]} {
	puts stderr $uiflib
	switch -- $seq_ccsType full - light {
	    puts stderr "using external version of seq_errDisplay"
	}
    } else {
	switch -- $seq_ccsType full - light {
	    rename seq_errDisplay {}
	}
	# Add the uiflib-path anyway to auto_path; even if CCS is not in, this
	# might come in handy.
	if {[lsearch -exact $auto_path $uiflib] < 0} {
	    lappend auto_path $uiflib [file dirname $uiflib]/libuifClass.tcl
	}
    }
    unset uiflib
}

# To take care of dynamic loading of shared libs, the manpage of pkg_mkIndex
# instructs to use the global var tcl_pkgPath. However, this does not seem
# to work as documented, and we need to fall back on auto_path.
# The order of package-load directories must be: 
#   $MODROOT/lib, $INTROOT/lib, $VLTROOT/lib, $TCLTK_ROOT/lib, "own" 
#  ("own" contains typically only the lib for tcltk, which is probably 
#  the same as the one under TCLTK_ROOT; however, when one tries out
#  new versions of Tcl/Tk, there might be some difference, so we'd better
#  take it in *before* TCLTK_ROOT).
# We want auto_path to contain these dirs containing somewhere pkgIndex.tcl
# files completely at the end.
# Remark that all these environment vars need to exist in the VLT std env,
# except then INTROOT. But do not complain too hard if one of them does not
# exist - some installations of Seq have e.g. nothing of Data Flow SW.
#
set orig_tcl_pkgPath $tcl_pkgPath
set tcl_pkgPath {}
foreach dir {MODROOT INTROOT VLTROOT TCLTK_ROOT} {
    if {[info exists env($dir)] && [file isdirectory $env($dir)/lib]} {
	# the environment variable is set, it points to an existing dir ...
	if {[set idx [lsearch -exact $tcl_pkgPath $env($dir)/lib]] < 0} {
	    # ... and this directory is not yet in tcl_pkgPath
	    lappend tcl_pkgPath $env($dir)/lib
	}
    } elseif {![string equal $dir MODROOT] && ![string equal $dir INTROOT]} {
	# environment variable is not set. INTROOT can be excused
	switch -- [info command seq_errAdd] {} {} default {
	    # Remark that by calling errResetStack (or any other CCS function)
            # the current setting of RTAPENV is taken; this value will be
	    # used for ccsInit(), independent of RTAPENV's value at the
	    # time ccsInit() is given. Warn about this.
	    seq_errResetStack
	    seq_errAdd "The environment variable $dir is not set" \
		    [info script]
	    switch -- $seq_ccsType light {
		seq_errAdd "(Remark that due to this error-call you cannot\
			effectively change RTAPENV - see seq_ccsInit(n))" \
			[info script]
	    }
	    seq_errCloseStack
	}
	if {$tcl_interactive} {
	    puts stderr \
		    "The environment variable $dir is not set, although it is\
		    \nper definition part of the VLT standard environment.\
		    \nYou won't have automatic access to any packages\
		    installed under that path."
	}
    }
}

# Add the original settings of tcl_pkgPath to the end (no duplicates).
foreach dir $orig_tcl_pkgPath {
    if {[lsearch -exact $tcl_pkgPath $dir] < 0} {
	lappend tcl_pkgPath $dir
    }
}

unset orig_tcl_pkgPath

# Remove any of these directories already in auto_path, so we can add the
# entire list at the tail, in the "proper" order.
foreach dir $tcl_pkgPath {
    while {[set idx [lsearch -exact $auto_path $dir]] >= 0} {
	set auto_path [lreplace $auto_path $idx $idx]
    } 
    lappend auto_path $dir
}
catch {unset dir idx};	# if the first one exists, then the other as well

set shlibext [info sharedlibextension]
# Now set the proper X-resources
if {[info exists tk_version]} {

    # Reset MOTIF style, otherwise ESO conventions do not work
    set tk_strictMotif 0
    
    # Load resources from a single place, in the following search order:
    #           - ./app-defaults
    #           - ../app-defaults        
    #           - $MODROOT/app-defaults        
    #           - $INTROOT/app-defaults  
    #           - $VLTROOT/app-defaults 
    if {![catch {seq_findFile app-defaults/XseqWish.private} rc]} {
        option readfile $rc
    }

    if {![catch {seq_findFile app-defaults/XseqWish} rc]} {
        option readfile $rc
    }
    unset rc

    # Make the multiple column listbox widget available
    source [file join ${seq_library} tablelist.tcl]
    # Remove the (in our case faulty) entry it made in auto_path
    set idx [lsearch -exact $auto_path $seq_library/scripts]
    set auto_path [lreplace $auto_path $idx $idx]
    unset idx

    # Img has a pkgIndex file under TCLTK_ROOT/Img1.x, so there is no need
    # to give a "package ifneeded Img" command
    catch {package require Img}
}

# oslx available with full CCS, CCS-light, NOCCS; version is same as seq
package ifneeded SeqOslx [infox appversion] \
	"load [file join [file dirname $seq_library] libseqOslx$shlibext]"

# msqltcl may be required
package ifneeded msqltcl 1.99 \
	"load [file join [file dirname $env(TCLTK_ROOT)/lib/x] libmsqltcl1.99$shlibext]"
# A "package require msqltcl" will fail if the static lib libmsql.a is not
# linked into the application (e.q. applications using seqInitInterp).
# It should in any case be considered the task of the application script
# to issue this "package require" command.
###catch {package require msqltcl 1.99}

unset shlibext
#---------------------------------------------------------------------------
# Define the seq_logData command for NOCCS (the logging system is out,
# so we need to give a stub).
# We will define the seq_logData command as a dump to stderr.
switch -- $seq_ccsType none {
    proc seq_logData {log} {
	puts stderr $log
    }
} default {
    # rename the "standard" seq_ccsExit command, and redefine the seq_ccsExit
    # command to include a "seq_ccsAsyncInput close" if appropriate
    # Test if the command exists (it won't if compiled without CCS)
    if {![string compare [info comm seq_ccsExit] "seq_ccsExit"] && \
	    ![string compare [info comm seq_ccsExitOrig] ""]} {
	rename seq_ccsExit seq_ccsExitOrig
    } else {
	proc seq_ccsExitOrig {} {
	    return
	}
    }
    
    proc seq_ccsExit {} {
	global seq_ccsStatus
	if {[info exists seq_ccsStatus]} {
	    if {$seq_ccsStatus == "OPEN"} {seq_ccsAsyncInput close}
	}
	seq_ccsExitOrig
    }
}

#---------------------------------------------------------------------------
switch -- [info comm tcl_exit] {} {
    # rename the "standard" exit command
    rename exit tcl_exit

    # Redefine the exit command so it includes a call to ccsExit if
    # seq_ccsStatus exists.
    # The exit-value is defaulted to 0 (success).
    proc exit {{value 0}} {
	global seq_ccsStatus
	
	if {[info exists OSE]} {seq_logTclErrors close}
	
	if {[info exists seq_ccsStatus]} {seq_ccsExit}
	
	tcl_exit $value
    }
}

proc quit {} {
    exit 0
}

#---------------------------------------------------------------------------
# Do an errResetStack, which sets the environment name in the error structure.
### Don't do this on CCS-lite: the first CCS-call (ie. errResetStack) 
### will pick up the RTAPENV setting, such that later mod's of RTAPENV
### are not possible (which would clash with e.g. vcc)
###switch -- $seq_ccsType full - light 
# Leave it in for full CCS, for backward compatibility reasons.
switch -- $seq_ccsType full {
    seq_errResetStack
}
#---------------------------------------------------------------------------
# Define icon-bitmap (if Tk is in). An icon bitmap or colour pixmap should 
# only be used if a proper one can be found in the standard search path; 
# If an application wants to define its iconwindow with its own pixmap,
# with the window already on the screen, CDE will require a 
# "update; wm withdraw .; wm deiconify ." - which makes the panel "flash"
# on the screen.
# For b/w bitmaps the situation is simpler: define them anytime.
# The name of the icon-file is here derived from the application name.
if {[info exists tk_version]} {
    set procName [file tail $argv0]
    if {[info exists env(HOSTNAME)]} {
	set host $env(HOSTNAME)
    } elseif {[info exists env(HOST)]} {
	set host $env(HOST)
    } else {
	set host [info hostname]
    }
    catch {wm title . ${procName}@$host}
    if {![catch {seq_findFile bitmaps/${procName}Icon.gif} pm]} {
	image create photo seqIcon -file $pm
	toplevel .seqIcon
	pack [label .seqIcon.l -image seqIcon]
	wm iconwindow . .seqIcon
    } elseif {![catch {seq_findFile bitmaps/${procName}Icon.xbm} bm]} {
	wm iconbitmap . @$bm
    }
    catch {wm iconname . ${procName}@$host}
    # order is important: "procname", "host" and "pm" exist for sure
    # ("unset" will not delete any var coming after the 1st non-existing one)
    catch {unset procName host pm bm}
}
#---------------------------------------------------------------------------
# Default: no splash screen
set splash 0
set splashLogo tclpower.gif
# Parse command line options (look for -splash option)
# (This code should possibly be in seqCon as well).
while {![lempty $argv]} {
    set arg [lvarpop argv]
    switch -glob -- $arg {
	-splash {set splash 1}
	-splashLogo {set splashLogo [lvarpop argv]}
	default	{lappend rest $arg}
    }
}
if {[info exists rest]} {
    set argv $rest
    unset rest
}
catch {unset arg}
# Remove window if interactive (SPR 950132) and Tk part of shell
if {$tcl_interactive  && [info exists tk_version]} {
    wm withdraw .
}
if {[info exists tk_version] && $splash} {
    if {![catch {seq_findFile bitmaps/$splashLogo} logo]} {
	toplevel .top
	frame .top.splash -borderwidth 4 -relief raised
	set logo [ image create photo -file $logo ]
	label .top.splash.logo -image $logo
	pack  .top.splash.logo -fill both
	place .top.splash -anchor center -relx .5 -rely .5
	namespace eval internal {
	    proc checkSplash {logo} {
		if {[winfo ismapped .] || \
			([winfo exists .tkcon] && [winfo ismapped .tkcon])} {
		    destroy .top
		    rename checkSplash {}
		    image delete $logo
		} else {
		    after 250 ::internal::checkSplash $logo
		}
	    }
	}
	::internal::checkSplash  $logo
	update
    } else {
	error "Splash logo missing: $logo"
    }
    unset logo
}
#---------------------------------------------------------------------------
# Define global variable tcl_rcFileName (for tclversion > 7.4)
if {[info exists tk_version]} {
    set tcl_rcFileName ~/.seqWishRc
} else {
    set tcl_rcFileName ~/.seqShRc
}

#---------------------------------------------------------------------------
# Next contains a re-definition of the Menu <Leave>-binding, as a workaround
# for a problem in Tk, which removes the activation of a menu-item on a
# tear-off menu, if button1 is pressed on it.  This occurs with HP VUE 3.0
# under certain conditions - see e-mail to vltsoft and J. Ousterhout of
# 25 September 1996.  Summarizing, it is because HP VUE generates 3
# events when button1 is pressed: a <Leave>-, an <Enter>- and a <B1>-event,
# instead of only the latter one.  So we'll try to suppress the phoney
# <Leave>-events, which deactivate menu-items.
# The phoney <Enter>-events do not have bad side-effects for menus, so
# we'll leave them as they are.

if {[info exists tk_version]} {
    bind Menu <Leave> {
	set rootx [winfo rootx %W]
	set rooty [winfo rooty %W]
	if {%X <= $rootx || %X >= [expr $rootx + [winfo width %W]] \
		|| %Y <= $rooty || %Y >= [expr $rooty + [winfo height %W]]} {
	    tkMenuLeave %W %X %Y %s
	}
    }
}

#---------------------------------------------------------------------
# Upon destruction of the window, ensure that ccsExit gets called (which
# is part of our re-defined "exit" command
if {[info exists tk_version]} {
   wm protocol . WM_DELETE_WINDOW {exit}
}
#---------------------------------------------------------------------
# Define the bgerror proc if there is no Tk
if {![info exists tk_version]} {
    proc bgerror {msg} {
	global errorInfo errorCode
	puts stderr "Background error: $msg"
	puts stderr "Error stack: $errorInfo"
	##puts stderr "Error code: $errorCode"
    }
}
#---------------------------------------------------------------------
if {!$tcl_interactive} return

# == Interactive Tcl session initialization ==

if {![info exists tcl_prompt1]} {
    set tcl_prompt1 "puts -nonewline stdout [file tail $argv0]>"
}
if {![info exists tcl_prompt2]} {
    set tcl_prompt2 {puts -nonewline stdout =>}
}

