#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: seqProgressBar.tcl,v 2.77 2003/03/18 19:44:23 vltsccm Exp $" 
#
# seqProgressBar.tcl
#
#
# who        when        what
# ---------  ----------  ----------------------------------------------
# eallaert   2000-12-19  initial implementation, based on istDiskMon

#
#************************************************************************
#   NAME
#      SeqProgressBar - panel editor class to display a progress bar
#                     
#   SYNOPSIS
#      SeqProgressBar <windowName> <varName> ?<options>?
#
#   DESCRIPTION
#      SeqProgressBar is a panel editor class that displays a barchart showing
#      the progress of some action. A new object will get the name <windowName>
#      and the parent-window of <windowName> must exist at the time the object
#      is created. The progress bar is linked with the global variable 
#      <varName>, and the barchart is updated whenever that global variable is
#      touched.
#   
#   OPTIONS
#     Options represent public variables of the object and can be set with:
#
#       <objectName> configure -<option1> <value1> ...
#
#      SeqProgressBar accepts following configuration options:
#       -font <fontName> : the font for the text to be displayed
#       -min  <minVal>   : the minimum value on the barchart; 0 by default
#       -max  <maxVal>   : the maximum value on the barchart; 100 by default
#       -text <text>     : the text (message) to appear within the object;
#                          empty by default
#       -variable <var>  : name of the global variable which is linked to the
#                          progress (should have a value between <minVal> and
#                          <maxVal>)
#
#   ENVIRONMENT VARIABLES
#   
#   EXAMPLE
#    seqWish> toplevel .top
#    seqWish> set day [clock format [clock seconds] -format %j]
#    seqWish> toplevel .top
#    seqWish> SeqProgressBar .top.progress day -text "Year 2001 Progress" \
#             -max 365
#    seqWish> pack .top.progress
#    seqWish> wm title .top "My 2001 Progress Bar"
#
#------------------------------------------------------------------------
#
#########################################################################
itk::usual SeqProgressBar {
    keep -relief -borderwidth -background -foreground
}

itcl::class SeqProgressBar {

    inherit UifOutputWidget

    constructor {args} {}
    destructor  {}

    itk_option define -font      font      Font      {{new century schoolbook} 14}
    itk_option define -min       min       Minimum   0
    itk_option define -max       max       Maximum   100
    itk_option define -text      text      Text      "Progressing ..."
    itk_option define -variable  variable  Variable  ""

}; # end of class


#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------

itcl::body SeqProgressBar::constructor {var args} {
    itk_component add lbl {
	label $itk_component(hull).lbl -relief flat
    } {
	keep -text -font
    }

    set itk_option(-variable) $var
    itk_component add bar {
	UifBarChart $itk_component(hull).bar -barColor Blue \
		-orient h -length 300 -variable $var
    } {
	keep -variable -minVal -maxVal
    }
   
    pack $itk_component(lbl) $itk_component(bar) -padx 10 -pady 10

    eval itk_initialize $args
}

itcl::body SeqProgressBar::destructor {args} {

    catch { after cancel $afterId }
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------  

itcl::configbody SeqProgressBar::text {
    $itk_component(lbl) configure -text $itk_option(-text)
}
#---------------------------------------------------------------
# OPTION : -font
#---------------------------------------------------------------  

itcl::configbody SeqProgressBar::font {

    $itk_component(lbl) configure -font $itk_option(-font)
}
#---------------------------------------------------------------
# OPTION : -min
#---------------------------------------------------------------  

itcl::configbody SeqProgressBar::min {

    $itk_component(bar) configure -minVal $itk_option(-min)
}
#---------------------------------------------------------------
# OPTION : -max
#---------------------------------------------------------------  

itcl::configbody SeqProgressBar::max {

    $itk_component(bar) configure -maxVal $itk_option(-max)
}


#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------  

itcl::configbody SeqProgressBar::variable {
    $itk_component(bar) configure -variable $itk_option(-variable)
}



#___oOo___




