#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclDictionary.tcl,v 1.17 2002/04/04 11:04:44 vltsccm Exp $"
#
# slxtclDictionary.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  02/05/97  created
#
#************************************************************************
#   NAME
#   slxtclDictionary - Load contents of a setup file in a GUI panel
# 
#   SYNOPSIS
#   slxtclDictionary <dictionaries> [<inst>]
# 
#   DESCRIPTION
#   Tcl procedure, used within GUI panels built with the VLT panel editor, 
#   to load in the specified instance <inst> of olsx, embedded in the
#   Sequencer, the specified list of <dictionaries>.
#   This function has to be called before any call to other slxtcl functions,
#   such as slxtclSaveSetup or slxtclLoadSetup
#   If <inst> is not specified, default is "slxtcl"
#
#   FILES
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   FAILURE if error occurs
#
#   EXAMPLES
#   # 1
#   # load dictionaries TCS UVES-ICS and CCDDCS
#   set dictionaries {TCS UVES-ICS CCDDCS}
#   slxtclDictionary $dictionaries

#   SEE ALSO
#   slxtclSaveSetup
#   slxtclLoadSetup
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#

proc slxtclDictionary { dictionaries {inst slxtcl} } {   

  set nDic [llength $dictionaries]
  for { set i 0} { $i < $nDic } {incr i} {
     set dic [lindex $dictionaries $i]
     set status [slxtclOslxIf OSETUP -loadDic $inst $dic merge]
     if {$status == "FAILURE"} {break}
  }
  return $status
}

#
# ___oOo___


