#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclOslxIf.tcl,v 1.17 2002/04/04 11:04:44 vltsccm Exp $"
#
# slxtclOslxIf.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  02/05/97  created
# alongino  15/10/97  made public
# alongino  05/03/98  removed the "after" instruction for seq_errDisplay
#
#************************************************************************
#   NAME
#   slxtclOslxIf - Interface between slxtcl library and embedded oslx
# 
#   SYNOPSIS
#   slxtclOslxIf <command> <param> <inst> <args>
# 
#   DESCRIPTION
#   Tcl procedure is used to get services from oslx embedded in the Sequencer.
#   It passes to specified instance <inst> of oslx the specified <command>
#   <param> <args>.
#   It returns the result as from oslx or FAILURE in case of error.
#   
#   FILES
#   <file>          setup file to be read
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   FAILURE if an error occurred.
#   If no error, last reply from oslx
#
#   EXAMPLES
#   # 1
#   # Save the contents of buffer oslxBuf into file
#   set status [slxtclOslxIf OSETUP -clear slxtcl]
#   if {$status != "FAILURE"} {
#     set status [slxtclOslxIf OSETUP -storeKeyword slxtcl $oslxBuf]
#   }
#   if {$status != "FAILURE"} {
#     set status [slxtclOslxIf OSETUP -saveAs slxtcl $file "\"$type\""]
#   }
#
#   SEE ALSO
#   slxtclReadSetup, slxtclWriteSetup, slxtclDictionary
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclOslxIf { command param inst args } {
  global gvar

  if {![info exists gvar(includedOslx)]} {
    package require SeqOslx
    set gvar(includedOslx) 1
  }
  if {[catch {eval seq_oslxCmd $command $param $inst $args} ret]} {
    # the "after" commands postpones the execution of errDisplay. If
    # the Sequencer script does not get idle (e.g. through update), it
    # can take much more than the foreseen 100 msec to pop-up the error
    # display. If meanwhile the error stack has changed, because some other
    # error occured, what one sees on errDisplay is NOT the original error
    # any more.
    # In addition, the suspend option should be used as well
    #after 100 seq_errDisplay
    seq_errDisplay -suspend
    set ret "FAILURE"
  }
  return $ret
}


#
# ___oOo___


