#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclPanToBuf.tcl,v 1.17 2002/04/04 11:04:42 vltsccm Exp $"
#
# slxtclPanToBuf.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/05/96  created
#
#************************************************************************
#   NAME
#   slxtclPanToBuf - Extract from a GUI panel setup keywords
# 
#   SYNOPSIS
#   slxtclPanToBuf
# 
#   DESCRIPTION
#   Tcl procedure to get the list of all variables in a GUI panel (built with
#   the VLT panel editor), which follow the syntax for setup variables 
#   (see slxtclSaveSetup).
#
#   The list of variable is filtered according to the filter settings
#   applied by previous calls to slxtclFilterSetup.
#
#   This procedure is intended to be used in cases where the list of keywords
#   and values extracted from the GUI panel has to be manipulated, e.g.
#   using functionality provided by oslx. For those cases where
#   the list has to be saved in a setup file as it is, it is recommended to
#   use slxtclSaveSetup.
#
#   The keywords associated to the variables and their current value are 
#   returned to the caller in a string with the following format:
#   <keyword1> <value1> ... <keywordn> <valuen>
#   This string can be used as it is as parameter value in the command
#   OSETUP -storeKeyword to oslx (see [2])
#
#   It belongs to the library libslxtcl.tcl; therefore, in order to be able
#   to use it, one has to register the panel edited with the Panel Editor
#   to this library.
#
#   RETURN VALUES
#   string containing name and values of the keywords found after filtering
#
#   EXAMPLES
#   # 1
#   # get keywords from panel, filter out DET keywords, and store the
#   # remaining in local variable
#   slxtclFilterSetup a *DET*
#   set oslxBuf [slxtclPanToBuf]
#   
#   # 2
#   # remove previous filtering, get all keywords from panel and store in 
#   # local variable
#   slxtclFilterSetup r all
#   set oslxBuf [slxtclPanToBuf]
#   
#   # 3
#   # get keywords from panel and put them in a oslx setup buffer 
#   set oslxBuf [slxtclPanToBuf]
#   seq_oslxCmd OSETUP -storeKeyword slxtcl $oslxBuf
#
#   SEE ALSO
#   slxtclSaveSetup, slxtclFilterSetup
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclPanToBuf { } {                                             

 # Declare global variables

   global gvar slxFits slxFitsValue
   global slxFitsFilter
   global slxFitsIndex slxFitsTmp 
   global slxFitsVarMod

   set slxFits {}
   set slxFitsValue {}
   #
   # extract all setup variables information
   #
   set varName [slxtclSetupList f]

   #
   # reorder according to order defined in panel
   #
   set slxFitsIndex [lsort -integer $slxFitsIndex]
   set leng [llength $slxFitsIndex]
   set count 0 
   while {$count < $leng} {
       set i [lrange $slxFitsIndex $count $count]
       set i [join $i]
       set index [lsearch $slxFitsVarMod "*.$i.*"]
       set index [join $index]
       set short [lrange $slxFitsTmp $index $index]
       set short [join $short]
       set name [lrange $varName $index $index]
       set name [join $name]
       set value $gvar($name)
       # add quotes to empty strings. Otherwise oslx gets confused
       if {[string length $value] == 0 } { set value "\" \"" }
       lappend slxFits $short
       lappend slxFitsValue $value
       # take out from slxFitsVarMod variable just handled to avoid problems
       # with keywords having the same index
       set slxFitsVarMod [lreplace $slxFitsVarMod $index $index "NULL"]
       incr count
   };  # end while

   #
   # Build buffer for oslx
   #
   set keyL {}
   set leng [llength $slxFits]
   set count 0
   while {$count < $leng} {; # build one list name value name value etc.
     lappend keyL [lindex $slxFits $count]
     lappend keyL [lindex $slxFitsValue $count]
     incr count
   }
   set oslxBuf [join $keyL]

   return $oslxBuf
}


#
# ___oOo___


