#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclReadSetup.tcl,v 1.17 2002/04/04 11:04:43 vltsccm Exp $"
#
# slxtclReadSetup.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/05/96  created
# alongino  09/07/96  added check on oslxServer error replies
# alongino  02/05/97  interface to oslx changed
# alongino  15/10/97  made public
#
#************************************************************************
#   NAME
#   slxtclReadSetup - Read setup keywords and values from a setup file
# 
#   SYNOPSIS
#   slxtclReadSetup <file> <inst>
# 
#   DESCRIPTION
#   Tcl procedure, used by slxtclLoadSetup, to retrieve information about
#   short FITS keywords contained in a setup file.
#   It returns keywords name and value in a string of the same format as
#   retruned by oslx.
#   For the meaning of parameters see slxtclLoadSetup.
#   
#   FILES
#   <file>          setup file to be read
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   FAILURE if error
#   string as returned by oslx
#
#   CAUTIONS
#   1) This procedure does NOT check if the appropriate data dictionary has 
#      been loaded through slxtclDictionary. The application has to make 
#      sure that this is true.
#
#   EXAMPLES
#   # 1
#   # extract from setup file myRef the list of keywords and values, store 
#   # them in the oslx setup buffer slxtcl and then load the values in the 
#   # panel
#   slxtclDictionary CCDDCS
#   set oslxBuf [slxtclReadSetup myRef slxtcl]
#   slxtclBufToPan $oslxBuf
#
#   SEE ALSO
#   slxtclLoadSetup, slxtclBufToPan, slxtclDictionary
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclReadSetup { file {inst slxtcl} } {

   set command "OSETUP"
   #
   # Clean buffer
   #
   set status [slxtclOslxIf $command -clear $inst]

   #
   # Load contents of setup file in buffer
   #
   if {$status != "FAILURE"} {
     set status [slxtclOslxIf $command -load $inst $file]
   }

   #
   # get buffer contents
   #
   if {$status != "FAILURE"} {
     set oslxBuf [slxtclOslxIf $command -getSetupBuf $inst]
   }

   #
   # return reply from oslx
   #
   if {$status == "FAILURE"} {set oslxBuf $status}
   return $oslxBuf
}


#
# ___oOo___


