#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclWriteSetup.tcl,v 1.17 2002/04/04 11:04:43 vltsccm Exp $"
#
# slxtclWriteSetup.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/05/96  created
# alongino  09/07/96  added check on oslxServer error replies
# alongino  02/05/97  interface to oslx changed
# alongino  15/10/97  made public
#
#************************************************************************
#   NAME
#   slxtclWriteSetup - Write setup keywords and values in a setup file
# 
#   SYNOPSIS
#   slxtclWriteSetup <oslxBuf> <file> <type> <inst>
# 
#   DESCRIPTION
#   Tcl procedure, used by slxtclSaveSetup, to write in a setup file all
#   keywords and values contained in the buffer <oslxBuf> as returned by 
#   slxtclPanToBuf.
#   For the meaning of parameters <file>, <type>, <inst> see slxtclSaveSetup. 
#   
#   FILES
#   <file>          setup file generated
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   FAILURE if error
#   string as returned by oslx
#
#   CAUTIONS
#   1) An already existing setup file is overwritten
#   2) This procedure does NOT check if the appropriate data dictionary has 
#      been loaded through slxtclDictionary. The application has to make 
#      sure that this is true.
#
#   EXAMPLES
#   # 1
#   # extract from the panel keywords and values, store them in the oslx setup
#   # buffer "slxtcl" of oslx and then save the contents of the oslx setup 
#   # buffer in the reference file myRef
#   slxtclDictionary CCDDCS
#   set oslxBuf [slxtclPanToBuf]
#   slxtclWriteSetup $oslxBuf myRef "Reference Setup" slxtcl
#
#   SEE ALSO
#   slxtclSaveSetup, slxtclPanToBuf, slxtclDictionary
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclWriteSetup { oslxBuf file {type "Reference Setup"} {inst slxtcl} } {
   set command "OSETUP"
   #
   # Clean buffer
   #
   set status [slxtclOslxIf $command -clear $inst]

   #
   # Fill in buffer
   #
   if {$status != "FAILURE"} {
     set status [slxtclOslxIf $command -storeKeyword $inst $oslxBuf]
   }

   #
   # Save contents of oslx setup buffer in a setup file
   # note that type string may contain blanks in betwwen
   #
   if {$status != "FAILURE"} {
     set status [slxtclOslxIf $command -saveAs $inst $file "\"$type\""]
   }

   #
   # Return oslx reply
   #
   return $status
}


#
# ___oOo___


