#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: promptDialog.tcl,v 3.36 2003/01/20 10:33:53 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Store it into a separate file .. with comments
#                     Contains only variables needed by both panels
#                     and panel editor
# mcomin    11/01/96  Man Page upgrade.
# mcomin    14/03/96  Replaced courR14 with full font name
#           06/03/97  Porting to TK 4.2
#
#
#************************************************************************
#   NAME
#
#     promptDialog - Procedure implementing the MOTIF promptDialog
# 
#   SYNOPSIS
#   
#     promptDialog <widgetName> <text> ?default value?
#
#   DESCRIPTION
#
#     This procedure interrupts the application and creates a dialog box 
#     to get input from the user.
#
#     The dialog box is formed by a message, an entry field and 
#     the two button "OK" and "CANCEL".
#
#     widgetName     <IN> - Name of the widget
#     text           <IN> - Describes the data to be typed in
#     default value  <IN> - Optional parameter specifying the current
#                           value
#
#     The application continues when the user clicks on "OK' or "CANCEL".
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#     The new value
#
#   CAUTIONS
#
#   This is also included in the panel files.
#
#   EXAMPLES
#
#      set newtitle [promptDialog .dialog "Enter New title " "Old Title"]
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : infoDialog - Creates and Information Dialog containing a text
#                          and an input field.  
#                          The user can 'accept' or 'cancel' the operation 
#                          described in the text.
#------------------------------------------------------------------------------

proc infoDialog {c msg } {
   global par
   
   toplevel $c

   set middle [frame $c.middle  -relief raised -bd 1]
   set bot    [frame $c.bot     -relief raised -bd 1]
   frame $bot.left   -relief sunken -bd 1
   
   button  $bot.left.accept  -text "Accept" -command "set flagDialog 1"   
   button  $bot.cancel -text "Cancel" -command "set flagDialog 0;  destroy $c"

   entry   $middle.entr -width 24  
   message $middle.msg  -justify left -text "$msg" -width 10c\
                        -font "-Adobe-courier-medium-r-normal--14*"
    
   pack $middle $bot -fill x
   pack $middle.msg -expand yes -pady 5 
   pack $middle.entr -side bottom  -pady 5 -padx 30 -anchor w

   pack $bot.left   -side left -expand yes -padx 20 -pady 20
   pack $bot.cancel -side left -expand yes -padx 20  
   pack $bot.left.accept -padx 12 -pady 12

   wm geometry $c +400+400
   
   bind $middle.entr <Return> "$bot.left.accept invoke"

   tkwait visibility $c
   grab $c
   focus $middle.entr
   update
}


#------------------------------------------------------------------------------
# PROCEDURE : promptDialog - Generates an information dialog
#------------------------------------------------------------------------------

proc promptDialog {root msg {defaultStr {}}   } {
    global par flagDialog

    set ret          ""    
    set c            $root
    set flagDialog  -1;        # To be used with tkwait

    infoDialog  $c "$msg"
    
    if {[string length $defaultStr] > 0} {
       $c.middle.entr insert 0 "$defaultStr"
    }
    
    update 
    vwait flagDialog ;      # Stops until variable as been modified
             
    if { $flagDialog } {
        set ret [getInput  $c.middle.entr]
        destroy $c
    }
    return $ret
}
