#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifDialog.tcl,v 3.36 2003/01/20 10:34:04 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  1996-11-30  created
# mbec      1997-07-10  header update (cf. TemplateServerMode)
# mbec      1997-08-29  button relief fix
# eallaert  1997-10-21  removed TemplateServer stuff (put straight into tplsrv)
#

#************************************************************************
#   NAME
#   uifDialog - display a dialog box for BOB
# 
#   SYNOPSIS
#   uifDialog <title> <message> <modality> ?<timeout>? ?<type>?  ?<cancelButton>?
# 
#   DESCRIPTION
#   uifDialog will display a dialog box with:
#
#   <title>   :      title of the dialog window
#   <message> :      text displayed in the dialog box
#   <modality>:      determines the degree of blocking of the application.  
#                    It can be :
#                    "global"      -> all applications are blocked while 
#                                     this dialog is active.
#                    "application" -> only this application is blocked
#                    "none"        -> no application will be blocked.
#   <timeout> :      timeout value in ms after which the dialog will deactivate
#                    automatically.  A 0 value indicates no timeout (default). 
#                    Timing-out has the same effect as pressing "Cancel" 
#                    if this button is displayed
#                    (see <cancelButton>); otherwise, it is same as pressing "OK".
#   <type> :         type of dialog box; can be "error" (default), "info", 
#                    "question" or "warning".
#   <cancelButton> : boolean, indicating whether "Cancel" button should be
#                    shown (0 by default).
#
#   This procedure is implemented based on [incr Widgets]'s messagedialog.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   Lines longer than 100 chars will be truncated.
#
#   EXAMPLES
#
#   SEE ALSO
#   This dialog has been derived from 'bobDialog'
#   [incr widgets] documentation (http://www.tcltk.com)
#
#
#   BUGS
#
#------------------------------------------------------------------------
#
# dialog box images
image create photo errorImg    -file [findVLTFile bitmaps/uifStop.gif]
image create photo infoImg     -file [findVLTFile bitmaps/uifInfo.gif]
image create photo questionImg -file [findVLTFile bitmaps/uifQuestion.gif]
image create photo warningImg  -file [findVLTFile bitmaps/uifWarning.gif]

proc uifDialog {title text mod {timeout 0} {type error} {cancelButton 0 } } {
    global gvar

    # Check text length
    foreach line [split $text \n] {
	if {[string length $line] > 100} {
	    set line "[string range $line 0 96] ..."
	}
	append ltext "${line}\n"
    }

    # test here if there's already one instead of destroying it each time 
    if {[cequal [info command .uifED] {}]} {
	messagedialog .uifED -title $title \
		-image ${type}Img -text $ltext -modality $mod \
		-padx 10 -textpadx 10

	.uifED hide Apply
	.uifED hide Help
	.uifED buttonconfigure OK -padx 3 -pady 3 -borderwidth 2

	if { $cancelButton == 0 } {
	    .uifED hide Cancel
	} else {
	    .uifED show Cancel
	    .uifED buttonconfigure Cancel -padx 3 -pady 3 -borderwidth 2
	}

    } else {
	.uifED configure -text $ltext -modality $mod -title $title  -image ${type}Img
	if { $cancelButton == 0 } {
	    .uifED hide Cancel
	} else {
	    .uifED show Cancel
	}
    }

    # Take care of automatic deactivation if so required.
    # Timing-out is same as pressing "Cancel" if this button is displayed;
    # otherwise, it is same as pressing "OK"
    if {$timeout > 0} {
	if {$cancelButton} {
	    after $timeout {.uifED deactivate 0}
	} else {
	    after $timeout {.uifED deactivate 1}
	}
    }

    return [.uifED activate]
}
