#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifResolvePath.tcl,v 3.36 2003/01/20 10:34:01 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    11/11/97  Created from the exiting procedure fhtResolvePath
#

#************************************************************************
#   NAME
#      uifResolvePath - Returns the absolute path of a file name.
#   SYNOPSIS
#      uifResolvePath  <filePath> ?checkFlag?
#   DESCRIPTION
#   This procedure processes the filePath and return the absolute path.
#
#    <filePath>   Relative file name : it can start with "~" and it can
#                 contain references to environment variables.
#
#    ?checkFlag?  Optional parameter to check if the absolute path
#                 leads to an existing directory.
#                 Valid values are NO_CHECK / CHECK : the first one is
#                 the default value.
#
#   Environment variables must start with the '$' sign and they can appear 
#   in each position the of <filePath>.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#   
#   Absolute path name or an empty string in case of failure.
#
#   CAUTIONS
#   The character '$' used to specify an environment variable must
#   be preceded by the '\' -e.g.- \$VLTROOT 
#   EXAMPLES
#
#   if { [set path [uifResolvePath <filePath>]] == "" } {
#      Failure translating <filePath>
#   }
#
#   Some examples of values for <filePath> and the returned path
#
#    filePath    checkFlag |  Returned values        | Comment
#   ______________________________________________________________________
#                          |                         |
#   \$VLTROOT/bin   CHECK  | /vlt/NOV97/bin          |
#   ~mcomin/NOV97   CHECK  | /home/te67e/mcomin/NOV9 |
#   /vlt/data/tmp   CHECK  | /vlt/data/tmp           |
#                          |                         |
#   \$DUMMY/tmp     CHECK  | ""                      | Env. variable not exist
#                          |                         | 
#   ~mcomin/catif   CHECK  | ""                      | Directory does not exist
#                          |                         |
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc uifResolvePath {path {checkFlag {NO_CHECK}} } {
    global env

    # Resolve a path like:
    # $VAR1/$VAR2/dir1/filename.ext
    set pathList [split $path /]
    set completePath ""

    foreach el $pathList {

	if {[string first \$ $el] == 0} {
	    set varName [string range $el 1 end]
            if {[info exists env($varName)] == 0 } {
               return ""
            }
	    set tmpPath "$env($varName)"
	} elseif {[string first ~ $el] == 0} {
	    set tmpPath "$env(HOME)"
	} else {
	    set tmpPath $el
	}
	if {[string index $tmpPath 0] == "/"} {
	    set completePath "${completePath}${tmpPath}"
	} else {
	    set completePath "$completePath/$tmpPath"
	}
    }
    # Make sure there is only one slash at the beginning of the path
    if {[string index $completePath 0] == "/"} {
	set completePath "/[string trimleft $completePath /]"
    }
  
    switch $checkFlag {
       CHECK {
	   if { ! [file exists $completePath] } { set completePath "" }
       }
       NO_CHECK { set do nothing }
       default {
           puts "uifResolvePath > Wrong value for parameter checkFlag : $checkFlag"
           set completePath ""
       }
    }

    return $completePath
}

#
# ___oOo___
