#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifActionButton.tcl,v 3.36 2003/01/20 10:33:43 vltsccm Exp $" 
#
# uifActionButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# M.Comin    07/11/94   Add methods and documentation
# M.COMIN    27/02/95   Add destructor method
# M.COMIN    15/03/95   Man page updated
# M.COMIN    22/03/95   Improved CmdTrace local method
# M.COMIN    14/06/95   Fixed bug with cmdTrace
# M.COMIN    31/07/95   Use args instead of config
# -------------------
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            13/05/96   Set 'usual' to empty
#            14/05/96   Update man page
#
#************************************************************************
#   NAME
#     UifActionButton - Class for VLT Action Buttons
#   SYNOPSIS
#     UifActionButton  <widget-name>
#   DESCRIPTION
#     UifActionButton defines a specific class of buttons which are normally
#     added in the bottom part of the panel.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifActionButton - Button widget.
#
#   METHODS
#     Inherits the methods of class uifInteractiveWidget.
# 
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget classe and supports :
#
#     command  <string>    Tcl statement to be executed.
#     label    <string>    String used to label the button
#     width    <integer>   Determine the widget length (in characters)
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifActionButton  .w   -label myAction -stateVar gvar(myVar) -width 10 \
#                           -command {puts "this is an action button"}
#
#     Re-configure the widget
#        .w  configure -label "new label"
#
#     Other operations
#        .w  disable ;                   Disable widget
#        set label [.w cget -label]      Get a widget's resource
#   SEE ALSO
#     UifWidget  UifInteractiveWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifActionButton { }


class UifActionButton {
    inherit UifInteractiveWidget

    constructor {args} {}

    # This is not automatically inherited !! needed for ptlib.
    public method invoke  {} { $itk_component(uifActionButton) invoke }

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifActionButton::constructor {args} {

   itk_component add uifActionButton {
      button $itk_interior.uifActionButton -bd 2 -relief raised \
             -highlightthickness 0 -font "*adobe-helvetica-bold-r-*-*-14*"
   } {
      rename -text -label label Label 

      keep -width -font -command -state -bitmap -disabledforeground -relief\
           -background -foreground -activeforeground -activebackground
   }

   pack $itk_component(uifActionButton)
   eval itk_initialize 
   eval configure -width 5 $args
}
