#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifBitmap.tcl,v 3.36 2003/01/20 10:33:44 vltsccm Exp $" 
#
# uifBitmap.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# M.COMIN    08/03/95   Add color to hot-spot
# M.COMIN    16/03/95   Man page updated
# M.COMIN    31/07/95   Use args instead of config
#
# mcomin     6/03/96    Porting to Tk4.0 and itcl2.0
#            13/05/96   Update man page
#
#************************************************************************
#   NAME
#     UifBitmap - Class for VLT Bitmap
#   SYNOPSIS
#     UifBitmap  <widget-name>
#   DESCRIPTION
#     UifBitmap defines the class for the VLT Bitmap widget. 
#     The bitmap can be also configured as "Hot-Spot" allowing the user 
#     to click on to trigger some actions.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     title     - Label giving a title to the bitmap
#     uifBitmap - Bitmap widget
#   METHODS
#
#   PUBLIC VARIABLES
#  
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from uifWidget and supports :
#
#     bitmap       <string>   Path where to load the bitmap from.
#     command      <string>   Tcl statement to be executed.
#     frame        <string>   This sets a frame around the bitmap : all
#                             TK frames are valid.
#     hotSpot      <1/0>      Configures a bitmap as "Hot-Spot"
#                             The Hot-Spot feature is enabled only if the 
#                             option "command" is not empty.
#     hotSpotColor <string>   Specifies the background color when the
#                             bitmap gets focus.
#     label        <string>   Label on top of the bitmap
#   SPECIFIC BINDINGS
#
#   CAUTION
#
#   EXAMPLES
#
#     Create an instance
#
#     UifBitmap .w -command {puts "NTT Bitmap"} -frame raised -hotSpot 1 \
#                     -hotSpotColor Green -label {NTT Bitmap}
#
#     Re-configure the widget
#        .w configure -label  "new Label" -frame groove -hotSpotColor Red
#
#     Other operations
#        set bitmapPath [findVLTFile bitmaps/ESO.xbm]  
#       .w configure -bitmap $bitmapPath     Set New Bitmap
#
#        set bitmapPath [.w cget -bitmap]    Get a widget's resource
#   SEE ALSO
#     UifWidget
#   BUGS
#------------------------------------------------------------------------
#/*

itk::usual UifBitmap { }

class UifBitmap {
    inherit UifWidget

    constructor {args} {}

    # NOTE : option bitmap is automatically inherited from the 
    # associated label component.

    itk_option define -command      command      Command      ""
    itk_option define -hotSpotColor hotSpotColor HotSpotColor Red
    itk_option define -label        label        Label        ""
    itk_option define -hotSpot      hotSpot      HotSpot      0

    private variable canvasBG ""
    private variable me       ""

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifBitmap::constructor { args } { 

   itk_component add frame {
       frame $itk_interior.frame
   } {
       keep -relief -borderwidth 
   }

   itk_component add title {
       label $itk_component(frame).label 
   } {
       keep -text
   }     

   itk_component add uifBitmap {
       label $itk_component(frame).uifBitmap -borderwidth 2
   } {
       keep -background -bitmap
       rename -relief -frame frame Frame
   }     

   pack $itk_component(frame) $itk_component(title) $itk_component(uifBitmap)

   eval itk_initialize

   set canvasBG  [$itk_interior cget -background]
   set me        $itk_component(uifBitmap)

   eval configure $args
}

#---------------------------------------------------------------
# OPTION : -hotSpot
#---------------------------------------------------------------

configbody UifBitmap::hotSpot {
   configure -command $itk_option(-command)   
}

#---------------------------------------------------------------
# OPTION : -command
#---------------------------------------------------------------

configbody UifBitmap::command {

   if { $itk_option(-hotSpot) } {     
      bind $me <Enter> "$me configure -background $itk_option(-hotSpotColor)"
      bind $me <Leave> "$me configure -background $canvasBG"
      bind $me <1>      $itk_option(-command)   
   } else {             
      bind $me <Enter> ""
      bind $me <Leave> ""   
      bind $me <1>     ""
   } 
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifBitmap::label {
   if { [cequal $itk_option(-label) ""] } {
      pack forget $itk_component(title) 
   } else {
      pack $itk_component(title) -before $itk_component(uifBitmap)        
   }
                          
   $itk_component(title) configure -text $itk_option(-label) 
}

#---------------------------------------------------------------
# OPTION : -hotSpotColor
#---------------------------------------------------------------

configbody UifBitmap::hotSpotColor {
   if {$itk_option(-hotSpot) && ! [cequal $itk_option(-command) ""] } {
      bind $me <Enter> "$me configure -background $itk_option(-hotSpotColor)"
   }
}


#___oOo___
