#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifButton.tcl,v 3.36 2003/01/20 10:33:45 vltsccm Exp $" 
#
# uifButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# M.Comin    07/11/94   Add methods and documentation
# M.COMIN    27/02/95   Add destructor method
# M.COMIN    14/06/95   Fixed bug with cmdTrace
# M.COMIN    31/07/95   Use args instead of config
# -------------------
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            14/05/96   Update man page
#            17/05/96   Add option bitmapFile
#
#************************************************************************
#   NAME
#     UifButton - Class for VLT pushButtons
#   SYNOPSIS
#     UifButton  <widget-name>
#   DESCRIPTION
#     UifButton defines the class for the VLT pushButtons.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifButton - Button widget.
#   METHODS
#     Inherits the methods of class UifInteractiveWidget.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget class and supports :
#
#     bitmapFile <string>   File to be used as bitmap
#     label    <string>     String used to label the button
#     popup    <1/0>        If set to 1, the string "..." is added to
#                           to the button label to let the user know that 
#                           the button calls a panel.
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifButton .w  -command {panCallPanel anotherPanel} -popup 1 \
#                   -label myLabel -stateVar gvar(myVar) -width 10
#
#     Re-configure the widget
#
#     .w  disable   ;# Disable the widget
#     .w  configure -label "new label" -width 15 -popup 0
#     .w  configure -command "sendCommand <env> <proc> <cmd> <par list>"
#
#     Other operations
#        set myCmd   [.w cget -command]   Get a widget's resource
#        set myLabel [.w cget -label]
#   SEE ALSO
#     UifWidget UifInteractiveWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifButton { }

class UifButton {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -label label Label "" { setText $itk_option(-label) }  
    itk_option define -popup popup Popup 0  { setText $itk_option(-label) }
    itk_option define -bitmapFile  bitmapFile BitmapFile ""

    public method setText { txt }

    # This is not automatically inherited !! needed for ptlib.
    public method invoke  {} { $itk_component(uifButton) invoke }
}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifButton::constructor {args} {

   itk_component add uifButton {
      button $itk_interior.uifButton -bd 2 -width 5 -relief raised \
             -highlightthickness 0
   } { 
      keep -width -font -background -command -foreground -state -bitmap
      keep -activeforeground -activebackground -disabledforeground -relief
   }

   pack $itk_component(uifButton)
   eval itk_initialize
   eval configure -popup 0  $args
}

#---------------------------------------------------------------
# METHOD : setText
#---------------------------------------------------------------

body UifButton::setText { txt } {
   if {$itk_option(-popup) && ![cequal $txt ""] } { 
      append txt " ..."
   }               
   $itk_component(uifButton) configure -text $txt
}
    
#---------------------------------------------------------------
# OPTION : -bitmapFile
#---------------------------------------------------------------

configbody UifButton::bitmapFile {

   if {[cequal $itk_option(-bitmapFile) ""] } {
      configure -bitmap "" ; # Reset to textual label
   } else {  
      set filePath [findVLTFile bitmaps/$itk_option(-bitmapFile)]
      configure -bitmap "@${filePath}"
   }
} 

