#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifCheckButton.tcl,v 3.36 2003/01/20 10:33:45 vltsccm Exp $" 
#
# uifCheckButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# M.Comin    07/11/94   Add methods and documentation
# M.COMIN    15/03/95   Man page updated
# M.COMIN    24/03/95   Method setOnValue setOffValue converted in publ. var.
# M.COMIN    31/07/95   Use args instead of config
# -------------------
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            07/05/96   Inherit now from uifInteractiveWidget
#            14/05/96   Update man page
#            26/04/98   Add default values for options
#
#************************************************************************
#   NAME
#     UifCheckButton - Class for VLT checkButtons
#   SYNOPSIS
#     UifCheckButton  <widget-name>
#   DESCRIPTION
#     UifCheckButton defines the class for the VLT checkButtons.
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifCheckButton - Check Button widget
#   METHODS
#
#     Inherits methods of class UifInteractiveWidget and supports 
#     the following :
#
#     flash
#        Flash the check button by re-displaying the widget several times
#        alternating between active and normal colors. 
#        This command is ignored when the check button is disabled.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget classe and supports :
#
#     command  <string>   Command to execute when the user clicks the 
#                         button. The <variable> is updated first and 
#                         then the command is executed.
#     label    <string>   String used to label the widget
#
#     offVal <string> or  Configures the widget 'OFF' state.
#            <integer>    It can also be DB symbolic address.
#
#     onVal  <string> or  Configures the widget 'ON' state.
#            <integer>    It can also be DB symbolic address.
#
#     variable <string>   Variable associated to the widget
#     width    <integer>  Widget length.
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifCheckButton .w -label myLabel -variable gvar(myVar) -width 10 \
#                       -onVal 18 -offVal-10 -command {callMyProcedure}
#
#     Re-configure the widget
#        .w  configure -onVal 321 -label  "new label"
#        .w  disable ;
#
#     Other operations
#
#        Set OFF value by reading a DB attribute.
#       .w configure -offVal "@wte16:PARAMS:SCALARS.scalar_int8"
#
#        set myLabel [.w cget -label]    Get a widget's resource
#        set myVar   [.w cget -variable]
#   SEE ALSO
#    UifWidget UifInteractiveWidget UifRadioButton
#   BUGS
#
#------------------------------------------------------------------------
#/*

option add *UifCheckButton.anchor nw widgetDefault

itk::usual UifCheckButton { }

class UifCheckButton {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -label  label  Label  ""
    itk_option define -onVal  onVal  OnVal  1
    itk_option define -offVal offVal OffVal ""

    # This is not automatically inherited !! needed for ptlib.
    public method invoke  {} { $itk_component(uifCheckButton) invoke }
    public method flash   {} { $itk_component(uifCheckButton) flash  }

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifCheckButton::constructor {args} {

   itk_component add uifCheckButton {
      checkbutton $itk_interior.uifCheckButton -bd 2 -highlightthickness 0 -relief flat
   } {
      rename -text -label label Label
      keep -width -anchor -relief -font -background -foreground -variable -state \
           -command -activeforeground -activebackground -highlightthickness
   }

   pack $itk_component(uifCheckButton) -side left

   itk_initialize; set initialized 1
   eval configure $args
}

#------------------------------------------------------
# OPTION : -onVal
#------------------------------------------------------

configbody UifCheckButton::onVal {
   if { [cequal [string index $itk_option(-onVal) 0] @] } {
      $itk_component(uifCheckButton) configure -onvalue [dbRead $itk_option(-onVal)]
   } else {
      $itk_component(uifCheckButton) configure -onvalue $itk_option(-onVal)
   }
} 
    
#------------------------------------------------------
# OPTION : -offVal
#------------------------------------------------------

configbody UifCheckButton::offVal {     
   if { [cequal [string index $itk_option(-offVal) 0] @] } {
      $itk_component(uifCheckButton) configure -offvalue [dbRead $itk_option(-offVal)]    
   } else { 
      $itk_component(uifCheckButton) configure -offvalue $itk_option(-offVal)
   }      
}
