#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifInteractiveWidget.tcl,v 3.36 2003/01/20 10:33:56 vltsccm Exp $"
#
# uifInteractWidget.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# mcomin     29/01/96   created 
#
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            14/05/96   Update man page
#            30/05/96   Fixed bug for initialization of stateVar variable
#            04/07/96   Fixed bug for stateVar variable : add case when
#                       variable exists
#            05/09/96   Add destructor for stateVar
#            10/04/97   Add option -initialState -default
#            25/04/98   Fixed bug in -stateVar : return statement taken
#                       away.
#
#************************************************************************
#   NAME
#     UifInteractiveWidget - Virtual class containing common variables
#                            and methods for interactive widgets 
#   SYNOPSIS
#    inherit UifInteractiveWidget 
#   DESCRIPTION
#     UifInteractiveWidget is used to inherit widget options and methods 
#     common to the VLT interactive widgets.
#   COMPONENTS
#
#   METHODS
#
#     The class supports the following methods :
#
#     enable      
#         Configures the widget as 'enabled'
#
#     disable     
#         Configures the widget as 'disabled'
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     stateVar      <name>     Name of the variable used to control the state
#                              of the widget : enabled or disabled.
#
#     default       <string>   Defines the default value for the widget.
#     initialState  <boolean>  Determines the widget's initial state.
#                              Applies to uifButtons and uifActionButtons only.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     uifWidget
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

class UifInteractiveWidget {
    inherit UifWidget

    constructor {args} {
       eval itk_initialize $args
    }

    destructor {}

    itk_option define -default  default  Default  ""
    itk_option define -stateVar stateVar StateVar ""
    itk_option define -initialState initialState InitialState normal

    public  method stateTraceOff {var}
    public  method stateTrace    { name elem  op }

    public method disable {} { $this configure -state disabled }
    public method enable  {} { $this configure -state normal }   

    private variable oldStateVar ""
       
}; # end of class definition

#------------------------------------------------------
# Destructor
#------------------------------------------------------

body UifInteractiveWidget::destructor {} { 
   if {![cequal $oldStateVar ""]} { stateTraceOff $itk_option(-stateVar)}
}

#---------------------------------------------------------------
# OPTION : -initialState
#---------------------------------------------------------------
  
configbody UifInteractiveWidget::default {
   if { $initialized && ! [cequal $itk_option(-variable) ""] } {
      uplevel #0 set $itk_option(-variable) \"$itk_option(-default)\"
   }
}

#---------------------------------------------------------------
# OPTION : -initialState
#---------------------------------------------------------------
  
configbody UifInteractiveWidget::initialState {
   set class [$wthis info class]

   if {[cequal $class UifButton] || [cequal $class UifActionButton]} {
      switch $itk_option(-initialState) {
         normal    { enable }
         disabled  { disable }
         default   { puts "$this : Wrong value for option initialState : $itk_option(-initialState)"}
      }
   }
}

#---------------------------------------------------------------
# OPTION : -stateVar
#---------------------------------------------------------------
  
configbody UifInteractiveWidget::stateVar {

   if {![cequal $oldStateVar ""]} { stateTraceOff $oldStateVar }

   if {![cequal $itk_option(-stateVar) ""]} {

      # Create the variable if it does not exist .... or update the
      # widget status according to the current value of the variable

      if { [uplevel #0 info exists $itk_option(-stateVar)]} {

         set tmp [uplevel #0 set $itk_option(-stateVar)]

         switch $tmp {
            "0"       { disable }
            ""        { set do nothing }
            "default" { enable }
         } ;# end switch 
      }

      uplevel #0 trace variable $itk_option(-stateVar) w \"$wthis stateTrace\"
   }

   set oldStateVar "$itk_option(-stateVar)"
}


#------------------------------------------------------
# METHOD : stateTrace
# This is invoked to set the widget's state according to the
# value variable 'stateVar'.
#------------------------------------------------------

body UifInteractiveWidget::stateTrace { name elem op } {

   if { ! [cequal $elem ""] } {
      set refValue [uplevel #0 set ${name}($elem)]
   } else {
      set refValue [uplevel #0 set ${name}]
   }

   if {$refValue == 0 } {
        disable
   } else {
        enable
   }
}


#------------------------------------------------------
# METHOD : stateTraceOff
#------------------------------------------------------

body UifInteractiveWidget::stateTraceOff { var } {
#    puts "DBG > Trace Off (var = $var) of  $this"
   uplevel #0 trace vdelete $var w \"$wthis stateTrace\"
}


