#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifLed.tcl,v 3.36 2003/01/20 10:33:59 vltsccm Exp $" 
#
# uifFrame.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    17/12/97   Created
#            26/04/98   Add frane below
#
#************************************************************************
#   NAME
#     UifLed - Basic LED widget 
#   SYNOPSIS
#     UifLed  <widget-name>
#   DESCRIPTION
#     UifLed defines a widget implementing a LED device and it is
#     just providing the functionality to change the Led color.
#
#     This is used to build other more complex widgets such as
#     UifOutputLed where the switching happens automatically according
#     to the value of a variable.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifLed - LED widget.
#
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Supports the following options :
#
#     colorOn   <string>     Color to display when the LED is ON.
#     colorOff  <string>     Color to display when the LED is OFF.
#
#     ledColor  <string>     Sets the led to a given color.
#
#                            The two other options do not change
#                            the led color but are used only to
#                            'store' the colors for ON/OFF states.
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifLed .w -colorOn red   # Create an instance
#
#     Re-configure the widget
#        .w configure -ledColor Blue
#     Other operations
#        .w setColor green   
#        .w configure -ledColor Blue   Set Led color
#
#   SEE ALSO
#     UifWidget UifOutputLed
#
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual UifLed { }

class UifLed {
    inherit itk::Widget

    constructor {args} {}

    itk_option define -colorOn   colorOn   ColorOn   Black
    itk_option define -colorOff  colorOff  ColorOff  Gray90

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifLed::constructor {args} {

   itk_component add f1  {
      frame $itk_interior.f1 -borderwidth 1 -relief ridge
   }

   pack $itk_component(f1)

   itk_component add uifLed  {
      frame $itk_component(f1).uifLed  -borderwidth 2 -width 10 -height 10 \
                                  -relief ridge
   } {
     rename -background -ledColor ledColor LedColor
   }

   pack $itk_component(uifLed) -padx 1 -pady 1
   eval itk_initialize $args
}


