#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputWidget.tcl,v 3.36 2003/01/20 10:33:55 vltsccm Exp $" 
#
# uifOutputWidget.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# mcomin     29/01/96   created 
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            15/05/96   Update man page
#            06/03/97   Add check if var == ""
#
#************************************************************************
#   NAME
#     UifOutputWidget - Virtual class containing common variables
#                       and methods for output widgets 
#   SYNOPSIS
#    inherit UifOutputWidget 
#   DESCRIPTION
#     UifOutputWidget is used to inherit public variables and methods common
#     to the VLT output widgets.
#   COMPONENTS
#
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     dbMode  <string>    Defines the updating mode when the output widget
#                         is connected to a database attribute.
#                         The following two values are valid :
#
#                         dbPoll   Set the update mode to 'polling'
#                         dbEvent  Set the update mode to 'events'
#
#     evtFilter <string>  Determine the condition triggering the event, such 
#                         as : "<" "<=" "=" ">" "=>" "!=" "w".
#
#     evtScript <string>  Defines the procedure to be called when the event
#                         is received.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     uifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

class UifOutputWidget {
    inherit UifWidget

    constructor {args} {
       eval itk_initialize $args
    }

    destructor {
       if {![cequal $oldVar ""] } { varTraceOff }
    }

    itk_option define -dbMode    dbMode    DbMode    {none}
    itk_option define -evtFilter evtFilter EvtFilter "w"
    itk_option define -evtScript evtScript EvtScript {}

    public method varTraceOff {}
    public method varTrace    {name elem op}
    public method setTraceVar {var}

    private variable oldVar ""
}; # end of class definition

#---------------------------------------------------------------
# METHOD : varTraceOff - Disable callback
#---------------------------------------------------------------

body UifOutputWidget::varTraceOff { } {
#   puts "DBG> #0 trace vdelete $oldVar w  \"$wthis varTrace\""
   uplevel #0 trace vdelete $oldVar w  \"$wthis varTrace\"
}

#---------------------------------------------------------------
# METHOD : varTrace
#---------------------------------------------------------------

body UifOutputWidget::varTrace { name elem  op } {
#    puts "DBG> UifOutputWidget::varTrace {$name} {$elem}  {$op}"

   if {[cequal $elem ""]} {
      $this setValue [uplevel #0 set [list ${name}]]
   } else {
      $this setValue [uplevel #0 set [list ${name}]($elem)]
   }
}

#---------------------------------------------------------------
# METHOD : setTraceVar
#---------------------------------------------------------------

body UifOutputWidget::setTraceVar { var } {

#   puts "DBG> UifOutputWidget::setTraceVar $var"

    if {![cequal $oldVar ""]} { varTraceOff }

    if {![cequal $var ""]} {
      # Create the variable if does not exists yet
      if {[uplevel #0 info exists $var] == 0 } {uplevel #0 set $var 0 }

      # Declare callback function to keep widget updated
#      puts "DBG> uplevel #0 trace variable $var w \"$wthis varTrace\""
      uplevel #0 trace variable $var w \"$wthis varTrace\"
   }

   set oldVar $var
}
