#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifPanel.tcl,v 3.36 2003/01/20 10:33:49 vltsccm Exp $" 
#
# uifPanel.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# mcomin    07/11/94   created 
# mcomin    17/06/95   Major modifications for the new class design.
# mcomin    18/06/95   Finished implementation for independence of data
#                       between instances of the same class.
# mcomin    27/07/95   public variable cwp invokes setInstanceData
# mcomin    22/08/95   add catch in configuring -variable
# mcomin    24/10/95   Processing of cwp takes into account <alias> 
#                      specification
# mcomin    25/10/95   Add environm. name to the alias spec.
# --------------------
# mcomin    12/01/96   Original variable is deleted in order to avoid 
#                      proliferation of unused variables
#           09/03/96   Ported to Tk4.0 and itcl 2.0
#           13/05/96   Update man page
#           04/07/96   Add config. for -stateVar
# mcomin    08/04/97   Add catch to get -helparea option
#           13/08/97   Add new option -parentCwp to cope with 
#                      cwp inherited from parent uifClass
#           21/11/97   Fixed bug processing user's defined event's proc.
#           02/03/98   Add UifOutputLed, UifOutputArray to the list of known
#                      widgets.
# eallaert  1999-02-15 Ported to Tcl/Tk 8.0 and itcl 3.0
# mcomin    17/05/2001 Fixed bug with <relative> specifier
#           23/05/2001 Re-coding of method setInstance
#           12/09/2001 Bug fixing in method setInstance
#           14/03/2002 Add panEvtDetach when re-configuring widgets !!
#
#************************************************************************
#   NAME
#     UifPanel - Parent class for xxx_uifClass widgets
#   SYNOPSIS
#     inherit UifPanel
#   DESCRIPTION
#     UifPanel is an internal class containing methods an variables commmon
#     to all classes built with the panel editor -e.g.- xxx_uifClass widgets.
#
#     The panel editor can 'import' classes to be used in the current panel.
#   COMPONENTS
#
#   METHODS
#
#     The class inherits methods from UifWidget class and supports 
#     the followings :
#
#     display
#        Displays the class (similar to the pack command)
#
#     unpack
#        Make the class disappear ( similar to the pack forget command)
#
#     getClassPath
#        Returns the full path to the 'canvas' containing the widgets.
#
#     refreshInput
#        Updates all input widgets present in the class according to
#        the current database values.
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     cwp     <string>  Set the class Current Working Point.
#     library <string>  Specifies the name of the library where the
#                       class come from (used only to store the information).
#     envName <string>  Name of the environment where to get the data from.
#                       This applies to all DB attributes which do not have
#                       a specific environment name.
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

class UifPanel {
    inherit itk::Widget
    
    constructor {args} {
	global tcl_version
        eval itk_initialize $args
	set wthis  [namespace tail $this]
    }

    destructor { deleteClass }

    itk_option define  -helparea  helparea  Helparea  ""
    itk_option define  -helptext  helptext  Helptext  ""

    # For backwards compatibility
    itk_option define  -canvasAnchor canvasAnchor CanvasAnchor nw

    itk_option define -cwp cwp Cwp {} {
	if { $initDone } { setInstanceData } 
    }

    itk_option define -parentCwp parentCwp ParentCwp ""
    itk_option define -library library Library ""
    itk_option define -envName envName EnvName ""

    public method  parseClassName  {name}
    public method  refreshInput    {}
    public method  setInstanceData {}

    public method  getClassPath {} {
	return ${this}.cv
    }

    public method setRecName {name} {
        set recName $name
    }

    public method display {} {
	$wthis init  
	panLoadWidget $className $itk_option(-helparea) $wthis.cv $wthis.cv
	loadShortHelp $itk_option(-helparea) $wthis [panClassWidgetNames $wthis 1]
    }
    
    public method classSize  { me w h } {
        $me configure -width $w -height $h
    }

    public method unpack { } {
	pack forget ${this}.cv
    }

    public method getPollList      {} { return $pollList }
    public method getEventList     {} { return $eventList }
    public method setCwpEnv        {cwp} { set cwpEnv $cwp }

    private method  deleteClass    {}
    private method  configVar      { w type var }

    protected variable wthis        ""
    protected variable initDone     0
    #    protected variable localEnv     {}
    protected variable cwpEnv       ""
    protected variable className    {}
    protected variable cv           {}   
    protected variable parentName   {}
    protected variable pollList     {}
    protected variable eventList    {}
    protected variable uifClassList {}
    protected variable recName      {}
}; # end of class definition

#------------------------------------------------------
# METHOD : deleteClass 
#------------------------------------------------------

body UifPanel::deleteClass {} {
    global panWidget panWidgetHlp
    foreach name [panWdgReferenceList $wthis canvas uifType] {
	panEvtDbDetach $name ; update
    }
    catch {unset panWidget($rec) panWidgetHlp($rec) }
}

#------------------------------------------------------
# METHOD : configVar 
#------------------------------------------------------

body UifPanel::configVar { w type var } {
    if {[cequal $type UifButton]} {
	$w configure -stateVar gvar($var)
    } else {
	catch { $w configure -variable gvar($var) }
    }
}


#---------------------------------------------------------------
# METHOD : parseClassName 
#---------------------------------------------------------------

body UifPanel::parseClassName {name} {
    set cv         $wthis.cv 
    set parentName [winfo parent $wthis]
    set className  [winfo name $wthis]
}    

#---------------------------------------------------------------
# METHOD : refreshInput
#---------------------------------------------------------------

body UifPanel::refreshInput {} {
    global panDbSymbAddr panDbData panWidget par db_var gvar \
	    i_dbVar db_type i_type

    #   puts "UifPanel::refreshInput     $this"
    foreach child [winfo children $cv] {
	set name ${className}.[winfo name $child]

	if {[info exists panDbData($name)] == 0 }  {continue}
	if {[getDbConfig $name $db_type] != "in"}  {continue}
	if {[getWidget   $name $i_dbVar ] == 0}    {continue}

	# Skip empty variable definitions
	if {[set myAddr [getDbConfig $name $db_var]] == "" } { continue }
	if {[set symbAddr [panGetWdgAttribute $name $myAddr]] != "" } {
	    set panDbSymbAddr($name) $symbAddr
	}                
	
	set newVal [dbRead $symbAddr] 
	set type   [getWidget $name $i_type]

	panUpdateLocalVar $name $type $newVal  

	if {[cequal $type UifScale]} { $child setValue $newVal }
    }; # end foreach ..

	update
	# Update uifClass embedded widgets

	foreach item $uifClassList {
	    $item refreshInput
	}
    }

    #---------------------------------------------------------------
    # METHOD : setInstanceData
    #---------------------------------------------------------------
    
    body UifPanel::setInstanceData {} {
	global ::panWidget ::panDbData ::panDbSymbAddr EDITING par env \
		db_var db_env db_type i_var gvar ::panWdgEnvName ::panActiveEnv \
		tcl_version ::tcsEnv

	if {[info exists tcsEnv]} { puts "TCS ENV $tcsEnv"}

	if { $EDITING } {
	    if {![cequal $par(EDIT_MODE) ".pan"] && [cequal $className $par(MYCLASS)] } { return }
	}

	if {[cequal $itk_option(-parentCwp) ""]} {
	    set currCwp $itk_option(-cwp)
	} else {
	    switch [string index $itk_option(-cwp) 0] {
		":"      {set currCwp "$itk_option(-cwp)" }
		"<"      {set currCwp "$itk_option(-cwp)" }
		default  {set currCwp "$itk_option(-parentCwp):$itk_option(-cwp)" }
	    }
	}

	set currCwp [string trimright $currCwp ":"]

	#  puts "\nDBG > parent = \"$itk_option(-parentCwp)\"  class = \"$itk_option(-cwp)\"  --> \"$currCwp\""
	#  puts "DBG > canvas name = $this.cv >> child = [winfo children $cv]"

	if {[cequal $itk_option(-envName) ""]} {
	    if {[cequal $cwpEnv ""]} {
		if {[info exists env(RTAPENV)]} { set cwpEnv @$env(RTAPENV) }
	    }
	} else {
	    if {[cequal [string index $itk_option(-envName) 0] "\$"]} {
		set dummyEnv [string range $itk_option(-envName) 1 end]
		set cwpEnv @[uplevel #0 set $dummyEnv]
	    } else {
		set cwpEnv @$itk_option(-envName)
	    }
	}

	switch [string index $currCwp 0] {
	    ":"      {set concatCwp 0}
	    "<"      {set concatCwp 2}
	    ""       {set concatCwp 3}
	    default  {set concatCwp 1}
	}

	#   puts "DBG > cwpEnv($cwpEnv) envName=($itk_option(-envName))"

	foreach item [winfo children $cv] {
	    set name  ${className}.[winfo name $item]
	    set type  [namespace tail [$item info class]]

	    if {[string match "*_uifClass" $type]} {
		lappend uifClassList [namespace tail $item]
	    }

	    # Re-assign the name associated to the UifXXX widget's data structures 

	    if {[cequal [string range $type 0 2] "Uif"] } { $item setRecName $name }

	    # Skip empty variable definitions
	    if {[set var [getWidget $name $i_var]] == ""} { continue }

	    if {[info exists panDbData($name)] == 0 } {
		configVar $item $type ${className}_${var}
		continue
	    }

	    ######################################
	    #   Process Database Attribute
	    ######################################

	    set tmp [getDbConfig $name $db_env]

	    switch $tmp {
		local - "" { set tmp1 $cwpEnv }
		default    {
		    if {[string index $tmp 0] == "\$"} {
			set dummyEnv [string range $tmp 1 end]
			set tmp1 @[uplevel #0 set $dummyEnv]
		    } else {
			set tmp1 @${tmp}
		    }
		}
	    }

	    set wdgEnv [ctoken tmp1 ":<"]
	    set panWdgEnvName($name)  $wdgEnv
	    set panActiveEnv($wdgEnv) 1

	    # Concatenate class cwp with variable name to get the final
	    # attribute name.

	    ######################################
	    #   Process Database Attribute
	    ######################################

	    set tmp [getDbConfig $name $db_env]
	    set c   [string index $tmp 0]

	    if {[cequal $c "\$"]} { 
		set localEnv "@[uplevel #0 set [string range $tmp 1 end]]"
	    } else {
		set localEnv $tmp
	    }
	    #      puts "DBG> wdgEnv = $wdgEnv  var = $var localEnv = [getDbConfig $name $db_env] newEnv $localEnv"

	    switch [string index $var 0] {
		":" {
		    if {[cequal $localEnv local] || [cequal $localEnv ""]} {
			set newAttr ${wdgEnv}${var}
		    } else {
			set newAttr ${localEnv}${var}
		    }

		    set panDbSymbAddr($name) $newAttr
		}
		"." {
		    switch $concatCwp {
			"1"     {set newAttr ${currCwp}${var}; set panDbSymbAddr($name)  ${wdgEnv}<relative>$newAttr }
			"3"     {set newAttr ${var};           set panDbSymbAddr($name)  ${wdgEnv}<relative>$newAttr }
			default {set newAttr ${currCwp}${var}; set panDbSymbAddr($name)  ${wdgEnv}$newAttr }
		    }
		}
		"<" {
		    set newAttr ${var}; set panDbSymbAddr($name) ${wdgEnv}$newAttr
		}
		default {
		    switch $concatCwp {
			"1"     {set newAttr ${currCwp}:${var};  set panDbSymbAddr($name)  ${wdgEnv}<relative>${newAttr}}
			"3"     {set newAttr ${var};             set panDbSymbAddr($name)  ${wdgEnv}<relative>${newAttr}}
			default {set newAttr ${currCwp}:${var};  set panDbSymbAddr($name)  ${wdgEnv}${newAttr}  }
		    }
		}
	    } ;#end switch

		# Re-Assign DB variable names 

		if {[cequal $itk_option(-envName) ""]} {
		    setDbConfig $name $db_var  $newAttr
		    configVar   $item $type    $newAttr
		    #         puts [format "DBG>newAddr  %-10s %-22s  %-45s" $wdgEnv $var $newAttr]
		} else {
		    setDbConfig $name $db_var  $panDbSymbAddr($name)
		    configVar   $item $type    $panDbSymbAddr($name)
		    #         puts [format "DBG>panDbSymbAddr  %-10s %-22s  %-45s" $wdgEnv $var $panDbSymbAddr($name)]
		}

		#
		# Update input widgets with current DB values
		#
		
		switch $type {
		    UifCheckButton -
		    UifRadioButton -
		    UifEntry -
		    UifListBox {  set newValue [panPollDbRead $name $type]
		    if {! [cequal $newValue panFAILURE]} {
			panUpdateLocalVar $name $type $newValue
		    }
		}

		UifOptionButton -
		UifScale   {  set newValue [panPollDbRead $name $type]
		if {! [cequal $newValue panFAILURE]} {
		    panUpdateLocalVar $name $type $newValue
		}

		if {[uplevel #0 info exists gvar($newAttr)]} {
		    $item setValue [uplevel #0 set gvar($newAttr)]
		}
	    }

	    UifBarChart -
	    UifOutputString -
	    UifOutputArray -
	    UifOutputLed -
	    UifOutputList -
	    UifOutput {

		#
		# Define event handler and local polling list
		#
		if {[cequal [$item cget -dbMode] dbEvent]} {
		    set script [lreplace [$item cget -evtScript]  1 1 $name]
		    $item configure -evtScript $script
		    panEvtDbDetach $name
		    panEvtDbAttach $item $name
		    lappend eventList $name
		} else {
		    # Add to polling list
		    lappend pollList $name
		}
	    }

	    default { continue }
	} ;#end switch
	}; # end foreach ...
}

#---------------------------------------------------------------
# OPTION : -helptext
#---------------------------------------------------------------

configbody UifPanel::helptext {
    if { ![cequal $itk_option(-helparea) ""] && \
	    ![cequal $itk_option(-helptext) ""]} {
	UifShortHelp::add $itk_option(-helparea) $this.cv $itk_option(-helptext)
    }
}

