#****************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifRadioButton.tcl,v 3.36 2003/01/20 10:33:49 vltsccm Exp $" 
#
# uifRadioButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# M.Comin    07/11/94   Add methods and documentation
# M.COMIN    24/03/95   Method setOnValue converted in publ. var.
# M.COMIN    31/07/95   Use args instead of config
#
# mcomin      6/03/96   Porting to Tk4.0 and itcl2.0
#            07/05/96   Inherit now from uifInteractiveWidget
#            13/05/96   Update man page
#            26/04/98   Add default values for options
#
#************************************************************************
#   NAME
#     UifRadioButton - Class for VLT radioButtons
#   SYNOPSIS
#     UifRadioButton  <widget-name>
#   DESCRIPTION
#     UifRadioButton defines the class for the VLT radioButtons.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifRadioButton - Radio Button  widget
#   METHODS
#
#     Inherits methods from UifIntercativeWidget class and supports 
#     the following :
#
#     flash
#        Flash the check button by re-displaying the widget several times
#        alternating between active and normal colors. 
#        This command is ignored when the check button is disabled.  
#
#   PUBLIC VARIABLES
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifInteractiveWidget class and supports :
#
#     command  <string>      Command to execute when the user clicks the 
#                            button. The <variable> is updated first and 
#                            then the command is executed.
#     label    <string>      String used to label the widget
#
#     onVal    <string> or   Configures the widget 'ON' state.
#              <integer>     It can also be DB symbolic address. 
#
#     variable <string>      Variable associated to the widget
#                            Several radioButtons are related by having
#                            sharing the same variable.
#     width    <integer>     Widget length.
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifRadioButton .w -label myLabel -variable gvar(myVar) \
#                       -onVal 18 -command {callMyProcedure} -width 10 
#
#    Re-configure the widget
#        .w  -label  "new label"
#
#    Other operations
#     .w  disable ;            
#     
#     Set OFF value by reading a DB attribute.
#     .w configure -offVal "@wte16:PARAMS:SCALARS.scalar_int8"
#
#     set myLabel [.w cget -label]      Get a widget's resource
#     set myVar   [.w cget -variable]
#   SEE ALSO
#     UifWidget  UifInteractiveWidget UifCheckButton
#   BUGS
#
#------------------------------------------------------------------------
#/*

option add *UifRadioButton.anchor nw widgetDefault

itk::usual UifRadioButton { }

class UifRadioButton {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -onVal  onVal  OnVal  1

    # This is not automatically inherited !! needed for ptlib.
    public method invoke  {} { $itk_component(uifRadioButton) invoke }
    public method flash   {} { $itk_component(uifRadioButton) flash  }

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifRadioButton::constructor {args} {

   itk_component add uifRadioButton {
      radiobutton $itk_interior.uifRadioButton -bd 2 -relief flat -highlightthickness 0
   } { 
      rename -text -label label Label
      keep -width -anchor -font -background -foreground -variable \
          -state  -relief -command -relief -activeforeground -activebackground 
   }

   pack $itk_component(uifRadioButton) -side left  -fill x

   itk_initialize ; set initialized 1 
   eval configure $args
}

#------------------------------------------------------
# OPTION : -onVal
#------------------------------------------------------

configbody UifRadioButton::onVal {
   if { [cequal [string index $itk_option(-onVal) 0] @] } {
      $itk_component(uifRadioButton) configure -value [dbRead $itk_option(-onVal)]
   } else {
      $itk_component(uifRadioButton) configure -value $itk_option(-onVal)
   }
} 



