#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifText.tcl,v 3.36 2003/01/20 10:33:51 vltsccm Exp $" 
#
# uifText.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# mcomin     18/02/95   created 
# mcomin     24/02/95   Cut & Paste implemented with internal buffer
#                       Navigation with arrows
# mcomin     20/03/95   Fixed bug in configuration
# mcomin     31/07/95   Use args instead of config
# mcomin     24/10/95   Add possibility to re-size changing window dimensions
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
#            26/05/97   Add -wrap option
#
#************************************************************************
#   NAME
#     UifText - Class for Text Widgets
#   SYNOPSIS
#     UifText <widget-name>
#   DESCRIPTION
#     UifText defines the class for the VLT text widgets.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     scrollh - Horizontal scroll bar
#     scrollv - Vertical   scroll bar
#     uifText - Text widget
#   METHODS
#
#     General remarks :
#
#     - Positions in the text are specified by two numbers <line>.<char>
#       <line> starts with 1
#       <char> starts with 0
#
#       Position 2.5 identifies the 2-th line and the 6-th character.
#
#     - Keyword "end" identifies the last character in the text widget
#
#     Inherits methods from UifIntercativeWidget class and supports 
#     the following methods :
#
#     insertText <pos> <text>
#        Insert text at the specified position
#
#     deleteText <from> <to>
#        Deletes the specified characted range
#
#     getText <from> <to>
#        Returns the text in the specified range
#  
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget class and supports :
#
#     font    <font name>
#     height  <integer>     Determine the height of the widget expressed
#                           in number of rows.
#     width   <integer>     Determine the widget length (in characters)
#
#   SPECIFIC BINDINGS
#
#     <Delete>       Deletes character left to the insertion cursor.
#                    It also deletes selected part of the text.
#     <BackSpace>    Deletes character left to the insertion cursor
#                    It also deletes selected part of the text.
#     <Arrow-Left>   Moves the insertion cursor leftwards
#     <Arrow-Right>  Moves the insertion cursor rightwards
#     <Arrow-Up>     Moves the insertion cursor one line upwards
#     <Arrow-Down>   Moves the insertion cursor one line downwards
#     <B1-Motion>    Drag the mouse left-button to select part of the text.
#
#     <Button-2>     Done by clicking the mouse middle-button.
#                    The selected part of the text is inserted in the position
#                    where the cursor is when the user clicks on <Button-2>.
# 
#     <Control-d>    Deletes the character at the left of the cursor current
#                    position.
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     UifIntercativeWidget  UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#

option add *UifText.relief sunken widgetDefault
option add *UifText.borderwidth 2 widgetDefault

itk::usual UifText { }

class UifText {
    inherit UifInteractiveWidget

    constructor {args} {}
        
    public method insertText { pos text } { 
      return [$itk_component(uifText) insert $pos $text]
    }

    public method deleteText { from to } {
      return [$itk_component(uifText) delete $from $to]
    }

    public method getText { from to } { 
      return [$itk_component(uifText) get $from $to]
    }

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifText::constructor {args} { 

   itk_component add f1 {
      frame $itk_interior.f1 
   }

   itk_component add f2 {
      frame $itk_interior.f2 
   }

   pack $itk_component(f1)  -fill both -expand 1
   pack $itk_component(f2)  -fill x

   itk_component add uifText {
      text $itk_component(f1).uifText -setgrid false -wrap none -borderwidth 2 \
                                    -font "-Adobe-courier-medium-r-normal--14*"
   } { 
      keep -relief -width -height -font
      keep -background -foreground -wrap
   }

   itk_component add scrollv {
      scrollbar $itk_component(f1).scrollv -orient v -borderwidth 2 -width 10\
                -command [code $itk_component(uifText) yview] -relief sunken
   } {
      keep -activebackground
   }

   itk_component add scrollh {
      scrollbar $itk_component(f2).scrollh -orient h -borderwidth 2 -width 10\
                -command [code $itk_component(uifText) xview] -relief sunken
   } {
      keep -activebackground
   }

   pack $itk_component(uifText)  -side left -fill both -expand 1      
   pack $itk_component(scrollv)  -side left -fill y
   pack $itk_component(scrollh)  -fill x

   eval itk_initialize 

   $itk_component(uifText) configure -relief sunken  -borderwidth 1 \
                        -yscrollcommand [code $itk_component(scrollv) set] \
	                -xscrollcommand [code $itk_component(scrollh) set]

   eval configure -wrap none $args
}    








