#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifWidget.tcl,v 3.36 2003/01/20 10:33:51 vltsccm Exp $" 
#
# uifWidget.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# mcomin     07/11/94   Add documentation
# mcomin     24/10/95   Add treatment for global help for classes
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#
#************************************************************************
#   NAME
#     UifWidget - Parent class for VLT widgets.
#   SYNOPSIS
#    inherit UifWidget  
#   DESCRIPTION
#     UifWidget is used to inherit special features for the implementation
#     of the UIF Common Conventions.
#
#     The user creates a widget by defining an instance of the widget class,
#     for example :
#
#         UifOutput  .out -variable dummyVar -color Red -frame 1
#
#     Widget resources can be set using :
#
#        <name-of-instance> configure <resource-name> <value>
#
#     The current value of the widget resources is returned by :
#
#        <name-of-instance> cget <resource-name>
#
#   COMPONENTS
#
#   METHODS
#
#     The class supports the following methods :
#
#     queryState
#         Returns 1 if the widget is 'enabled' or 0 if 'disabled'.
#
#     getComponentPath  (OBSOLETE .. use tk statement 'component')
#         Returns the full name of a widget's component.
#
#         Mega-widgets are formed by a number of elementary 'components' and 
#         sometimes it is necessary to directly access one of them -e.g.- when
#         defining specific binding setup.
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     helparea  <name>     Name of the widget implementing the short help.
#                          The short-help functionality is also activated.
#
#     helptext  <string>   String describing the short help text associated
#                          to the widget.                            
#                          The short-help functionality is also activated.
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

class UifWidget {
    inherit itk::Widget

    constructor {args} {}
    destructor  { unsetData }

    itk_option define  -helparea     helparea     Helparea     ""
    itk_option define  -helptext     helptext     Helptext     ""
    itk_option define  -canvasAnchor canvasAnchor CanvasAnchor nw
    itk_option define  -qualityBg qualityBg QualityBg "Grey75"

    public method queryState  {}

    public method getComponentPath {name} {
       return $itk_component($name)
    }

    public method unsetData {} {
       uifUnsetData $wthis $recName $elemName
    }

    # Sets the record's name of the associated data structures

    public method setRecName {name} {
        set recName $name
    }

    public method addToDataList     {var}
    public method displayScanStatus {scanFlag { color Black }}

    private variable recName     "" ; # Name of the data structures 
                                      # associated to the widget
    private variable arrayName   ""
    private variable elemName    ""
    private variable origBgColor Black

    protected variable wthis  ""
    protected variable initialized 0

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifWidget::constructor { args } {
    global ::tcl_version
    eval itk_initialize $args
    set wthis  [namespace tail $this]
    set origBgColor [$this cget -background]
}

#---------------------------------------------------------------
# METHOD : displayScanStatus - 
#---------------------------------------------------------------

body UifWidget::displayScanStatus { scanFlag { color Black } } {
    if {$scanFlag == 0} { 
       set color $origBgColor 
    } else {
       set color $itk_option(-qualityBg)
    }
    configure -background $color
}

#---------------------------------------------------------------
# METHOD : addToDataList -
#---------------------------------------------------------------

body UifWidget::addToDataList { var } {
   if {![cequal $var ""]} {
      uifAddToDataList $wthis $var
   }
}

#---------------------------------------------------------------
# METHOD : queryState 
#---------------------------------------------------------------

body UifWidget::queryState {} {
   if {[$this cget -state] == "disabled"} {
      return 0
   } else {
      return 1   
   }
}

#---------------------------------------------------------------
# OPTION : -helptext
#---------------------------------------------------------------

configbody UifWidget::helptext {
   if { ![cequal $itk_option(-helparea) ""] && \
        ![cequal $itk_option(-helptext) ""]} {
      UifShortHelp::add $itk_option(-helparea) $this $itk_option(-helptext)
   }
}



